/* Generated automatically by the program `gencodes'
   from the machine description file `md'.  */

#ifndef GCC_INSN_CODES_H
#define GCC_INSN_CODES_H

enum insn_code {
  CODE_FOR_nothing = 0,
  CODE_FOR_x86_fnstsw_1 = 51,
  CODE_FOR_x86_sahf_1 = 52,
  CODE_FOR_insvhi_1 = 113,
  CODE_FOR_insvsi_1 = 114,
  CODE_FOR_insvdi_1 = 115,
  CODE_FOR_swapxf = 132,
  CODE_FOR_zero_extendqidi2 = 136,
  CODE_FOR_zero_extendhidi2 = 137,
  CODE_FOR_zero_extendqisi2_and = 138,
  CODE_FOR_zero_extendhisi2_and = 139,
  CODE_FOR_zero_extendqihi2_and = 142,
  CODE_FOR_extendsidi2_1 = 150,
  CODE_FOR_extendqidi2 = 151,
  CODE_FOR_extendhidi2 = 152,
  CODE_FOR_extendhisi2 = 153,
  CODE_FOR_extendqisi2 = 155,
  CODE_FOR_extendqihi2 = 157,
  CODE_FOR_truncxfsf2_i387_noop = 168,
  CODE_FOR_truncxfdf2_i387_noop = 169,
  CODE_FOR_fix_truncsfsi_sse = 174,
  CODE_FOR_fix_truncsfdi_sse = 175,
  CODE_FOR_fix_truncdfsi_sse = 176,
  CODE_FOR_fix_truncdfdi_sse = 177,
  CODE_FOR_fix_trunchi_fisttp_i387_1 = 178,
  CODE_FOR_fix_truncsi_fisttp_i387_1 = 179,
  CODE_FOR_fix_truncdi_fisttp_i387_1 = 180,
  CODE_FOR_fix_trunchi_i387_fisttp = 181,
  CODE_FOR_fix_truncsi_i387_fisttp = 182,
  CODE_FOR_fix_truncdi_i387_fisttp = 183,
  CODE_FOR_fix_trunchi_i387_fisttp_with_temp = 184,
  CODE_FOR_fix_truncsi_i387_fisttp_with_temp = 185,
  CODE_FOR_fix_truncdi_i387_fisttp_with_temp = 186,
  CODE_FOR_fix_truncdi_i387 = 190,
  CODE_FOR_fix_truncdi_i387_with_temp = 191,
  CODE_FOR_fix_trunchi_i387 = 192,
  CODE_FOR_fix_truncsi_i387 = 193,
  CODE_FOR_fix_trunchi_i387_with_temp = 194,
  CODE_FOR_fix_truncsi_i387_with_temp = 195,
  CODE_FOR_x86_fnstcw_1 = 196,
  CODE_FOR_x86_fldcw_1 = 197,
  CODE_FOR_floathisf2 = 198,
  CODE_FOR_floathidf2 = 199,
  CODE_FOR_floathixf2 = 200,
  CODE_FOR_floatsixf2 = 201,
  CODE_FOR_floatdixf2 = 202,
  CODE_FOR_floatdisf2_i387_with_xmm = 211,
  CODE_FOR_floatdidf2_i387_with_xmm = 212,
  CODE_FOR_floatdixf2_i387_with_xmm = 213,
  CODE_FOR_addsi_1_zext = 223,
  CODE_FOR_addqi_ext_1 = 245,
  CODE_FOR_addqi3_carry = 299,
  CODE_FOR_addhi3_carry = 300,
  CODE_FOR_addsi3_carry = 301,
  CODE_FOR_adddi3_carry = 302,
  CODE_FOR_addcarrysi = 309,
  CODE_FOR_addcarrydi = 310,
  CODE_FOR_subqi3_carry = 311,
  CODE_FOR_subhi3_carry = 312,
  CODE_FOR_subsi3_carry = 313,
  CODE_FOR_subdi3_carry = 314,
  CODE_FOR_subsi3_carry_ccc = 321,
  CODE_FOR_subdi3_carry_ccc = 322,
  CODE_FOR_subsi3_carry_ccgz = 325,
  CODE_FOR_subdi3_carry_ccgz = 326,
  CODE_FOR_subborrowsi = 327,
  CODE_FOR_subborrowdi = 328,
  CODE_FOR_divmodsi4_1 = 377,
  CODE_FOR_divmoddi4_1 = 378,
  CODE_FOR_divmodsi4_zext_1 = 379,
  CODE_FOR_divmodsi4_zext_2 = 380,
  CODE_FOR_divmodhiqi3 = 391,
  CODE_FOR_udivmodsi4_1 = 392,
  CODE_FOR_udivmoddi4_1 = 393,
  CODE_FOR_udivmodsi4_zext_1 = 394,
  CODE_FOR_udivmodsi4_zext_2 = 395,
  CODE_FOR_udivmodhiqi3 = 410,
  CODE_FOR_andqi_ext_1 = 436,
  CODE_FOR_copysignsf3_const = 523,
  CODE_FOR_copysigndf3_const = 524,
  CODE_FOR_copysigntf3_const = 525,
  CODE_FOR_copysignsf3_var = 526,
  CODE_FOR_copysigndf3_var = 527,
  CODE_FOR_copysigntf3_var = 528,
  CODE_FOR_x86_64_shld = 542,
  CODE_FOR_x86_shld = 543,
  CODE_FOR_x86_64_shrd = 578,
  CODE_FOR_x86_shrd = 579,
  CODE_FOR_ashrdi3_cvt = 580,
  CODE_FOR_ashrsi3_cvt = 582,
  CODE_FOR_ix86_rotldi3_doubleword = 627,
  CODE_FOR_ix86_rotlti3_doubleword = 628,
  CODE_FOR_ix86_rotrdi3_doubleword = 629,
  CODE_FOR_ix86_rotrti3_doubleword = 630,
  CODE_FOR_setcc_sf_sse = 680,
  CODE_FOR_setcc_df_sse = 681,
  CODE_FOR_jump = 683,
  CODE_FOR_blockage = 710,
  CODE_FOR_prologue_use = 712,
  CODE_FOR_simple_return_internal = 713,
  CODE_FOR_interrupt_return = 714,
  CODE_FOR_simple_return_internal_long = 715,
  CODE_FOR_simple_return_pop_internal = 716,
  CODE_FOR_simple_return_indirect_internal = 717,
  CODE_FOR_nop = 718,
  CODE_FOR_nops = 719,
  CODE_FOR_pad = 720,
  CODE_FOR_set_got_rex64 = 723,
  CODE_FOR_set_rip_rex64 = 724,
  CODE_FOR_set_got_offset_rex64 = 725,
  CODE_FOR_eh_return_internal = 726,
  CODE_FOR_leave = 727,
  CODE_FOR_leave_rex64 = 728,
  CODE_FOR_split_stack_return = 729,
  CODE_FOR_ffssi2_no_cmove = 730,
  CODE_FOR_ctzsi2 = 737,
  CODE_FOR_ctzdi2 = 738,
  CODE_FOR_bsr_rex64 = 741,
  CODE_FOR_bsr = 742,
  CODE_FOR_clzsi2_lzcnt = 744,
  CODE_FOR_clzdi2_lzcnt = 745,
  CODE_FOR_tzcnt_si = 748,
  CODE_FOR_lzcnt_si = 749,
  CODE_FOR_tzcnt_di = 750,
  CODE_FOR_lzcnt_di = 751,
  CODE_FOR_tzcnt_hi = 756,
  CODE_FOR_lzcnt_hi = 757,
  CODE_FOR_bmi_bextr_si = 758,
  CODE_FOR_bmi_bextr_di = 759,
  CODE_FOR_bmi2_pdep_si3 = 778,
  CODE_FOR_bmi2_pdep_di3 = 779,
  CODE_FOR_bmi2_pext_si3 = 780,
  CODE_FOR_bmi2_pext_di3 = 781,
  CODE_FOR_tbm_bextri_si = 782,
  CODE_FOR_tbm_bextri_di = 783,
  CODE_FOR_popcountsi2 = 802,
  CODE_FOR_popcountdi2 = 803,
  CODE_FOR_popcounthi2 = 807,
  CODE_FOR_bswaphi_lowpart = 813,
  CODE_FOR_paritydi2_cmp = 814,
  CODE_FOR_paritysi2_cmp = 815,
   CODE_FOR_tls_initial_exec_64_sun = CODE_FOR_nothing,
  CODE_FOR_truncxfsf2_i387_noop_unspec = 866,
  CODE_FOR_truncxfdf2_i387_noop_unspec = 867,
  CODE_FOR_sqrtxf2 = 868,
  CODE_FOR_sqrt_extendsfxf2_i387 = 869,
  CODE_FOR_sqrt_extenddfxf2_i387 = 870,
  CODE_FOR_fpremxf4_i387 = 874,
  CODE_FOR_fprem1xf4_i387 = 875,
  CODE_FOR_sincosxf3 = 882,
  CODE_FOR_sincos_extendsfxf3_i387 = 883,
  CODE_FOR_sincos_extenddfxf3_i387 = 884,
  CODE_FOR_fptanxf4_i387 = 885,
  CODE_FOR_fptan_extendsfxf4_i387 = 886,
  CODE_FOR_fptan_extenddfxf4_i387 = 887,
  CODE_FOR_fpatan_extendsfxf3_i387 = 889,
  CODE_FOR_fpatan_extenddfxf3_i387 = 890,
  CODE_FOR_fyl2xxf3_i387 = 891,
  CODE_FOR_fyl2x_extendsfxf3_i387 = 892,
  CODE_FOR_fyl2x_extenddfxf3_i387 = 893,
  CODE_FOR_fyl2xp1xf3_i387 = 894,
  CODE_FOR_fyl2xp1_extendsfxf3_i387 = 895,
  CODE_FOR_fyl2xp1_extenddfxf3_i387 = 896,
  CODE_FOR_fxtractxf3_i387 = 897,
  CODE_FOR_fxtract_extendsfxf3_i387 = 898,
  CODE_FOR_fxtract_extenddfxf3_i387 = 899,
  CODE_FOR_fscalexf4_i387 = 901,
  CODE_FOR_sse4_1_roundsf2 = 902,
  CODE_FOR_sse4_1_rounddf2 = 903,
  CODE_FOR_rintxf2 = 904,
  CODE_FOR_rintsf2_frndint = 905,
  CODE_FOR_rintdf2_frndint = 906,
  CODE_FOR_fistdi2 = 908,
  CODE_FOR_fistdi2_with_temp = 909,
  CODE_FOR_fisthi2 = 912,
  CODE_FOR_fistsi2 = 913,
  CODE_FOR_fisthi2_with_temp = 914,
  CODE_FOR_fistsi2_with_temp = 915,
  CODE_FOR_frndintsf2_floor = 916,
  CODE_FOR_frndintsf2_ceil = 917,
  CODE_FOR_frndintsf2_trunc = 918,
  CODE_FOR_frndintdf2_floor = 919,
  CODE_FOR_frndintdf2_ceil = 920,
  CODE_FOR_frndintdf2_trunc = 921,
  CODE_FOR_frndintxf2_floor = 922,
  CODE_FOR_frndintxf2_ceil = 923,
  CODE_FOR_frndintxf2_trunc = 924,
  CODE_FOR_frndintsf2_floor_i387 = 925,
  CODE_FOR_frndintsf2_ceil_i387 = 926,
  CODE_FOR_frndintsf2_trunc_i387 = 927,
  CODE_FOR_frndintdf2_floor_i387 = 928,
  CODE_FOR_frndintdf2_ceil_i387 = 929,
  CODE_FOR_frndintdf2_trunc_i387 = 930,
  CODE_FOR_frndintxf2_floor_i387 = 931,
  CODE_FOR_frndintxf2_ceil_i387 = 932,
  CODE_FOR_frndintxf2_trunc_i387 = 933,
  CODE_FOR_frndintxf2_mask_pm = 934,
  CODE_FOR_frndintxf2_mask_pm_i387 = 935,
  CODE_FOR_fistdi2_floor = 942,
  CODE_FOR_fistdi2_ceil = 943,
  CODE_FOR_fistdi2_floor_with_temp = 944,
  CODE_FOR_fistdi2_ceil_with_temp = 945,
  CODE_FOR_fisthi2_floor = 946,
  CODE_FOR_fisthi2_ceil = 947,
  CODE_FOR_fistsi2_floor = 948,
  CODE_FOR_fistsi2_ceil = 949,
  CODE_FOR_fisthi2_floor_with_temp = 950,
  CODE_FOR_fisthi2_ceil_with_temp = 951,
  CODE_FOR_fistsi2_floor_with_temp = 952,
  CODE_FOR_fistsi2_ceil_with_temp = 953,
  CODE_FOR_fxamsf2_i387 = 954,
  CODE_FOR_fxamdf2_i387 = 955,
  CODE_FOR_fxamxf2_i387 = 956,
  CODE_FOR_fxamsf2_i387_with_temp = 957,
  CODE_FOR_fxamdf2_i387_with_temp = 958,
  CODE_FOR_movmsk_df = 959,
  CODE_FOR_cld = 960,
  CODE_FOR_smaxsf3 = 1011,
  CODE_FOR_sminsf3 = 1012,
  CODE_FOR_smaxdf3 = 1013,
  CODE_FOR_smindf3 = 1014,
  CODE_FOR_pro_epilogue_adjust_stack_si_add = 1019,
  CODE_FOR_pro_epilogue_adjust_stack_di_add = 1020,
  CODE_FOR_pro_epilogue_adjust_stack_si_sub = 1021,
  CODE_FOR_pro_epilogue_adjust_stack_di_sub = 1022,
  CODE_FOR_allocate_stack_worker_probe_si = 1023,
  CODE_FOR_allocate_stack_worker_probe_di = 1024,
  CODE_FOR_probe_stack_si = 1025,
  CODE_FOR_probe_stack_di = 1026,
  CODE_FOR_adjust_stack_and_probesi = 1027,
  CODE_FOR_adjust_stack_and_probedi = 1028,
  CODE_FOR_probe_stack_rangesi = 1029,
  CODE_FOR_probe_stack_rangedi = 1030,
  CODE_FOR_trap = 1031,
  CODE_FOR_ud2 = 1032,
  CODE_FOR_stack_protect_set_si = 1036,
  CODE_FOR_stack_protect_set_di = 1037,
  CODE_FOR_stack_protect_test_si = 1038,
  CODE_FOR_stack_protect_test_di = 1039,
  CODE_FOR_sse4_2_crc32qi = 1040,
  CODE_FOR_sse4_2_crc32hi = 1041,
  CODE_FOR_sse4_2_crc32si = 1042,
  CODE_FOR_sse4_2_crc32di = 1043,
  CODE_FOR_rdpmc = 1044,
  CODE_FOR_rdpmc_rex64 = 1045,
  CODE_FOR_rdtsc = 1046,
  CODE_FOR_rdtsc_rex64 = 1047,
  CODE_FOR_rdtscp = 1048,
  CODE_FOR_rdtscp_rex64 = 1049,
  CODE_FOR_fxsave = 1050,
  CODE_FOR_fxsave64 = 1051,
  CODE_FOR_fxrstor = 1052,
  CODE_FOR_fxrstor64 = 1053,
  CODE_FOR_xsave = 1054,
  CODE_FOR_xsaveopt = 1055,
  CODE_FOR_xsavec = 1056,
  CODE_FOR_xsaves = 1057,
  CODE_FOR_xsave_rex64 = 1058,
  CODE_FOR_xsaveopt_rex64 = 1059,
  CODE_FOR_xsavec_rex64 = 1060,
  CODE_FOR_xsaves_rex64 = 1061,
  CODE_FOR_xsave64 = 1062,
  CODE_FOR_xsaveopt64 = 1063,
  CODE_FOR_xsavec64 = 1064,
  CODE_FOR_xsaves64 = 1065,
  CODE_FOR_xrstor = 1066,
  CODE_FOR_xrstors = 1067,
  CODE_FOR_xrstor_rex64 = 1068,
  CODE_FOR_xrstors_rex64 = 1069,
  CODE_FOR_xrstor64 = 1070,
  CODE_FOR_xrstors64 = 1071,
  CODE_FOR_xsetbv = 1072,
  CODE_FOR_xsetbv_rex64 = 1073,
  CODE_FOR_xgetbv = 1074,
  CODE_FOR_xgetbv_rex64 = 1075,
  CODE_FOR_fnstenv = 1076,
  CODE_FOR_fldenv = 1077,
  CODE_FOR_fnstsw = 1078,
  CODE_FOR_fnclex = 1079,
  CODE_FOR_lwp_slwpcbsi = 1082,
  CODE_FOR_lwp_slwpcbdi = 1083,
  CODE_FOR_rdfsbasesi = 1088,
  CODE_FOR_rdgsbasesi = 1089,
  CODE_FOR_rdfsbasedi = 1090,
  CODE_FOR_rdgsbasedi = 1091,
  CODE_FOR_wrfsbasesi = 1092,
  CODE_FOR_wrgsbasesi = 1093,
  CODE_FOR_wrfsbasedi = 1094,
  CODE_FOR_wrgsbasedi = 1095,
  CODE_FOR_rdrandhi_1 = 1096,
  CODE_FOR_rdrandsi_1 = 1097,
  CODE_FOR_rdranddi_1 = 1098,
  CODE_FOR_rdseedhi_1 = 1099,
  CODE_FOR_rdseedsi_1 = 1100,
  CODE_FOR_rdseeddi_1 = 1101,
  CODE_FOR_rdsspsi = 1103,
  CODE_FOR_rdsspdi = 1104,
  CODE_FOR_incsspsi = 1105,
  CODE_FOR_incsspdi = 1106,
  CODE_FOR_saveprevssp = 1107,
  CODE_FOR_rstorssp = 1108,
  CODE_FOR_wrsssi = 1109,
  CODE_FOR_wrssdi = 1110,
  CODE_FOR_wrusssi = 1111,
  CODE_FOR_wrussdi = 1112,
  CODE_FOR_setssbsy = 1113,
  CODE_FOR_clrssbsy = 1114,
  CODE_FOR_nop_endbr = 1115,
  CODE_FOR_xbegin_1 = 1116,
  CODE_FOR_xend = 1117,
  CODE_FOR_xabort = 1118,
  CODE_FOR_xtest_1 = 1119,
  CODE_FOR_clwb = 1120,
  CODE_FOR_clflushopt = 1121,
  CODE_FOR_mwaitx = 1122,
  CODE_FOR_monitorx_si = 1123,
  CODE_FOR_monitorx_di = 1124,
  CODE_FOR_clzero_si = 1125,
  CODE_FOR_clzero_di = 1126,
  CODE_FOR_move_size_reloc_si = 1141,
  CODE_FOR_move_size_reloc_di = 1142,
  CODE_FOR_rdpid = 1145,
  CODE_FOR_rdpid_rex64 = 1146,
  CODE_FOR_wbinvd = 1147,
  CODE_FOR_wbnoinvd = 1148,
  CODE_FOR_movdirisi = 1149,
  CODE_FOR_movdiridi = 1150,
  CODE_FOR_movdir64b_si = 1151,
  CODE_FOR_movdir64b_di = 1152,
  CODE_FOR_sse_movntq = 1158,
  CODE_FOR_mmx_ieee_maxv2sf3 = 1164,
  CODE_FOR_mmx_ieee_minv2sf3 = 1165,
  CODE_FOR_mmx_rcpv2sf2 = 1166,
  CODE_FOR_mmx_rcpit1v2sf3 = 1167,
  CODE_FOR_mmx_rcpit2v2sf3 = 1168,
  CODE_FOR_mmx_rsqrtv2sf2 = 1169,
  CODE_FOR_mmx_rsqit1v2sf3 = 1170,
  CODE_FOR_mmx_haddv2sf3 = 1171,
  CODE_FOR_mmx_hsubv2sf3 = 1172,
  CODE_FOR_mmx_addsubv2sf3 = 1173,
  CODE_FOR_mmx_gtv2sf3 = 1175,
  CODE_FOR_mmx_gev2sf3 = 1176,
  CODE_FOR_mmx_pf2id = 1177,
  CODE_FOR_mmx_pf2iw = 1178,
  CODE_FOR_mmx_pi2fw = 1179,
  CODE_FOR_mmx_floatv2si2 = 1180,
  CODE_FOR_mmx_pswapdv2sf2 = 1181,
  CODE_FOR_mmx_ashrv4hi3 = 1212,
  CODE_FOR_mmx_ashrv2si3 = 1213,
  CODE_FOR_mmx_ashlv4hi3 = 1214,
  CODE_FOR_mmx_lshrv4hi3 = 1215,
  CODE_FOR_mmx_ashlv2si3 = 1216,
  CODE_FOR_mmx_lshrv2si3 = 1217,
  CODE_FOR_mmx_ashlv1di3 = 1218,
  CODE_FOR_mmx_lshrv1di3 = 1219,
  CODE_FOR_mmx_gtv8qi3 = 1223,
  CODE_FOR_mmx_gtv4hi3 = 1224,
  CODE_FOR_mmx_gtv2si3 = 1225,
  CODE_FOR_mmx_andnotv8qi3 = 1226,
  CODE_FOR_mmx_andnotv4hi3 = 1227,
  CODE_FOR_mmx_andnotv2si3 = 1228,
  CODE_FOR_mmx_packsswb = 1238,
  CODE_FOR_mmx_packssdw = 1239,
  CODE_FOR_mmx_packuswb = 1240,
  CODE_FOR_mmx_punpckhbw = 1241,
  CODE_FOR_mmx_punpcklbw = 1242,
  CODE_FOR_mmx_punpckhwd = 1243,
  CODE_FOR_mmx_punpcklwd = 1244,
  CODE_FOR_mmx_punpckhdq = 1245,
  CODE_FOR_mmx_punpckldq = 1246,
  CODE_FOR_mmx_pextrw = 1248,
  CODE_FOR_mmx_pshufw_1 = 1249,
  CODE_FOR_mmx_pswapdv2si2 = 1250,
  CODE_FOR_mmx_psadbw = 1259,
  CODE_FOR_mmx_pmovmskb = 1260,
  CODE_FOR_movv64qi_internal = 1265,
  CODE_FOR_movv32qi_internal = 1266,
  CODE_FOR_movv16qi_internal = 1267,
  CODE_FOR_movv32hi_internal = 1268,
  CODE_FOR_movv16hi_internal = 1269,
  CODE_FOR_movv8hi_internal = 1270,
  CODE_FOR_movv16si_internal = 1271,
  CODE_FOR_movv8si_internal = 1272,
  CODE_FOR_movv4si_internal = 1273,
  CODE_FOR_movv8di_internal = 1274,
  CODE_FOR_movv4di_internal = 1275,
  CODE_FOR_movv2di_internal = 1276,
  CODE_FOR_movv4ti_internal = 1277,
  CODE_FOR_movv2ti_internal = 1278,
  CODE_FOR_movv1ti_internal = 1279,
  CODE_FOR_movv16sf_internal = 1280,
  CODE_FOR_movv8sf_internal = 1281,
  CODE_FOR_movv4sf_internal = 1282,
  CODE_FOR_movv8df_internal = 1283,
  CODE_FOR_movv4df_internal = 1284,
  CODE_FOR_movv2df_internal = 1285,
  CODE_FOR_avx512f_loadv16si_mask = 1286,
  CODE_FOR_avx512vl_loadv8si_mask = 1287,
  CODE_FOR_avx512vl_loadv4si_mask = 1288,
  CODE_FOR_avx512f_loadv8di_mask = 1289,
  CODE_FOR_avx512vl_loadv4di_mask = 1290,
  CODE_FOR_avx512vl_loadv2di_mask = 1291,
  CODE_FOR_avx512f_loadv16sf_mask = 1292,
  CODE_FOR_avx512vl_loadv8sf_mask = 1293,
  CODE_FOR_avx512vl_loadv4sf_mask = 1294,
  CODE_FOR_avx512f_loadv8df_mask = 1295,
  CODE_FOR_avx512vl_loadv4df_mask = 1296,
  CODE_FOR_avx512vl_loadv2df_mask = 1297,
  CODE_FOR_avx512bw_loadv64qi_mask = 1298,
  CODE_FOR_avx512vl_loadv16qi_mask = 1299,
  CODE_FOR_avx512vl_loadv32qi_mask = 1300,
  CODE_FOR_avx512bw_loadv32hi_mask = 1301,
  CODE_FOR_avx512vl_loadv16hi_mask = 1302,
  CODE_FOR_avx512vl_loadv8hi_mask = 1303,
  CODE_FOR_avx512f_blendmv16si = 1304,
  CODE_FOR_avx512vl_blendmv8si = 1305,
  CODE_FOR_avx512vl_blendmv4si = 1306,
  CODE_FOR_avx512f_blendmv8di = 1307,
  CODE_FOR_avx512vl_blendmv4di = 1308,
  CODE_FOR_avx512vl_blendmv2di = 1309,
  CODE_FOR_avx512f_blendmv16sf = 1310,
  CODE_FOR_avx512vl_blendmv8sf = 1311,
  CODE_FOR_avx512vl_blendmv4sf = 1312,
  CODE_FOR_avx512f_blendmv8df = 1313,
  CODE_FOR_avx512vl_blendmv4df = 1314,
  CODE_FOR_avx512vl_blendmv2df = 1315,
  CODE_FOR_avx512bw_blendmv64qi = 1316,
  CODE_FOR_avx512vl_blendmv16qi = 1317,
  CODE_FOR_avx512vl_blendmv32qi = 1318,
  CODE_FOR_avx512bw_blendmv32hi = 1319,
  CODE_FOR_avx512vl_blendmv16hi = 1320,
  CODE_FOR_avx512vl_blendmv8hi = 1321,
  CODE_FOR_avx512f_storev16si_mask = 1322,
  CODE_FOR_avx512vl_storev8si_mask = 1323,
  CODE_FOR_avx512vl_storev4si_mask = 1324,
  CODE_FOR_avx512f_storev8di_mask = 1325,
  CODE_FOR_avx512vl_storev4di_mask = 1326,
  CODE_FOR_avx512vl_storev2di_mask = 1327,
  CODE_FOR_avx512f_storev16sf_mask = 1328,
  CODE_FOR_avx512vl_storev8sf_mask = 1329,
  CODE_FOR_avx512vl_storev4sf_mask = 1330,
  CODE_FOR_avx512f_storev8df_mask = 1331,
  CODE_FOR_avx512vl_storev4df_mask = 1332,
  CODE_FOR_avx512vl_storev2df_mask = 1333,
  CODE_FOR_avx512bw_storev64qi_mask = 1334,
  CODE_FOR_avx512vl_storev16qi_mask = 1335,
  CODE_FOR_avx512vl_storev32qi_mask = 1336,
  CODE_FOR_avx512bw_storev32hi_mask = 1337,
  CODE_FOR_avx512vl_storev16hi_mask = 1338,
  CODE_FOR_avx512vl_storev8hi_mask = 1339,
  CODE_FOR_sse2_movq128 = 1340,
  CODE_FOR_movdi_to_sse = 1341,
  CODE_FOR_avx_lddqu256 = 1342,
  CODE_FOR_sse3_lddqu = 1343,
  CODE_FOR_sse2_movntisi = 1344,
  CODE_FOR_sse2_movntidi = 1345,
  CODE_FOR_avx512f_movntv16sf = 1346,
  CODE_FOR_avx_movntv8sf = 1347,
  CODE_FOR_sse_movntv4sf = 1348,
  CODE_FOR_avx512f_movntv8df = 1349,
  CODE_FOR_avx_movntv4df = 1350,
  CODE_FOR_sse2_movntv2df = 1351,
  CODE_FOR_avx512f_movntv8di = 1352,
  CODE_FOR_avx_movntv4di = 1353,
  CODE_FOR_sse2_movntv2di = 1354,
  CODE_FOR_kandqi = 1355,
  CODE_FOR_kiorqi = 1356,
  CODE_FOR_kxorqi = 1357,
  CODE_FOR_kandhi = 1358,
  CODE_FOR_kiorhi = 1359,
  CODE_FOR_kxorhi = 1360,
  CODE_FOR_kandsi = 1361,
  CODE_FOR_kiorsi = 1362,
  CODE_FOR_kxorsi = 1363,
  CODE_FOR_kanddi = 1364,
  CODE_FOR_kiordi = 1365,
  CODE_FOR_kxordi = 1366,
  CODE_FOR_kandnqi = 1367,
  CODE_FOR_kandnhi = 1368,
  CODE_FOR_kandnsi = 1369,
  CODE_FOR_kandndi = 1370,
  CODE_FOR_kxnorqi = 1371,
  CODE_FOR_kxnorhi = 1372,
  CODE_FOR_kxnorsi = 1373,
  CODE_FOR_kxnordi = 1374,
  CODE_FOR_knotqi = 1375,
  CODE_FOR_knothi = 1376,
  CODE_FOR_knotsi = 1377,
  CODE_FOR_knotdi = 1378,
  CODE_FOR_kaddqi = 1379,
  CODE_FOR_kaddhi = 1380,
  CODE_FOR_kaddsi = 1381,
  CODE_FOR_kadddi = 1382,
  CODE_FOR_kashiftqi = 1383,
  CODE_FOR_klshiftrtqi = 1384,
  CODE_FOR_kashifthi = 1385,
  CODE_FOR_klshiftrthi = 1386,
  CODE_FOR_kashiftsi = 1387,
  CODE_FOR_klshiftrtsi = 1388,
  CODE_FOR_kashiftdi = 1389,
  CODE_FOR_klshiftrtdi = 1390,
  CODE_FOR_ktestqi = 1391,
  CODE_FOR_ktesthi = 1392,
  CODE_FOR_ktestsi = 1393,
  CODE_FOR_ktestdi = 1394,
  CODE_FOR_kortestqi = 1395,
  CODE_FOR_kortesthi = 1396,
  CODE_FOR_kortestsi = 1397,
  CODE_FOR_kortestdi = 1398,
  CODE_FOR_kunpckhi = 1399,
  CODE_FOR_kunpcksi = 1400,
  CODE_FOR_kunpckdi = 1401,
  CODE_FOR_sse_vmaddv4sf3 = 1456,
  CODE_FOR_sse_vmaddv4sf3_round = 1457,
  CODE_FOR_sse_vmaddv4sf3_mask = 1458,
  CODE_FOR_sse_vmaddv4sf3_mask_round = 1459,
  CODE_FOR_sse_vmsubv4sf3 = 1460,
  CODE_FOR_sse_vmsubv4sf3_round = 1461,
  CODE_FOR_sse_vmsubv4sf3_mask = 1462,
  CODE_FOR_sse_vmsubv4sf3_mask_round = 1463,
  CODE_FOR_sse2_vmaddv2df3 = 1464,
  CODE_FOR_sse2_vmaddv2df3_round = 1465,
  CODE_FOR_sse2_vmaddv2df3_mask = 1466,
  CODE_FOR_sse2_vmaddv2df3_mask_round = 1467,
  CODE_FOR_sse2_vmsubv2df3 = 1468,
  CODE_FOR_sse2_vmsubv2df3_round = 1469,
  CODE_FOR_sse2_vmsubv2df3_mask = 1470,
  CODE_FOR_sse2_vmsubv2df3_mask_round = 1471,
  CODE_FOR_sse_vmmulv4sf3 = 1488,
  CODE_FOR_sse_vmmulv4sf3_round = 1489,
  CODE_FOR_sse_vmmulv4sf3_mask = 1490,
  CODE_FOR_sse_vmmulv4sf3_mask_round = 1491,
  CODE_FOR_sse_vmdivv4sf3 = 1492,
  CODE_FOR_sse_vmdivv4sf3_round = 1493,
  CODE_FOR_sse_vmdivv4sf3_mask = 1494,
  CODE_FOR_sse_vmdivv4sf3_mask_round = 1495,
  CODE_FOR_sse2_vmmulv2df3 = 1496,
  CODE_FOR_sse2_vmmulv2df3_round = 1497,
  CODE_FOR_sse2_vmmulv2df3_mask = 1498,
  CODE_FOR_sse2_vmmulv2df3_mask_round = 1499,
  CODE_FOR_sse2_vmdivv2df3 = 1500,
  CODE_FOR_sse2_vmdivv2df3_round = 1501,
  CODE_FOR_sse2_vmdivv2df3_mask = 1502,
  CODE_FOR_sse2_vmdivv2df3_mask_round = 1503,
  CODE_FOR_avx512f_divv16sf3 = 1504,
  CODE_FOR_avx512f_divv16sf3_round = 1505,
  CODE_FOR_avx512f_divv16sf3_mask = 1506,
  CODE_FOR_avx512f_divv16sf3_mask_round = 1507,
  CODE_FOR_avx_divv8sf3 = 1508,
   CODE_FOR_avx_divv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx_divv8sf3_mask = 1509,
   CODE_FOR_avx_divv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_divv4sf3 = 1510,
   CODE_FOR_sse_divv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_sse_divv4sf3_mask = 1511,
   CODE_FOR_sse_divv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_divv8df3 = 1512,
  CODE_FOR_avx512f_divv8df3_round = 1513,
  CODE_FOR_avx512f_divv8df3_mask = 1514,
  CODE_FOR_avx512f_divv8df3_mask_round = 1515,
  CODE_FOR_avx_divv4df3 = 1516,
   CODE_FOR_avx_divv4df3_round = CODE_FOR_nothing,
  CODE_FOR_avx_divv4df3_mask = 1517,
   CODE_FOR_avx_divv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse2_divv2df3 = 1518,
   CODE_FOR_sse2_divv2df3_round = CODE_FOR_nothing,
  CODE_FOR_sse2_divv2df3_mask = 1519,
   CODE_FOR_sse2_divv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx_rcpv8sf2 = 1520,
  CODE_FOR_sse_rcpv4sf2 = 1521,
  CODE_FOR_sse_vmrcpv4sf2 = 1522,
  CODE_FOR_rcp14v16sf_mask = 1524,
  CODE_FOR_rcp14v8sf_mask = 1526,
  CODE_FOR_rcp14v4sf_mask = 1528,
  CODE_FOR_rcp14v8df_mask = 1530,
  CODE_FOR_rcp14v4df_mask = 1532,
  CODE_FOR_rcp14v2df_mask = 1534,
  CODE_FOR_srcp14v4sf = 1535,
  CODE_FOR_srcp14v2df = 1536,
  CODE_FOR_srcp14v4sf_mask = 1537,
  CODE_FOR_srcp14v2df_mask = 1538,
  CODE_FOR_avx512f_sqrtv16sf2 = 1539,
  CODE_FOR_avx512f_sqrtv16sf2_round = 1540,
  CODE_FOR_avx512f_sqrtv16sf2_mask = 1541,
  CODE_FOR_avx512f_sqrtv16sf2_mask_round = 1542,
  CODE_FOR_avx_sqrtv8sf2 = 1543,
   CODE_FOR_avx_sqrtv8sf2_round = CODE_FOR_nothing,
  CODE_FOR_avx_sqrtv8sf2_mask = 1544,
   CODE_FOR_avx_sqrtv8sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_sqrtv4sf2 = 1545,
   CODE_FOR_sse_sqrtv4sf2_round = CODE_FOR_nothing,
  CODE_FOR_sse_sqrtv4sf2_mask = 1546,
   CODE_FOR_sse_sqrtv4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_sqrtv8df2 = 1547,
  CODE_FOR_avx512f_sqrtv8df2_round = 1548,
  CODE_FOR_avx512f_sqrtv8df2_mask = 1549,
  CODE_FOR_avx512f_sqrtv8df2_mask_round = 1550,
  CODE_FOR_avx_sqrtv4df2 = 1551,
   CODE_FOR_avx_sqrtv4df2_round = CODE_FOR_nothing,
  CODE_FOR_avx_sqrtv4df2_mask = 1552,
   CODE_FOR_avx_sqrtv4df2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse2_sqrtv2df2 = 1553,
   CODE_FOR_sse2_sqrtv2df2_round = CODE_FOR_nothing,
  CODE_FOR_sse2_sqrtv2df2_mask = 1554,
   CODE_FOR_sse2_sqrtv2df2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_vmsqrtv4sf2 = 1555,
  CODE_FOR_sse_vmsqrtv4sf2_round = 1556,
  CODE_FOR_sse_vmsqrtv4sf2_mask = 1557,
  CODE_FOR_sse_vmsqrtv4sf2_mask_round = 1558,
  CODE_FOR_sse2_vmsqrtv2df2 = 1559,
  CODE_FOR_sse2_vmsqrtv2df2_round = 1560,
  CODE_FOR_sse2_vmsqrtv2df2_mask = 1561,
  CODE_FOR_sse2_vmsqrtv2df2_mask_round = 1562,
  CODE_FOR_avx_rsqrtv8sf2 = 1563,
  CODE_FOR_sse_rsqrtv4sf2 = 1564,
  CODE_FOR_rsqrt14v16sf_mask = 1566,
  CODE_FOR_rsqrt14v8sf_mask = 1568,
  CODE_FOR_rsqrt14v4sf_mask = 1570,
  CODE_FOR_rsqrt14v8df_mask = 1572,
  CODE_FOR_rsqrt14v4df_mask = 1574,
  CODE_FOR_rsqrt14v2df_mask = 1576,
  CODE_FOR_rsqrt14v4sf = 1577,
  CODE_FOR_rsqrt14v2df = 1578,
  CODE_FOR_rsqrt14_v4sf_mask = 1579,
  CODE_FOR_rsqrt14_v2df_mask = 1580,
  CODE_FOR_sse_vmrsqrtv4sf2 = 1581,
  CODE_FOR_ieee_maxv16sf3 = 1614,
  CODE_FOR_ieee_maxv16sf3_round = 1615,
  CODE_FOR_ieee_maxv16sf3_mask = 1616,
  CODE_FOR_ieee_maxv16sf3_mask_round = 1617,
  CODE_FOR_ieee_minv16sf3 = 1618,
  CODE_FOR_ieee_minv16sf3_round = 1619,
  CODE_FOR_ieee_minv16sf3_mask = 1620,
  CODE_FOR_ieee_minv16sf3_mask_round = 1621,
  CODE_FOR_ieee_maxv8sf3 = 1622,
   CODE_FOR_ieee_maxv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8sf3_mask = 1623,
   CODE_FOR_ieee_maxv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8sf3 = 1624,
   CODE_FOR_ieee_minv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8sf3_mask = 1625,
   CODE_FOR_ieee_minv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4sf3 = 1626,
   CODE_FOR_ieee_maxv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4sf3_mask = 1627,
   CODE_FOR_ieee_maxv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4sf3 = 1628,
   CODE_FOR_ieee_minv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4sf3_mask = 1629,
   CODE_FOR_ieee_minv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8df3 = 1630,
  CODE_FOR_ieee_maxv8df3_round = 1631,
  CODE_FOR_ieee_maxv8df3_mask = 1632,
  CODE_FOR_ieee_maxv8df3_mask_round = 1633,
  CODE_FOR_ieee_minv8df3 = 1634,
  CODE_FOR_ieee_minv8df3_round = 1635,
  CODE_FOR_ieee_minv8df3_mask = 1636,
  CODE_FOR_ieee_minv8df3_mask_round = 1637,
  CODE_FOR_ieee_maxv4df3 = 1638,
   CODE_FOR_ieee_maxv4df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4df3_mask = 1639,
   CODE_FOR_ieee_maxv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4df3 = 1640,
   CODE_FOR_ieee_minv4df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4df3_mask = 1641,
   CODE_FOR_ieee_minv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv2df3 = 1642,
   CODE_FOR_ieee_maxv2df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv2df3_mask = 1643,
   CODE_FOR_ieee_maxv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv2df3 = 1644,
   CODE_FOR_ieee_minv2df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv2df3_mask = 1645,
   CODE_FOR_ieee_minv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_vmsmaxv4sf3 = 1646,
  CODE_FOR_sse_vmsmaxv4sf3_mask = 1647,
  CODE_FOR_sse_vmsmaxv4sf3_round = 1648,
  CODE_FOR_sse_vmsmaxv4sf3_mask_round = 1649,
  CODE_FOR_sse_vmsminv4sf3 = 1650,
  CODE_FOR_sse_vmsminv4sf3_mask = 1651,
  CODE_FOR_sse_vmsminv4sf3_round = 1652,
  CODE_FOR_sse_vmsminv4sf3_mask_round = 1653,
  CODE_FOR_sse2_vmsmaxv2df3 = 1654,
  CODE_FOR_sse2_vmsmaxv2df3_mask = 1655,
  CODE_FOR_sse2_vmsmaxv2df3_round = 1656,
  CODE_FOR_sse2_vmsmaxv2df3_mask_round = 1657,
  CODE_FOR_sse2_vmsminv2df3 = 1658,
  CODE_FOR_sse2_vmsminv2df3_mask = 1659,
  CODE_FOR_sse2_vmsminv2df3_round = 1660,
  CODE_FOR_sse2_vmsminv2df3_mask_round = 1661,
  CODE_FOR_avx_addsubv4df3 = 1662,
  CODE_FOR_sse3_addsubv2df3 = 1663,
  CODE_FOR_avx_addsubv8sf3 = 1664,
  CODE_FOR_sse3_addsubv4sf3 = 1665,
  CODE_FOR_avx_haddv4df3 = 1666,
  CODE_FOR_avx_hsubv4df3 = 1667,
  CODE_FOR_sse3_hsubv2df3 = 1669,
  CODE_FOR_avx_haddv8sf3 = 1672,
  CODE_FOR_avx_hsubv8sf3 = 1673,
  CODE_FOR_sse3_haddv4sf3 = 1674,
  CODE_FOR_sse3_hsubv4sf3 = 1675,
  CODE_FOR_reducepv16sf_mask = 1677,
  CODE_FOR_reducepv8sf_mask = 1679,
  CODE_FOR_reducepv4sf_mask = 1681,
  CODE_FOR_reducepv8df_mask = 1683,
  CODE_FOR_reducepv4df_mask = 1685,
  CODE_FOR_reducepv2df_mask = 1687,
  CODE_FOR_reducesv4sf = 1688,
  CODE_FOR_reducesv4sf_mask = 1689,
  CODE_FOR_reducesv2df = 1690,
  CODE_FOR_reducesv2df_mask = 1691,
  CODE_FOR_avx_cmpv8sf3 = 1692,
  CODE_FOR_avx_cmpv4sf3 = 1693,
  CODE_FOR_avx_cmpv4df3 = 1694,
  CODE_FOR_avx_cmpv2df3 = 1695,
  CODE_FOR_avx_vmcmpv4sf3 = 1696,
  CODE_FOR_avx_vmcmpv2df3 = 1697,
  CODE_FOR_avx_maskcmpv8sf3 = 1702,
  CODE_FOR_sse_maskcmpv4sf3 = 1703,
  CODE_FOR_avx_maskcmpv4df3 = 1704,
  CODE_FOR_sse2_maskcmpv2df3 = 1705,
  CODE_FOR_sse_vmmaskcmpv4sf3 = 1706,
  CODE_FOR_sse2_vmmaskcmpv2df3 = 1707,
  CODE_FOR_avx512f_cmpv16si3 = 1708,
  CODE_FOR_avx512f_cmpv16si3_round = 1709,
  CODE_FOR_avx512f_cmpv16si3_mask = 1710,
  CODE_FOR_avx512f_cmpv16si3_mask_round = 1711,
  CODE_FOR_avx512vl_cmpv8si3 = 1712,
   CODE_FOR_avx512vl_cmpv8si3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv8si3_mask = 1713,
   CODE_FOR_avx512vl_cmpv8si3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4si3 = 1714,
   CODE_FOR_avx512vl_cmpv4si3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4si3_mask = 1715,
   CODE_FOR_avx512vl_cmpv4si3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv8di3 = 1716,
  CODE_FOR_avx512f_cmpv8di3_round = 1717,
  CODE_FOR_avx512f_cmpv8di3_mask = 1718,
  CODE_FOR_avx512f_cmpv8di3_mask_round = 1719,
  CODE_FOR_avx512vl_cmpv4di3 = 1720,
   CODE_FOR_avx512vl_cmpv4di3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4di3_mask = 1721,
   CODE_FOR_avx512vl_cmpv4di3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2di3 = 1722,
   CODE_FOR_avx512vl_cmpv2di3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2di3_mask = 1723,
   CODE_FOR_avx512vl_cmpv2di3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv16sf3 = 1724,
  CODE_FOR_avx512f_cmpv16sf3_round = 1725,
  CODE_FOR_avx512f_cmpv16sf3_mask = 1726,
  CODE_FOR_avx512f_cmpv16sf3_mask_round = 1727,
  CODE_FOR_avx512vl_cmpv8sf3 = 1728,
   CODE_FOR_avx512vl_cmpv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv8sf3_mask = 1729,
   CODE_FOR_avx512vl_cmpv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4sf3 = 1730,
   CODE_FOR_avx512vl_cmpv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4sf3_mask = 1731,
   CODE_FOR_avx512vl_cmpv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv8df3 = 1732,
  CODE_FOR_avx512f_cmpv8df3_round = 1733,
  CODE_FOR_avx512f_cmpv8df3_mask = 1734,
  CODE_FOR_avx512f_cmpv8df3_mask_round = 1735,
  CODE_FOR_avx512vl_cmpv4df3 = 1736,
   CODE_FOR_avx512vl_cmpv4df3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4df3_mask = 1737,
   CODE_FOR_avx512vl_cmpv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2df3 = 1738,
   CODE_FOR_avx512vl_cmpv2df3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2df3_mask = 1739,
   CODE_FOR_avx512vl_cmpv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_cmpv64qi3 = 1740,
  CODE_FOR_avx512bw_cmpv64qi3_mask = 1741,
  CODE_FOR_avx512vl_cmpv16qi3 = 1742,
  CODE_FOR_avx512vl_cmpv16qi3_mask = 1743,
  CODE_FOR_avx512vl_cmpv32qi3 = 1744,
  CODE_FOR_avx512vl_cmpv32qi3_mask = 1745,
  CODE_FOR_avx512bw_cmpv32hi3 = 1746,
  CODE_FOR_avx512bw_cmpv32hi3_mask = 1747,
  CODE_FOR_avx512vl_cmpv16hi3 = 1748,
  CODE_FOR_avx512vl_cmpv16hi3_mask = 1749,
  CODE_FOR_avx512vl_cmpv8hi3 = 1750,
  CODE_FOR_avx512vl_cmpv8hi3_mask = 1751,
  CODE_FOR_avx512bw_ucmpv64qi3 = 1752,
  CODE_FOR_avx512bw_ucmpv64qi3_mask = 1753,
  CODE_FOR_avx512vl_ucmpv16qi3 = 1754,
  CODE_FOR_avx512vl_ucmpv16qi3_mask = 1755,
  CODE_FOR_avx512vl_ucmpv32qi3 = 1756,
  CODE_FOR_avx512vl_ucmpv32qi3_mask = 1757,
  CODE_FOR_avx512bw_ucmpv32hi3 = 1758,
  CODE_FOR_avx512bw_ucmpv32hi3_mask = 1759,
  CODE_FOR_avx512vl_ucmpv16hi3 = 1760,
  CODE_FOR_avx512vl_ucmpv16hi3_mask = 1761,
  CODE_FOR_avx512vl_ucmpv8hi3 = 1762,
  CODE_FOR_avx512vl_ucmpv8hi3_mask = 1763,
  CODE_FOR_avx512f_ucmpv16si3 = 1764,
  CODE_FOR_avx512f_ucmpv16si3_mask = 1765,
  CODE_FOR_avx512vl_ucmpv8si3 = 1766,
  CODE_FOR_avx512vl_ucmpv8si3_mask = 1767,
  CODE_FOR_avx512vl_ucmpv4si3 = 1768,
  CODE_FOR_avx512vl_ucmpv4si3_mask = 1769,
  CODE_FOR_avx512f_ucmpv8di3 = 1770,
  CODE_FOR_avx512f_ucmpv8di3_mask = 1771,
  CODE_FOR_avx512vl_ucmpv4di3 = 1772,
  CODE_FOR_avx512vl_ucmpv4di3_mask = 1773,
  CODE_FOR_avx512vl_ucmpv2di3 = 1774,
  CODE_FOR_avx512vl_ucmpv2di3_mask = 1775,
  CODE_FOR_avx512f_vmcmpv4sf3 = 1776,
  CODE_FOR_avx512f_vmcmpv4sf3_round = 1777,
  CODE_FOR_avx512f_vmcmpv2df3 = 1778,
  CODE_FOR_avx512f_vmcmpv2df3_round = 1779,
  CODE_FOR_avx512f_vmcmpv4sf3_mask = 1780,
  CODE_FOR_avx512f_vmcmpv4sf3_mask_round = 1781,
  CODE_FOR_avx512f_vmcmpv2df3_mask = 1782,
  CODE_FOR_avx512f_vmcmpv2df3_mask_round = 1783,
  CODE_FOR_avx512f_maskcmpv16sf3 = 1784,
  CODE_FOR_avx512f_maskcmpv8sf3 = 1785,
  CODE_FOR_avx512f_maskcmpv4sf3 = 1786,
  CODE_FOR_avx512f_maskcmpv8df3 = 1787,
  CODE_FOR_avx512f_maskcmpv4df3 = 1788,
  CODE_FOR_avx512f_maskcmpv2df3 = 1789,
  CODE_FOR_sse_comi = 1790,
  CODE_FOR_sse_comi_round = 1791,
  CODE_FOR_sse_ucomi = 1792,
  CODE_FOR_sse_ucomi_round = 1793,
  CODE_FOR_sse2_comi = 1794,
  CODE_FOR_sse2_comi_round = 1795,
  CODE_FOR_sse2_ucomi = 1796,
  CODE_FOR_sse2_ucomi_round = 1797,
  CODE_FOR_avx_andnotv8sf3 = 1798,
  CODE_FOR_avx_andnotv8sf3_mask = 1799,
  CODE_FOR_sse_andnotv4sf3 = 1800,
  CODE_FOR_sse_andnotv4sf3_mask = 1801,
  CODE_FOR_avx_andnotv4df3 = 1802,
  CODE_FOR_avx_andnotv4df3_mask = 1803,
  CODE_FOR_sse2_andnotv2df3 = 1804,
  CODE_FOR_sse2_andnotv2df3_mask = 1805,
  CODE_FOR_avx512f_andnotv16sf3 = 1806,
  CODE_FOR_avx512f_andnotv16sf3_mask = 1807,
  CODE_FOR_avx512f_andnotv8df3 = 1808,
  CODE_FOR_avx512f_andnotv8df3_mask = 1809,
  CODE_FOR_fma_fmadd_v16sf_maskz_1 = 1866,
  CODE_FOR_fma_fmadd_v16sf_maskz_1_round = 1868,
  CODE_FOR_fma_fmadd_v8sf_maskz_1 = 1870,
   CODE_FOR_fma_fmadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v4sf_maskz_1 = 1872,
   CODE_FOR_fma_fmadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v8df_maskz_1 = 1875,
  CODE_FOR_fma_fmadd_v8df_maskz_1_round = 1877,
  CODE_FOR_fma_fmadd_v4df_maskz_1 = 1879,
   CODE_FOR_fma_fmadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v2df_maskz_1 = 1881,
   CODE_FOR_fma_fmadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v16sf_mask = 1882,
  CODE_FOR_avx512f_fmadd_v16sf_mask_round = 1883,
  CODE_FOR_avx512vl_fmadd_v8sf_mask = 1884,
   CODE_FOR_avx512vl_fmadd_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmadd_v4sf_mask = 1885,
   CODE_FOR_avx512vl_fmadd_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v8df_mask = 1886,
  CODE_FOR_avx512f_fmadd_v8df_mask_round = 1887,
  CODE_FOR_avx512vl_fmadd_v4df_mask = 1888,
   CODE_FOR_avx512vl_fmadd_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmadd_v2df_mask = 1889,
   CODE_FOR_avx512vl_fmadd_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v16sf_mask3 = 1890,
  CODE_FOR_avx512f_fmadd_v16sf_mask3_round = 1891,
  CODE_FOR_avx512vl_fmadd_v8sf_mask3 = 1892,
  CODE_FOR_avx512vl_fmadd_v8sf_mask3_round = 1893,
  CODE_FOR_avx512vl_fmadd_v4sf_mask3 = 1894,
  CODE_FOR_avx512vl_fmadd_v4sf_mask3_round = 1895,
  CODE_FOR_avx512f_fmadd_v8df_mask3 = 1896,
  CODE_FOR_avx512f_fmadd_v8df_mask3_round = 1897,
  CODE_FOR_avx512vl_fmadd_v4df_mask3 = 1898,
  CODE_FOR_avx512vl_fmadd_v4df_mask3_round = 1899,
  CODE_FOR_avx512vl_fmadd_v2df_mask3 = 1900,
  CODE_FOR_avx512vl_fmadd_v2df_mask3_round = 1901,
  CODE_FOR_fma_fmsub_v16sf_maskz_1 = 1910,
  CODE_FOR_fma_fmsub_v16sf_maskz_1_round = 1912,
  CODE_FOR_fma_fmsub_v8sf_maskz_1 = 1914,
   CODE_FOR_fma_fmsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v4sf_maskz_1 = 1916,
   CODE_FOR_fma_fmsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v8df_maskz_1 = 1919,
  CODE_FOR_fma_fmsub_v8df_maskz_1_round = 1921,
  CODE_FOR_fma_fmsub_v4df_maskz_1 = 1923,
   CODE_FOR_fma_fmsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v2df_maskz_1 = 1925,
   CODE_FOR_fma_fmsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsub_v16sf_mask = 1926,
  CODE_FOR_avx512f_fmsub_v16sf_mask_round = 1927,
  CODE_FOR_avx512vl_fmsub_v8sf_mask = 1928,
  CODE_FOR_avx512vl_fmsub_v8sf_mask_round = 1929,
  CODE_FOR_avx512vl_fmsub_v4sf_mask = 1930,
  CODE_FOR_avx512vl_fmsub_v4sf_mask_round = 1931,
  CODE_FOR_avx512f_fmsub_v8df_mask = 1932,
  CODE_FOR_avx512f_fmsub_v8df_mask_round = 1933,
  CODE_FOR_avx512vl_fmsub_v4df_mask = 1934,
  CODE_FOR_avx512vl_fmsub_v4df_mask_round = 1935,
  CODE_FOR_avx512vl_fmsub_v2df_mask = 1936,
  CODE_FOR_avx512vl_fmsub_v2df_mask_round = 1937,
  CODE_FOR_avx512f_fmsub_v16sf_mask3 = 1938,
  CODE_FOR_avx512f_fmsub_v16sf_mask3_round = 1939,
  CODE_FOR_avx512vl_fmsub_v8sf_mask3 = 1940,
   CODE_FOR_avx512vl_fmsub_v8sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmsub_v4sf_mask3 = 1941,
   CODE_FOR_avx512vl_fmsub_v4sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsub_v8df_mask3 = 1942,
  CODE_FOR_avx512f_fmsub_v8df_mask3_round = 1943,
  CODE_FOR_avx512vl_fmsub_v4df_mask3 = 1944,
   CODE_FOR_avx512vl_fmsub_v4df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmsub_v2df_mask3 = 1945,
   CODE_FOR_avx512vl_fmsub_v2df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v16sf_maskz_1 = 1954,
  CODE_FOR_fma_fnmadd_v16sf_maskz_1_round = 1956,
  CODE_FOR_fma_fnmadd_v8sf_maskz_1 = 1958,
   CODE_FOR_fma_fnmadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v4sf_maskz_1 = 1960,
   CODE_FOR_fma_fnmadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v8df_maskz_1 = 1963,
  CODE_FOR_fma_fnmadd_v8df_maskz_1_round = 1965,
  CODE_FOR_fma_fnmadd_v4df_maskz_1 = 1967,
   CODE_FOR_fma_fnmadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v2df_maskz_1 = 1969,
   CODE_FOR_fma_fnmadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v16sf_mask = 1970,
  CODE_FOR_avx512f_fnmadd_v16sf_mask_round = 1971,
  CODE_FOR_avx512vl_fnmadd_v8sf_mask = 1972,
   CODE_FOR_avx512vl_fnmadd_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v4sf_mask = 1973,
   CODE_FOR_avx512vl_fnmadd_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v8df_mask = 1974,
  CODE_FOR_avx512f_fnmadd_v8df_mask_round = 1975,
  CODE_FOR_avx512vl_fnmadd_v4df_mask = 1976,
   CODE_FOR_avx512vl_fnmadd_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v2df_mask = 1977,
   CODE_FOR_avx512vl_fnmadd_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v16sf_mask3 = 1978,
  CODE_FOR_avx512f_fnmadd_v16sf_mask3_round = 1979,
  CODE_FOR_avx512vl_fnmadd_v8sf_mask3 = 1980,
   CODE_FOR_avx512vl_fnmadd_v8sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v4sf_mask3 = 1981,
   CODE_FOR_avx512vl_fnmadd_v4sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v8df_mask3 = 1982,
  CODE_FOR_avx512f_fnmadd_v8df_mask3_round = 1983,
  CODE_FOR_avx512vl_fnmadd_v4df_mask3 = 1984,
   CODE_FOR_avx512vl_fnmadd_v4df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v2df_mask3 = 1985,
   CODE_FOR_avx512vl_fnmadd_v2df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v16sf_maskz_1 = 2008,
  CODE_FOR_fma_fnmsub_v16sf_maskz_1_round = 2010,
  CODE_FOR_fma_fnmsub_v8sf_maskz_1 = 2012,
   CODE_FOR_fma_fnmsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v4sf_maskz_1 = 2014,
   CODE_FOR_fma_fnmsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v8df_maskz_1 = 2017,
  CODE_FOR_fma_fnmsub_v8df_maskz_1_round = 2019,
  CODE_FOR_fma_fnmsub_v4df_maskz_1 = 2021,
   CODE_FOR_fma_fnmsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v2df_maskz_1 = 2023,
   CODE_FOR_fma_fnmsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v16sf_mask = 2024,
  CODE_FOR_avx512f_fnmsub_v16sf_mask_round = 2025,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask = 2026,
   CODE_FOR_avx512vl_fnmsub_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask = 2027,
   CODE_FOR_avx512vl_fnmsub_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v8df_mask = 2028,
  CODE_FOR_avx512f_fnmsub_v8df_mask_round = 2029,
  CODE_FOR_avx512vl_fnmsub_v4df_mask = 2030,
   CODE_FOR_avx512vl_fnmsub_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmsub_v2df_mask = 2031,
   CODE_FOR_avx512vl_fnmsub_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v16sf_mask3 = 2032,
  CODE_FOR_avx512f_fnmsub_v16sf_mask3_round = 2033,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask3 = 2034,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask3_round = 2035,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask3 = 2036,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask3_round = 2037,
  CODE_FOR_avx512f_fnmsub_v8df_mask3 = 2038,
  CODE_FOR_avx512f_fnmsub_v8df_mask3_round = 2039,
  CODE_FOR_avx512vl_fnmsub_v4df_mask3 = 2040,
  CODE_FOR_avx512vl_fnmsub_v4df_mask3_round = 2041,
  CODE_FOR_avx512vl_fnmsub_v2df_mask3 = 2042,
  CODE_FOR_avx512vl_fnmsub_v2df_mask3_round = 2043,
  CODE_FOR_fma_fmaddsub_v16sf_maskz_1 = 2050,
  CODE_FOR_fma_fmaddsub_v16sf_maskz_1_round = 2052,
  CODE_FOR_fma_fmaddsub_v8sf_maskz_1 = 2054,
   CODE_FOR_fma_fmaddsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v4sf_maskz_1 = 2056,
   CODE_FOR_fma_fmaddsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v8df_maskz_1 = 2059,
  CODE_FOR_fma_fmaddsub_v8df_maskz_1_round = 2061,
  CODE_FOR_fma_fmaddsub_v4df_maskz_1 = 2063,
   CODE_FOR_fma_fmaddsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v2df_maskz_1 = 2065,
   CODE_FOR_fma_fmaddsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask = 2066,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask_round = 2067,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask = 2068,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask_round = 2069,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask = 2070,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask_round = 2071,
  CODE_FOR_avx512f_fmaddsub_v8df_mask = 2072,
  CODE_FOR_avx512f_fmaddsub_v8df_mask_round = 2073,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask = 2074,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask_round = 2075,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask = 2076,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask_round = 2077,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask3 = 2078,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask3_round = 2079,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask3 = 2080,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask3_round = 2081,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask3 = 2082,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask3_round = 2083,
  CODE_FOR_avx512f_fmaddsub_v8df_mask3 = 2084,
  CODE_FOR_avx512f_fmaddsub_v8df_mask3_round = 2085,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask3 = 2086,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask3_round = 2087,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask3 = 2088,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask3_round = 2089,
  CODE_FOR_fma_fmsubadd_v16sf_maskz_1 = 2096,
  CODE_FOR_fma_fmsubadd_v16sf_maskz_1_round = 2098,
  CODE_FOR_fma_fmsubadd_v8sf_maskz_1 = 2100,
   CODE_FOR_fma_fmsubadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v4sf_maskz_1 = 2102,
   CODE_FOR_fma_fmsubadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v8df_maskz_1 = 2105,
  CODE_FOR_fma_fmsubadd_v8df_maskz_1_round = 2107,
  CODE_FOR_fma_fmsubadd_v4df_maskz_1 = 2109,
   CODE_FOR_fma_fmsubadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v2df_maskz_1 = 2111,
   CODE_FOR_fma_fmsubadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask = 2112,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask_round = 2113,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask = 2114,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask_round = 2115,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask = 2116,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask_round = 2117,
  CODE_FOR_avx512f_fmsubadd_v8df_mask = 2118,
  CODE_FOR_avx512f_fmsubadd_v8df_mask_round = 2119,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask = 2120,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask_round = 2121,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask = 2122,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask_round = 2123,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask3 = 2124,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask3_round = 2125,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask3 = 2126,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask3_round = 2127,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask3 = 2128,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask3_round = 2129,
  CODE_FOR_avx512f_fmsubadd_v8df_mask3 = 2130,
  CODE_FOR_avx512f_fmsubadd_v8df_mask3_round = 2131,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask3 = 2132,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask3_round = 2133,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask3 = 2134,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask3_round = 2135,
  CODE_FOR_sse_cvtpi2ps = 2160,
  CODE_FOR_sse_cvtps2pi = 2161,
  CODE_FOR_sse_cvttps2pi = 2162,
  CODE_FOR_sse_cvtsi2ss = 2163,
  CODE_FOR_sse_cvtsi2ss_round = 2164,
  CODE_FOR_sse_cvtsi2ssq = 2165,
  CODE_FOR_sse_cvtsi2ssq_round = 2166,
  CODE_FOR_sse_cvtss2si = 2167,
  CODE_FOR_sse_cvtss2si_round = 2168,
  CODE_FOR_sse_cvtss2si_2 = 2169,
  CODE_FOR_sse_cvtss2siq = 2170,
  CODE_FOR_sse_cvtss2siq_round = 2171,
  CODE_FOR_sse_cvtss2siq_2 = 2172,
  CODE_FOR_sse_cvttss2si = 2173,
  CODE_FOR_sse_cvttss2si_round = 2174,
  CODE_FOR_sse_cvttss2siq = 2175,
  CODE_FOR_sse_cvttss2siq_round = 2176,
  CODE_FOR_cvtusi2ss32 = 2177,
  CODE_FOR_cvtusi2ss32_round = 2178,
  CODE_FOR_cvtusi2sd32 = 2179,
   CODE_FOR_cvtusi2sd32_round = CODE_FOR_nothing,
  CODE_FOR_cvtusi2ss64 = 2180,
  CODE_FOR_cvtusi2ss64_round = 2181,
  CODE_FOR_cvtusi2sd64 = 2182,
  CODE_FOR_cvtusi2sd64_round = 2183,
  CODE_FOR_floatv16siv16sf2 = 2184,
  CODE_FOR_floatv16siv16sf2_round = 2185,
  CODE_FOR_floatv16siv16sf2_mask = 2186,
  CODE_FOR_floatv16siv16sf2_mask_round = 2187,
  CODE_FOR_floatv8siv8sf2 = 2188,
   CODE_FOR_floatv8siv8sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv8siv8sf2_mask = 2189,
   CODE_FOR_floatv8siv8sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatv4siv4sf2 = 2190,
   CODE_FOR_floatv4siv4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv4siv4sf2_mask = 2191,
   CODE_FOR_floatv4siv4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufloatv16siv16sf2 = 2192,
  CODE_FOR_ufloatv16siv16sf2_round = 2193,
  CODE_FOR_ufloatv16siv16sf2_mask = 2194,
  CODE_FOR_ufloatv16siv16sf2_mask_round = 2195,
  CODE_FOR_ufloatv8siv8sf2 = 2196,
  CODE_FOR_ufloatv8siv8sf2_round = 2197,
  CODE_FOR_ufloatv8siv8sf2_mask = 2198,
  CODE_FOR_ufloatv8siv8sf2_mask_round = 2199,
  CODE_FOR_ufloatv4siv4sf2 = 2200,
  CODE_FOR_ufloatv4siv4sf2_round = 2201,
  CODE_FOR_ufloatv4siv4sf2_mask = 2202,
  CODE_FOR_ufloatv4siv4sf2_mask_round = 2203,
  CODE_FOR_avx_fix_notruncv8sfv8si = 2204,
  CODE_FOR_avx_fix_notruncv8sfv8si_mask = 2205,
  CODE_FOR_sse2_fix_notruncv4sfv4si = 2206,
  CODE_FOR_sse2_fix_notruncv4sfv4si_mask = 2207,
  CODE_FOR_avx512f_fix_notruncv16sfv16si = 2208,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_round = 2209,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_mask = 2210,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_mask_round = 2211,
  CODE_FOR_avx512f_ufix_notruncv16sfv16si_mask = 2214,
  CODE_FOR_avx512f_ufix_notruncv16sfv16si_mask_round = 2215,
  CODE_FOR_avx512vl_ufix_notruncv8sfv8si_mask = 2218,
  CODE_FOR_avx512vl_ufix_notruncv8sfv8si_mask_round = 2219,
  CODE_FOR_avx512vl_ufix_notruncv4sfv4si_mask = 2222,
  CODE_FOR_avx512vl_ufix_notruncv4sfv4si_mask_round = 2223,
  CODE_FOR_avx512dq_cvtps2qqv8di_mask = 2226,
  CODE_FOR_avx512dq_cvtps2qqv8di_mask_round = 2227,
  CODE_FOR_avx512dq_cvtps2qqv4di_mask = 2229,
   CODE_FOR_avx512dq_cvtps2qqv4di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_cvtps2qqv2di_mask = 2231,
  CODE_FOR_avx512dq_cvtps2uqqv8di_mask = 2234,
  CODE_FOR_avx512dq_cvtps2uqqv8di_mask_round = 2235,
  CODE_FOR_avx512dq_cvtps2uqqv4di_mask = 2237,
   CODE_FOR_avx512dq_cvtps2uqqv4di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_cvtps2uqqv2di_mask = 2239,
  CODE_FOR_fix_truncv16sfv16si2 = 2240,
  CODE_FOR_fix_truncv16sfv16si2_round = 2241,
  CODE_FOR_fix_truncv16sfv16si2_mask = 2242,
  CODE_FOR_fix_truncv16sfv16si2_mask_round = 2243,
  CODE_FOR_ufix_truncv16sfv16si2 = 2244,
  CODE_FOR_ufix_truncv16sfv16si2_round = 2245,
  CODE_FOR_ufix_truncv16sfv16si2_mask = 2246,
  CODE_FOR_ufix_truncv16sfv16si2_mask_round = 2247,
  CODE_FOR_fix_truncv8sfv8si2 = 2248,
  CODE_FOR_fix_truncv8sfv8si2_mask = 2249,
  CODE_FOR_fix_truncv4sfv4si2 = 2250,
  CODE_FOR_fix_truncv4sfv4si2_mask = 2251,
  CODE_FOR_sse2_cvtpi2pd = 2252,
  CODE_FOR_sse2_cvtpd2pi = 2253,
  CODE_FOR_sse2_cvttpd2pi = 2254,
  CODE_FOR_sse2_cvtsi2sd = 2255,
  CODE_FOR_sse2_cvtsi2sdq = 2256,
  CODE_FOR_sse2_cvtsi2sdq_round = 2257,
  CODE_FOR_avx512f_vcvtss2usi = 2258,
  CODE_FOR_avx512f_vcvtss2usi_round = 2259,
  CODE_FOR_avx512f_vcvtss2usiq = 2260,
  CODE_FOR_avx512f_vcvtss2usiq_round = 2261,
  CODE_FOR_avx512f_vcvttss2usi = 2262,
  CODE_FOR_avx512f_vcvttss2usi_round = 2263,
  CODE_FOR_avx512f_vcvttss2usiq = 2264,
  CODE_FOR_avx512f_vcvttss2usiq_round = 2265,
  CODE_FOR_avx512f_vcvtsd2usi = 2266,
  CODE_FOR_avx512f_vcvtsd2usi_round = 2267,
  CODE_FOR_avx512f_vcvtsd2usiq = 2268,
  CODE_FOR_avx512f_vcvtsd2usiq_round = 2269,
  CODE_FOR_avx512f_vcvttsd2usi = 2270,
  CODE_FOR_avx512f_vcvttsd2usi_round = 2271,
  CODE_FOR_avx512f_vcvttsd2usiq = 2272,
  CODE_FOR_avx512f_vcvttsd2usiq_round = 2273,
  CODE_FOR_sse2_cvtsd2si = 2274,
  CODE_FOR_sse2_cvtsd2si_round = 2275,
  CODE_FOR_sse2_cvtsd2si_2 = 2276,
  CODE_FOR_sse2_cvtsd2siq = 2277,
  CODE_FOR_sse2_cvtsd2siq_round = 2278,
  CODE_FOR_sse2_cvtsd2siq_2 = 2279,
  CODE_FOR_sse2_cvttsd2si = 2280,
  CODE_FOR_sse2_cvttsd2si_round = 2281,
  CODE_FOR_sse2_cvttsd2siq = 2282,
  CODE_FOR_sse2_cvttsd2siq_round = 2283,
  CODE_FOR_floatv8siv8df2 = 2284,
  CODE_FOR_floatv8siv8df2_mask = 2285,
  CODE_FOR_floatv4siv4df2 = 2286,
  CODE_FOR_floatv4siv4df2_mask = 2287,
  CODE_FOR_floatv8div8df2 = 2288,
  CODE_FOR_floatv8div8df2_round = 2289,
  CODE_FOR_floatv8div8df2_mask = 2290,
  CODE_FOR_floatv8div8df2_mask_round = 2291,
  CODE_FOR_ufloatv8div8df2 = 2292,
  CODE_FOR_ufloatv8div8df2_round = 2293,
  CODE_FOR_ufloatv8div8df2_mask = 2294,
  CODE_FOR_ufloatv8div8df2_mask_round = 2295,
  CODE_FOR_floatv4div4df2 = 2296,
  CODE_FOR_floatv4div4df2_round = 2297,
  CODE_FOR_floatv4div4df2_mask = 2298,
  CODE_FOR_floatv4div4df2_mask_round = 2299,
  CODE_FOR_ufloatv4div4df2 = 2300,
  CODE_FOR_ufloatv4div4df2_round = 2301,
  CODE_FOR_ufloatv4div4df2_mask = 2302,
  CODE_FOR_ufloatv4div4df2_mask_round = 2303,
  CODE_FOR_floatv2div2df2 = 2304,
  CODE_FOR_floatv2div2df2_round = 2305,
  CODE_FOR_floatv2div2df2_mask = 2306,
  CODE_FOR_floatv2div2df2_mask_round = 2307,
  CODE_FOR_ufloatv2div2df2 = 2308,
  CODE_FOR_ufloatv2div2df2_round = 2309,
  CODE_FOR_ufloatv2div2df2_mask = 2310,
  CODE_FOR_ufloatv2div2df2_mask_round = 2311,
  CODE_FOR_floatv8div8sf2 = 2312,
  CODE_FOR_floatv8div8sf2_round = 2313,
  CODE_FOR_floatv8div8sf2_mask = 2314,
  CODE_FOR_floatv8div8sf2_mask_round = 2315,
  CODE_FOR_ufloatv8div8sf2 = 2316,
  CODE_FOR_ufloatv8div8sf2_round = 2317,
  CODE_FOR_ufloatv8div8sf2_mask = 2318,
  CODE_FOR_ufloatv8div8sf2_mask_round = 2319,
  CODE_FOR_floatv4div4sf2 = 2320,
   CODE_FOR_floatv4div4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv4div4sf2_mask = 2321,
   CODE_FOR_floatv4div4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufloatv4div4sf2 = 2322,
   CODE_FOR_ufloatv4div4sf2_round = CODE_FOR_nothing,
  CODE_FOR_ufloatv4div4sf2_mask = 2323,
   CODE_FOR_ufloatv4div4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatv2div2sf2_mask = 2326,
  CODE_FOR_ufloatv2div2sf2_mask = 2327,
  CODE_FOR_ufloatv8siv8df2 = 2330,
  CODE_FOR_ufloatv8siv8df2_mask = 2331,
  CODE_FOR_ufloatv4siv4df2 = 2332,
  CODE_FOR_ufloatv4siv4df2_mask = 2333,
  CODE_FOR_ufloatv2siv2df2 = 2334,
  CODE_FOR_ufloatv2siv2df2_mask = 2335,
  CODE_FOR_avx512f_cvtdq2pd512_2 = 2336,
  CODE_FOR_avx_cvtdq2pd256_2 = 2337,
  CODE_FOR_sse2_cvtdq2pd = 2338,
  CODE_FOR_sse2_cvtdq2pd_mask = 2339,
  CODE_FOR_avx512f_cvtpd2dq512 = 2340,
  CODE_FOR_avx512f_cvtpd2dq512_round = 2341,
  CODE_FOR_avx512f_cvtpd2dq512_mask = 2342,
  CODE_FOR_avx512f_cvtpd2dq512_mask_round = 2343,
  CODE_FOR_avx_cvtpd2dq256 = 2344,
  CODE_FOR_avx_cvtpd2dq256_mask = 2345,
  CODE_FOR_sse2_cvtpd2dq = 2347,
  CODE_FOR_sse2_cvtpd2dq_mask = 2348,
  CODE_FOR_ufix_notruncv8dfv8si2 = 2349,
  CODE_FOR_ufix_notruncv8dfv8si2_round = 2350,
  CODE_FOR_ufix_notruncv8dfv8si2_mask = 2351,
  CODE_FOR_ufix_notruncv8dfv8si2_mask_round = 2352,
  CODE_FOR_ufix_notruncv4dfv4si2 = 2353,
  CODE_FOR_ufix_notruncv4dfv4si2_round = 2354,
  CODE_FOR_ufix_notruncv4dfv4si2_mask = 2355,
  CODE_FOR_ufix_notruncv4dfv4si2_mask_round = 2356,
  CODE_FOR_ufix_notruncv2dfv2si2 = 2357,
  CODE_FOR_ufix_notruncv2dfv2si2_mask = 2358,
  CODE_FOR_fix_truncv8dfv8si2 = 2359,
  CODE_FOR_fix_truncv8dfv8si2_round = 2360,
  CODE_FOR_fix_truncv8dfv8si2_mask = 2361,
  CODE_FOR_fix_truncv8dfv8si2_mask_round = 2362,
  CODE_FOR_ufix_truncv8dfv8si2 = 2363,
  CODE_FOR_ufix_truncv8dfv8si2_round = 2364,
  CODE_FOR_ufix_truncv8dfv8si2_mask = 2365,
  CODE_FOR_ufix_truncv8dfv8si2_mask_round = 2366,
  CODE_FOR_ufix_truncv2dfv2si2 = 2367,
  CODE_FOR_ufix_truncv2dfv2si2_mask = 2368,
  CODE_FOR_fix_truncv4dfv4si2 = 2369,
  CODE_FOR_fix_truncv4dfv4si2_mask = 2370,
  CODE_FOR_ufix_truncv4dfv4si2 = 2371,
  CODE_FOR_ufix_truncv4dfv4si2_mask = 2372,
  CODE_FOR_fix_truncv8dfv8di2 = 2373,
  CODE_FOR_fix_truncv8dfv8di2_round = 2374,
  CODE_FOR_fix_truncv8dfv8di2_mask = 2375,
  CODE_FOR_fix_truncv8dfv8di2_mask_round = 2376,
  CODE_FOR_ufix_truncv8dfv8di2 = 2377,
  CODE_FOR_ufix_truncv8dfv8di2_round = 2378,
  CODE_FOR_ufix_truncv8dfv8di2_mask = 2379,
  CODE_FOR_ufix_truncv8dfv8di2_mask_round = 2380,
  CODE_FOR_fix_truncv4dfv4di2 = 2381,
   CODE_FOR_fix_truncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv4dfv4di2_mask = 2382,
   CODE_FOR_fix_truncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufix_truncv4dfv4di2 = 2383,
   CODE_FOR_ufix_truncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_ufix_truncv4dfv4di2_mask = 2384,
   CODE_FOR_ufix_truncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2dfv2di2 = 2385,
   CODE_FOR_fix_truncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2dfv2di2_mask = 2386,
   CODE_FOR_fix_truncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufix_truncv2dfv2di2 = 2387,
   CODE_FOR_ufix_truncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_ufix_truncv2dfv2di2_mask = 2388,
   CODE_FOR_ufix_truncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv8dfv8di2 = 2389,
  CODE_FOR_fix_notruncv8dfv8di2_round = 2390,
  CODE_FOR_fix_notruncv8dfv8di2_mask = 2391,
  CODE_FOR_fix_notruncv8dfv8di2_mask_round = 2392,
  CODE_FOR_fix_notruncv4dfv4di2 = 2393,
   CODE_FOR_fix_notruncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv4dfv4di2_mask = 2394,
   CODE_FOR_fix_notruncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv2dfv2di2 = 2395,
   CODE_FOR_fix_notruncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv2dfv2di2_mask = 2396,
   CODE_FOR_fix_notruncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv8dfv8di2 = 2397,
  CODE_FOR_ufix_notruncv8dfv8di2_round = 2398,
  CODE_FOR_ufix_notruncv8dfv8di2_mask = 2399,
  CODE_FOR_ufix_notruncv8dfv8di2_mask_round = 2400,
  CODE_FOR_ufix_notruncv4dfv4di2 = 2401,
   CODE_FOR_ufix_notruncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv4dfv4di2_mask = 2402,
   CODE_FOR_ufix_notruncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv2dfv2di2 = 2403,
   CODE_FOR_ufix_notruncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv2dfv2di2_mask = 2404,
   CODE_FOR_ufix_notruncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv8sfv8di2 = 2405,
  CODE_FOR_fix_truncv8sfv8di2_round = 2406,
  CODE_FOR_fix_truncv8sfv8di2_mask = 2407,
  CODE_FOR_fix_truncv8sfv8di2_mask_round = 2408,
  CODE_FOR_ufix_truncv8sfv8di2 = 2409,
  CODE_FOR_ufix_truncv8sfv8di2_round = 2410,
  CODE_FOR_ufix_truncv8sfv8di2_mask = 2411,
  CODE_FOR_ufix_truncv8sfv8di2_mask_round = 2412,
  CODE_FOR_fix_truncv4sfv4di2 = 2413,
   CODE_FOR_fix_truncv4sfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv4sfv4di2_mask = 2414,
   CODE_FOR_fix_truncv4sfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufix_truncv4sfv4di2 = 2415,
   CODE_FOR_ufix_truncv4sfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_ufix_truncv4sfv4di2_mask = 2416,
   CODE_FOR_ufix_truncv4sfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2sfv2di2 = 2417,
  CODE_FOR_fix_truncv2sfv2di2_mask = 2418,
  CODE_FOR_ufix_truncv2sfv2di2 = 2419,
  CODE_FOR_ufix_truncv2sfv2di2_mask = 2420,
  CODE_FOR_ufix_truncv8sfv8si2 = 2421,
  CODE_FOR_ufix_truncv8sfv8si2_mask = 2422,
  CODE_FOR_ufix_truncv4sfv4si2 = 2423,
  CODE_FOR_ufix_truncv4sfv4si2_mask = 2424,
  CODE_FOR_sse2_cvttpd2dq = 2425,
  CODE_FOR_sse2_cvttpd2dq_mask = 2426,
  CODE_FOR_sse2_cvtsd2ss = 2427,
  CODE_FOR_sse2_cvtsd2ss_round = 2428,
  CODE_FOR_sse2_cvtss2sd = 2430,
  CODE_FOR_sse2_cvtss2sd_round = 2431,
  CODE_FOR_avx512f_cvtpd2ps512_mask = 2435,
  CODE_FOR_avx512f_cvtpd2ps512_mask_round = 2436,
  CODE_FOR_avx_cvtpd2ps256 = 2437,
  CODE_FOR_avx_cvtpd2ps256_mask = 2438,
  CODE_FOR_avx512f_cvtps2pd512 = 2441,
  CODE_FOR_avx512f_cvtps2pd512_round = 2442,
  CODE_FOR_avx512f_cvtps2pd512_mask = 2443,
  CODE_FOR_avx512f_cvtps2pd512_mask_round = 2444,
  CODE_FOR_avx_cvtps2pd256 = 2445,
   CODE_FOR_avx_cvtps2pd256_round = CODE_FOR_nothing,
  CODE_FOR_avx_cvtps2pd256_mask = 2446,
   CODE_FOR_avx_cvtps2pd256_mask_round = CODE_FOR_nothing,
  CODE_FOR_vec_unpacks_lo_v16sf = 2448,
  CODE_FOR_avx512bw_cvtb2maskv64qi = 2449,
  CODE_FOR_avx512vl_cvtb2maskv16qi = 2450,
  CODE_FOR_avx512vl_cvtb2maskv32qi = 2451,
  CODE_FOR_avx512bw_cvtw2maskv32hi = 2452,
  CODE_FOR_avx512vl_cvtw2maskv16hi = 2453,
  CODE_FOR_avx512vl_cvtw2maskv8hi = 2454,
  CODE_FOR_avx512f_cvtd2maskv16si = 2455,
  CODE_FOR_avx512vl_cvtd2maskv8si = 2456,
  CODE_FOR_avx512vl_cvtd2maskv4si = 2457,
  CODE_FOR_avx512f_cvtq2maskv8di = 2458,
  CODE_FOR_avx512vl_cvtq2maskv4di = 2459,
  CODE_FOR_avx512vl_cvtq2maskv2di = 2460,
  CODE_FOR_sse2_cvtps2pd = 2473,
  CODE_FOR_sse2_cvtps2pd_mask = 2474,
  CODE_FOR_sse_movhlps = 2475,
  CODE_FOR_sse_movlhps = 2476,
  CODE_FOR_avx512f_unpckhps512_mask = 2478,
  CODE_FOR_avx_unpckhps256 = 2479,
  CODE_FOR_avx_unpckhps256_mask = 2480,
  CODE_FOR_vec_interleave_highv4sf = 2481,
  CODE_FOR_vec_interleave_highv4sf_mask = 2482,
  CODE_FOR_avx512f_unpcklps512_mask = 2484,
  CODE_FOR_avx_unpcklps256 = 2485,
  CODE_FOR_avx_unpcklps256_mask = 2486,
  CODE_FOR_unpcklps128_mask = 2487,
  CODE_FOR_vec_interleave_lowv4sf = 2488,
  CODE_FOR_avx_movshdup256 = 2489,
  CODE_FOR_avx_movshdup256_mask = 2490,
  CODE_FOR_sse3_movshdup = 2491,
  CODE_FOR_sse3_movshdup_mask = 2492,
  CODE_FOR_avx512f_movshdup512_mask = 2494,
  CODE_FOR_avx_movsldup256 = 2495,
  CODE_FOR_avx_movsldup256_mask = 2496,
  CODE_FOR_sse3_movsldup = 2497,
  CODE_FOR_sse3_movsldup_mask = 2498,
  CODE_FOR_avx512f_movsldup512_mask = 2500,
  CODE_FOR_avx_shufps256_1 = 2501,
  CODE_FOR_avx_shufps256_1_mask = 2502,
  CODE_FOR_sse_shufps_v4sf_mask = 2503,
  CODE_FOR_sse_shufps_v4si = 2504,
  CODE_FOR_sse_shufps_v4sf = 2505,
  CODE_FOR_sse_storehps = 2506,
  CODE_FOR_sse_loadhps = 2507,
  CODE_FOR_sse_storelps = 2508,
  CODE_FOR_sse_loadlps = 2509,
  CODE_FOR_sse_movss = 2510,
  CODE_FOR_avx2_vec_dupv8sf = 2511,
  CODE_FOR_avx2_vec_dupv4sf = 2512,
  CODE_FOR_avx2_vec_dupv8sf_1 = 2513,
  CODE_FOR_avx512f_vec_dupv16sf_1 = 2514,
  CODE_FOR_avx512f_vec_dupv8df_1 = 2515,
  CODE_FOR_vec_setv4si_0 = 2519,
  CODE_FOR_vec_setv4sf_0 = 2520,
  CODE_FOR_vec_setv8si_0 = 2522,
  CODE_FOR_vec_setv8sf_0 = 2523,
  CODE_FOR_vec_setv16si_0 = 2524,
  CODE_FOR_vec_setv16sf_0 = 2525,
  CODE_FOR_sse4_1_insertps = 2526,
  CODE_FOR_avx512dq_vextractf64x2_1_maskm = 2530,
  CODE_FOR_avx512dq_vextracti64x2_1_maskm = 2531,
  CODE_FOR_avx512f_vextractf32x4_1_maskm = 2532,
  CODE_FOR_avx512f_vextracti32x4_1_maskm = 2533,
  CODE_FOR_avx512dq_vextractf64x2_1_mask = 2535,
  CODE_FOR_avx512dq_vextracti64x2_1_mask = 2537,
  CODE_FOR_avx512f_vextractf32x4_1_mask = 2539,
  CODE_FOR_avx512f_vextracti32x4_1_mask = 2541,
  CODE_FOR_vec_extract_lo_v8df_maskm = 2542,
  CODE_FOR_vec_extract_lo_v8di_maskm = 2543,
  CODE_FOR_vec_extract_lo_v8df = 2544,
  CODE_FOR_vec_extract_lo_v8df_mask = 2545,
  CODE_FOR_vec_extract_lo_v8di = 2546,
  CODE_FOR_vec_extract_lo_v8di_mask = 2547,
  CODE_FOR_vec_extract_hi_v8df_maskm = 2548,
  CODE_FOR_vec_extract_hi_v8di_maskm = 2549,
  CODE_FOR_vec_extract_hi_v8df = 2550,
  CODE_FOR_vec_extract_hi_v8df_mask = 2551,
  CODE_FOR_vec_extract_hi_v8di = 2552,
  CODE_FOR_vec_extract_hi_v8di_mask = 2553,
  CODE_FOR_vec_extract_hi_v16sf_maskm = 2554,
  CODE_FOR_vec_extract_hi_v16si_maskm = 2555,
  CODE_FOR_vec_extract_hi_v16sf = 2556,
  CODE_FOR_vec_extract_hi_v16sf_mask = 2557,
  CODE_FOR_vec_extract_hi_v16si = 2558,
  CODE_FOR_vec_extract_hi_v16si_mask = 2559,
  CODE_FOR_vec_extract_lo_v16sf = 2560,
  CODE_FOR_vec_extract_lo_v16sf_mask = 2561,
  CODE_FOR_vec_extract_lo_v16si = 2562,
  CODE_FOR_vec_extract_lo_v16si_mask = 2563,
  CODE_FOR_vec_extract_lo_v4di = 2564,
  CODE_FOR_vec_extract_lo_v4di_mask = 2565,
  CODE_FOR_vec_extract_lo_v4df = 2566,
  CODE_FOR_vec_extract_lo_v4df_mask = 2567,
  CODE_FOR_vec_extract_hi_v4di = 2568,
  CODE_FOR_vec_extract_hi_v4di_mask = 2569,
  CODE_FOR_vec_extract_hi_v4df = 2570,
  CODE_FOR_vec_extract_hi_v4df_mask = 2571,
  CODE_FOR_vec_extract_lo_v8si = 2572,
  CODE_FOR_vec_extract_lo_v8si_mask = 2573,
  CODE_FOR_vec_extract_lo_v8sf = 2574,
  CODE_FOR_vec_extract_lo_v8sf_mask = 2575,
  CODE_FOR_vec_extract_lo_v8si_maskm = 2576,
  CODE_FOR_vec_extract_lo_v8sf_maskm = 2577,
  CODE_FOR_vec_extract_hi_v8si_maskm = 2578,
  CODE_FOR_vec_extract_hi_v8sf_maskm = 2579,
  CODE_FOR_vec_extract_hi_v8si_mask = 2580,
  CODE_FOR_vec_extract_hi_v8sf_mask = 2581,
  CODE_FOR_vec_extract_hi_v8si = 2582,
  CODE_FOR_vec_extract_hi_v8sf = 2583,
  CODE_FOR_vec_extract_lo_v32hi = 2584,
  CODE_FOR_vec_extract_hi_v32hi = 2585,
  CODE_FOR_vec_extract_lo_v16hi = 2586,
  CODE_FOR_vec_extract_hi_v16hi = 2587,
  CODE_FOR_vec_extract_lo_v64qi = 2588,
  CODE_FOR_vec_extract_hi_v64qi = 2589,
  CODE_FOR_vec_extract_lo_v32qi = 2590,
  CODE_FOR_vec_extract_hi_v32qi = 2591,
  CODE_FOR_avx512f_unpckhpd512_mask = 2593,
  CODE_FOR_avx_unpckhpd256 = 2594,
  CODE_FOR_avx_unpckhpd256_mask = 2595,
  CODE_FOR_avx512vl_unpckhpd128_mask = 2596,
  CODE_FOR_avx512vl_unpcklpd128_mask = 2602,
  CODE_FOR_avx512f_vmscalefv4sf = 2604,
  CODE_FOR_avx512f_vmscalefv4sf_round = 2605,
  CODE_FOR_avx512f_vmscalefv4sf_mask = 2606,
  CODE_FOR_avx512f_vmscalefv4sf_mask_round = 2607,
  CODE_FOR_avx512f_vmscalefv2df = 2608,
  CODE_FOR_avx512f_vmscalefv2df_round = 2609,
  CODE_FOR_avx512f_vmscalefv2df_mask = 2610,
  CODE_FOR_avx512f_vmscalefv2df_mask_round = 2611,
  CODE_FOR_avx512f_scalefv16sf = 2612,
  CODE_FOR_avx512f_scalefv16sf_round = 2613,
  CODE_FOR_avx512f_scalefv16sf_mask = 2614,
  CODE_FOR_avx512f_scalefv16sf_mask_round = 2615,
  CODE_FOR_avx512vl_scalefv8sf = 2616,
  CODE_FOR_avx512vl_scalefv8sf_round = 2617,
  CODE_FOR_avx512vl_scalefv8sf_mask = 2618,
  CODE_FOR_avx512vl_scalefv8sf_mask_round = 2619,
  CODE_FOR_avx512vl_scalefv4sf = 2620,
  CODE_FOR_avx512vl_scalefv4sf_round = 2621,
  CODE_FOR_avx512vl_scalefv4sf_mask = 2622,
  CODE_FOR_avx512vl_scalefv4sf_mask_round = 2623,
  CODE_FOR_avx512f_scalefv8df = 2624,
  CODE_FOR_avx512f_scalefv8df_round = 2625,
  CODE_FOR_avx512f_scalefv8df_mask = 2626,
  CODE_FOR_avx512f_scalefv8df_mask_round = 2627,
  CODE_FOR_avx512vl_scalefv4df = 2628,
  CODE_FOR_avx512vl_scalefv4df_round = 2629,
  CODE_FOR_avx512vl_scalefv4df_mask = 2630,
  CODE_FOR_avx512vl_scalefv4df_mask_round = 2631,
  CODE_FOR_avx512vl_scalefv2df = 2632,
  CODE_FOR_avx512vl_scalefv2df_round = 2633,
  CODE_FOR_avx512vl_scalefv2df_mask = 2634,
  CODE_FOR_avx512vl_scalefv2df_mask_round = 2635,
  CODE_FOR_avx512f_vternlogv16si = 2636,
  CODE_FOR_avx512f_vternlogv16si_maskz_1 = 2637,
  CODE_FOR_avx512vl_vternlogv8si = 2638,
  CODE_FOR_avx512vl_vternlogv8si_maskz_1 = 2639,
  CODE_FOR_avx512vl_vternlogv4si = 2640,
  CODE_FOR_avx512vl_vternlogv4si_maskz_1 = 2641,
  CODE_FOR_avx512f_vternlogv8di = 2642,
  CODE_FOR_avx512f_vternlogv8di_maskz_1 = 2643,
  CODE_FOR_avx512vl_vternlogv4di = 2644,
  CODE_FOR_avx512vl_vternlogv4di_maskz_1 = 2645,
  CODE_FOR_avx512vl_vternlogv2di = 2646,
  CODE_FOR_avx512vl_vternlogv2di_maskz_1 = 2647,
  CODE_FOR_avx512f_vternlogv16si_mask = 2648,
  CODE_FOR_avx512vl_vternlogv8si_mask = 2649,
  CODE_FOR_avx512vl_vternlogv4si_mask = 2650,
  CODE_FOR_avx512f_vternlogv8di_mask = 2651,
  CODE_FOR_avx512vl_vternlogv4di_mask = 2652,
  CODE_FOR_avx512vl_vternlogv2di_mask = 2653,
  CODE_FOR_avx512f_getexpv16sf = 2654,
  CODE_FOR_avx512f_getexpv16sf_round = 2655,
  CODE_FOR_avx512f_getexpv16sf_mask = 2656,
  CODE_FOR_avx512f_getexpv16sf_mask_round = 2657,
  CODE_FOR_avx512vl_getexpv8sf = 2658,
  CODE_FOR_avx512vl_getexpv8sf_round = 2659,
  CODE_FOR_avx512vl_getexpv8sf_mask = 2660,
  CODE_FOR_avx512vl_getexpv8sf_mask_round = 2661,
  CODE_FOR_avx512vl_getexpv4sf = 2662,
  CODE_FOR_avx512vl_getexpv4sf_round = 2663,
  CODE_FOR_avx512vl_getexpv4sf_mask = 2664,
  CODE_FOR_avx512vl_getexpv4sf_mask_round = 2665,
  CODE_FOR_avx512f_getexpv8df = 2666,
  CODE_FOR_avx512f_getexpv8df_round = 2667,
  CODE_FOR_avx512f_getexpv8df_mask = 2668,
  CODE_FOR_avx512f_getexpv8df_mask_round = 2669,
  CODE_FOR_avx512vl_getexpv4df = 2670,
  CODE_FOR_avx512vl_getexpv4df_round = 2671,
  CODE_FOR_avx512vl_getexpv4df_mask = 2672,
  CODE_FOR_avx512vl_getexpv4df_mask_round = 2673,
  CODE_FOR_avx512vl_getexpv2df = 2674,
  CODE_FOR_avx512vl_getexpv2df_round = 2675,
  CODE_FOR_avx512vl_getexpv2df_mask = 2676,
  CODE_FOR_avx512vl_getexpv2df_mask_round = 2677,
  CODE_FOR_avx512f_sgetexpv4sf = 2678,
  CODE_FOR_avx512f_sgetexpv4sf_mask = 2679,
  CODE_FOR_avx512f_sgetexpv4sf_round = 2680,
  CODE_FOR_avx512f_sgetexpv4sf_mask_round = 2681,
  CODE_FOR_avx512f_sgetexpv2df = 2682,
  CODE_FOR_avx512f_sgetexpv2df_mask = 2683,
  CODE_FOR_avx512f_sgetexpv2df_round = 2684,
  CODE_FOR_avx512f_sgetexpv2df_mask_round = 2685,
  CODE_FOR_avx512f_alignv16si_mask = 2687,
  CODE_FOR_avx512vl_alignv8si_mask = 2689,
  CODE_FOR_avx512vl_alignv4si_mask = 2691,
  CODE_FOR_avx512f_alignv8di_mask = 2693,
  CODE_FOR_avx512vl_alignv4di_mask = 2695,
  CODE_FOR_avx512vl_alignv2di_mask = 2697,
  CODE_FOR_avx512f_fixupimmv16sf = 2698,
  CODE_FOR_avx512f_fixupimmv16sf_round = 2699,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_1 = 2700,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_1_round = 2701,
  CODE_FOR_avx512vl_fixupimmv8sf = 2702,
  CODE_FOR_avx512vl_fixupimmv8sf_round = 2703,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_1 = 2704,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_1_round = 2705,
  CODE_FOR_avx512vl_fixupimmv4sf = 2706,
  CODE_FOR_avx512vl_fixupimmv4sf_round = 2707,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_1 = 2708,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_1_round = 2709,
  CODE_FOR_avx512f_fixupimmv8df = 2710,
  CODE_FOR_avx512f_fixupimmv8df_round = 2711,
  CODE_FOR_avx512f_fixupimmv8df_maskz_1 = 2712,
  CODE_FOR_avx512f_fixupimmv8df_maskz_1_round = 2713,
  CODE_FOR_avx512vl_fixupimmv4df = 2714,
  CODE_FOR_avx512vl_fixupimmv4df_round = 2715,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_1 = 2716,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_1_round = 2717,
  CODE_FOR_avx512vl_fixupimmv2df = 2718,
  CODE_FOR_avx512vl_fixupimmv2df_round = 2719,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_1 = 2720,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_1_round = 2721,
  CODE_FOR_avx512f_fixupimmv16sf_mask = 2722,
  CODE_FOR_avx512f_fixupimmv16sf_mask_round = 2723,
  CODE_FOR_avx512vl_fixupimmv8sf_mask = 2724,
  CODE_FOR_avx512vl_fixupimmv8sf_mask_round = 2725,
  CODE_FOR_avx512vl_fixupimmv4sf_mask = 2726,
  CODE_FOR_avx512vl_fixupimmv4sf_mask_round = 2727,
  CODE_FOR_avx512f_fixupimmv8df_mask = 2728,
  CODE_FOR_avx512f_fixupimmv8df_mask_round = 2729,
  CODE_FOR_avx512vl_fixupimmv4df_mask = 2730,
  CODE_FOR_avx512vl_fixupimmv4df_mask_round = 2731,
  CODE_FOR_avx512vl_fixupimmv2df_mask = 2732,
  CODE_FOR_avx512vl_fixupimmv2df_mask_round = 2733,
  CODE_FOR_avx512f_sfixupimmv4sf = 2734,
  CODE_FOR_avx512f_sfixupimmv4sf_round = 2735,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_1 = 2736,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_1_round = 2737,
  CODE_FOR_avx512f_sfixupimmv2df = 2738,
  CODE_FOR_avx512f_sfixupimmv2df_round = 2739,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_1 = 2740,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_1_round = 2741,
  CODE_FOR_avx512f_sfixupimmv4sf_mask = 2742,
  CODE_FOR_avx512f_sfixupimmv4sf_mask_round = 2743,
  CODE_FOR_avx512f_sfixupimmv2df_mask = 2744,
  CODE_FOR_avx512f_sfixupimmv2df_mask_round = 2745,
  CODE_FOR_avx512f_rndscalev16sf = 2746,
  CODE_FOR_avx512f_rndscalev16sf_round = 2747,
  CODE_FOR_avx512f_rndscalev16sf_mask = 2748,
  CODE_FOR_avx512f_rndscalev16sf_mask_round = 2749,
  CODE_FOR_avx512vl_rndscalev8sf = 2750,
  CODE_FOR_avx512vl_rndscalev8sf_round = 2751,
  CODE_FOR_avx512vl_rndscalev8sf_mask = 2752,
  CODE_FOR_avx512vl_rndscalev8sf_mask_round = 2753,
  CODE_FOR_avx512vl_rndscalev4sf = 2754,
  CODE_FOR_avx512vl_rndscalev4sf_round = 2755,
  CODE_FOR_avx512vl_rndscalev4sf_mask = 2756,
  CODE_FOR_avx512vl_rndscalev4sf_mask_round = 2757,
  CODE_FOR_avx512f_rndscalev8df = 2758,
  CODE_FOR_avx512f_rndscalev8df_round = 2759,
  CODE_FOR_avx512f_rndscalev8df_mask = 2760,
  CODE_FOR_avx512f_rndscalev8df_mask_round = 2761,
  CODE_FOR_avx512vl_rndscalev4df = 2762,
  CODE_FOR_avx512vl_rndscalev4df_round = 2763,
  CODE_FOR_avx512vl_rndscalev4df_mask = 2764,
  CODE_FOR_avx512vl_rndscalev4df_mask_round = 2765,
  CODE_FOR_avx512vl_rndscalev2df = 2766,
  CODE_FOR_avx512vl_rndscalev2df_round = 2767,
  CODE_FOR_avx512vl_rndscalev2df_mask = 2768,
  CODE_FOR_avx512vl_rndscalev2df_mask_round = 2769,
  CODE_FOR_avx512f_rndscalev4sf = 2770,
  CODE_FOR_avx512f_rndscalev4sf_round = 2771,
  CODE_FOR_avx512f_rndscalev2df = 2772,
  CODE_FOR_avx512f_rndscalev2df_round = 2773,
  CODE_FOR_avx512f_shufps512_1 = 2774,
  CODE_FOR_avx512f_shufps512_1_mask = 2775,
  CODE_FOR_avx512f_shufpd512_1 = 2776,
  CODE_FOR_avx512f_shufpd512_1_mask = 2777,
  CODE_FOR_avx_shufpd256_1 = 2778,
  CODE_FOR_avx_shufpd256_1_mask = 2779,
  CODE_FOR_sse2_shufpd_v2df_mask = 2780,
  CODE_FOR_avx2_interleave_highv4di = 2781,
  CODE_FOR_avx2_interleave_highv4di_mask = 2782,
  CODE_FOR_avx512f_interleave_highv8di_mask = 2784,
  CODE_FOR_vec_interleave_highv2di = 2785,
  CODE_FOR_vec_interleave_highv2di_mask = 2786,
  CODE_FOR_avx2_interleave_lowv4di = 2787,
  CODE_FOR_avx2_interleave_lowv4di_mask = 2788,
  CODE_FOR_avx512f_interleave_lowv8di_mask = 2790,
  CODE_FOR_vec_interleave_lowv2di = 2791,
  CODE_FOR_vec_interleave_lowv2di_mask = 2792,
  CODE_FOR_sse2_shufpd_v2di = 2793,
  CODE_FOR_sse2_shufpd_v2df = 2794,
  CODE_FOR_sse2_storehpd = 2795,
  CODE_FOR_sse2_storelpd = 2797,
  CODE_FOR_sse2_loadhpd = 2799,
  CODE_FOR_sse2_loadlpd = 2800,
  CODE_FOR_sse2_movsd = 2801,
  CODE_FOR_vec_dupv2df = 2802,
  CODE_FOR_vec_dupv2df_mask = 2803,
  CODE_FOR_vec_concatv2df = 2804,
  CODE_FOR_vec_setv8df_0 = 2805,
  CODE_FOR_vec_setv4df_0 = 2806,
  CODE_FOR_avx512f_ss_truncatev16siv16qi2_mask = 2819,
  CODE_FOR_avx512f_truncatev16siv16qi2_mask = 2820,
  CODE_FOR_avx512f_us_truncatev16siv16qi2_mask = 2821,
  CODE_FOR_avx512f_ss_truncatev16siv16hi2_mask = 2822,
  CODE_FOR_avx512f_truncatev16siv16hi2_mask = 2823,
  CODE_FOR_avx512f_us_truncatev16siv16hi2_mask = 2824,
  CODE_FOR_avx512f_ss_truncatev8div8si2_mask = 2825,
  CODE_FOR_avx512f_truncatev8div8si2_mask = 2826,
  CODE_FOR_avx512f_us_truncatev8div8si2_mask = 2827,
  CODE_FOR_avx512f_ss_truncatev8div8hi2_mask = 2828,
  CODE_FOR_avx512f_truncatev8div8hi2_mask = 2829,
  CODE_FOR_avx512f_us_truncatev8div8hi2_mask = 2830,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2 = 2831,
  CODE_FOR_avx512bw_truncatev32hiv32qi2 = 2832,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2 = 2833,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2_mask = 2834,
  CODE_FOR_avx512bw_truncatev32hiv32qi2_mask = 2835,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2_mask = 2836,
  CODE_FOR_avx512vl_ss_truncatev4div4si2_mask = 2846,
  CODE_FOR_avx512vl_truncatev4div4si2_mask = 2847,
  CODE_FOR_avx512vl_us_truncatev4div4si2_mask = 2848,
  CODE_FOR_avx512vl_ss_truncatev8siv8hi2_mask = 2849,
  CODE_FOR_avx512vl_truncatev8siv8hi2_mask = 2850,
  CODE_FOR_avx512vl_us_truncatev8siv8hi2_mask = 2851,
  CODE_FOR_avx512vl_ss_truncatev16hiv16qi2_mask = 2852,
  CODE_FOR_avx512vl_truncatev16hiv16qi2_mask = 2853,
  CODE_FOR_avx512vl_us_truncatev16hiv16qi2_mask = 2854,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2_mask = 2873,
  CODE_FOR_avx512vl_truncatev2div2qi2_mask = 2874,
  CODE_FOR_avx512vl_us_truncatev2div2qi2_mask = 2875,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2_mask_store = 2879,
  CODE_FOR_avx512vl_truncatev2div2qi2_mask_store = 2880,
  CODE_FOR_avx512vl_us_truncatev2div2qi2_mask_store = 2881,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2_mask = 2888,
  CODE_FOR_avx512vl_truncatev4siv4qi2_mask = 2889,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2_mask = 2890,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2_mask = 2891,
  CODE_FOR_avx512vl_truncatev4div4qi2_mask = 2892,
  CODE_FOR_avx512vl_us_truncatev4div4qi2_mask = 2893,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2_mask_store = 2900,
  CODE_FOR_avx512vl_truncatev4siv4qi2_mask_store = 2901,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2_mask_store = 2902,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2_mask_store = 2903,
  CODE_FOR_avx512vl_truncatev4div4qi2_mask_store = 2904,
  CODE_FOR_avx512vl_us_truncatev4div4qi2_mask_store = 2905,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2_mask = 2912,
  CODE_FOR_avx512vl_truncatev8hiv8qi2_mask = 2913,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2_mask = 2914,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2_mask = 2915,
  CODE_FOR_avx512vl_truncatev8siv8qi2_mask = 2916,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2_mask = 2917,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2_mask_store = 2924,
  CODE_FOR_avx512vl_truncatev8hiv8qi2_mask_store = 2925,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2_mask_store = 2926,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2_mask_store = 2927,
  CODE_FOR_avx512vl_truncatev8siv8qi2_mask_store = 2928,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2_mask_store = 2929,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2_mask = 2945,
  CODE_FOR_avx512vl_truncatev4siv4hi2_mask = 2946,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2_mask = 2947,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2_mask = 2948,
  CODE_FOR_avx512vl_truncatev4div4hi2_mask = 2949,
  CODE_FOR_avx512vl_us_truncatev4div4hi2_mask = 2950,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2_mask_store = 2957,
  CODE_FOR_avx512vl_truncatev4siv4hi2_mask_store = 2958,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2_mask_store = 2959,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2_mask_store = 2960,
  CODE_FOR_avx512vl_truncatev4div4hi2_mask_store = 2961,
  CODE_FOR_avx512vl_us_truncatev4div4hi2_mask_store = 2962,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2_mask = 2966,
  CODE_FOR_avx512vl_truncatev2div2hi2_mask = 2967,
  CODE_FOR_avx512vl_us_truncatev2div2hi2_mask = 2968,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2_mask_store = 2972,
  CODE_FOR_avx512vl_truncatev2div2hi2_mask_store = 2973,
  CODE_FOR_avx512vl_us_truncatev2div2hi2_mask_store = 2974,
  CODE_FOR_avx512vl_ss_truncatev2div2si2_mask = 2981,
  CODE_FOR_avx512vl_truncatev2div2si2_mask = 2982,
  CODE_FOR_avx512vl_us_truncatev2div2si2_mask = 2983,
  CODE_FOR_avx512vl_ss_truncatev2div2si2_mask_store = 2987,
  CODE_FOR_avx512vl_truncatev2div2si2_mask_store = 2988,
  CODE_FOR_avx512vl_us_truncatev2div2si2_mask_store = 2989,
  CODE_FOR_avx512f_ss_truncatev8div16qi2_mask = 2996,
  CODE_FOR_avx512f_truncatev8div16qi2_mask = 2997,
  CODE_FOR_avx512f_us_truncatev8div16qi2_mask = 2998,
  CODE_FOR_avx512f_ss_truncatev8div16qi2_mask_store = 3002,
  CODE_FOR_avx512f_truncatev8div16qi2_mask_store = 3003,
  CODE_FOR_avx512f_us_truncatev8div16qi2_mask_store = 3004,
  CODE_FOR_avx512bw_pmaddwd512v32hi = 3131,
  CODE_FOR_avx512bw_pmaddwd512v32hi_mask = 3132,
  CODE_FOR_avx512bw_pmaddwd512v16hi = 3133,
  CODE_FOR_avx512bw_pmaddwd512v16hi_mask = 3134,
  CODE_FOR_avx512bw_pmaddwd512v8hi = 3135,
  CODE_FOR_avx512bw_pmaddwd512v8hi_mask = 3136,
  CODE_FOR_avx512dq_mulv8di3 = 3139,
  CODE_FOR_avx512dq_mulv8di3_mask = 3140,
  CODE_FOR_avx512dq_mulv4di3 = 3141,
  CODE_FOR_avx512dq_mulv4di3_mask = 3142,
  CODE_FOR_avx512dq_mulv2di3 = 3143,
  CODE_FOR_avx512dq_mulv2di3_mask = 3144,
  CODE_FOR_ashrv16hi3_mask = 3152,
  CODE_FOR_ashrv8hi3_mask = 3154,
  CODE_FOR_ashrv8si3_mask = 3156,
  CODE_FOR_ashrv4si3_mask = 3158,
  CODE_FOR_ashrv2di3_mask = 3160,
  CODE_FOR_ashrv16hi3 = 3161,
  CODE_FOR_ashrv8hi3 = 3162,
  CODE_FOR_ashrv8si3 = 3163,
  CODE_FOR_ashrv4si3 = 3164,
  CODE_FOR_ashrv32hi3 = 3165,
  CODE_FOR_ashrv32hi3_mask = 3166,
  CODE_FOR_ashrv4di3 = 3167,
  CODE_FOR_ashrv4di3_mask = 3168,
  CODE_FOR_ashrv16si3 = 3169,
  CODE_FOR_ashrv16si3_mask = 3170,
  CODE_FOR_ashrv8di3 = 3171,
  CODE_FOR_ashrv8di3_mask = 3172,
  CODE_FOR_ashlv16hi3_mask = 3174,
  CODE_FOR_lshrv16hi3_mask = 3176,
  CODE_FOR_ashlv8hi3_mask = 3178,
  CODE_FOR_lshrv8hi3_mask = 3180,
  CODE_FOR_ashlv8si3_mask = 3182,
  CODE_FOR_lshrv8si3_mask = 3184,
  CODE_FOR_ashlv4si3_mask = 3186,
  CODE_FOR_lshrv4si3_mask = 3188,
  CODE_FOR_ashlv4di3_mask = 3190,
  CODE_FOR_lshrv4di3_mask = 3192,
  CODE_FOR_ashlv2di3_mask = 3194,
  CODE_FOR_lshrv2di3_mask = 3196,
  CODE_FOR_ashlv16hi3 = 3197,
  CODE_FOR_lshrv16hi3 = 3198,
  CODE_FOR_ashlv8hi3 = 3199,
  CODE_FOR_lshrv8hi3 = 3200,
  CODE_FOR_ashlv8si3 = 3201,
  CODE_FOR_lshrv8si3 = 3202,
  CODE_FOR_ashlv4si3 = 3203,
  CODE_FOR_lshrv4si3 = 3204,
  CODE_FOR_ashlv4di3 = 3205,
  CODE_FOR_lshrv4di3 = 3206,
  CODE_FOR_ashlv2di3 = 3207,
  CODE_FOR_lshrv2di3 = 3208,
  CODE_FOR_ashlv32hi3 = 3209,
  CODE_FOR_ashlv32hi3_mask = 3210,
  CODE_FOR_lshrv32hi3 = 3211,
  CODE_FOR_lshrv32hi3_mask = 3212,
  CODE_FOR_ashlv16si3 = 3213,
  CODE_FOR_ashlv16si3_mask = 3214,
  CODE_FOR_lshrv16si3 = 3215,
  CODE_FOR_lshrv16si3_mask = 3216,
  CODE_FOR_ashlv8di3 = 3217,
  CODE_FOR_ashlv8di3_mask = 3218,
  CODE_FOR_lshrv8di3 = 3219,
  CODE_FOR_lshrv8di3_mask = 3220,
  CODE_FOR_avx512bw_ashlv4ti3 = 3221,
  CODE_FOR_avx512bw_lshrv4ti3 = 3222,
  CODE_FOR_avx512bw_ashlv2ti3 = 3223,
  CODE_FOR_avx512bw_lshrv2ti3 = 3224,
  CODE_FOR_avx512bw_ashlv1ti3 = 3225,
  CODE_FOR_avx512bw_lshrv1ti3 = 3226,
  CODE_FOR_avx2_ashlv2ti3 = 3227,
  CODE_FOR_avx2_lshrv2ti3 = 3228,
  CODE_FOR_sse2_ashlv1ti3 = 3229,
  CODE_FOR_sse2_lshrv1ti3 = 3230,
  CODE_FOR_avx512f_rolvv16si = 3231,
  CODE_FOR_avx512f_rolvv16si_mask = 3232,
  CODE_FOR_avx512f_rorvv16si = 3233,
  CODE_FOR_avx512f_rorvv16si_mask = 3234,
  CODE_FOR_avx512vl_rolvv8si = 3235,
  CODE_FOR_avx512vl_rolvv8si_mask = 3236,
  CODE_FOR_avx512vl_rorvv8si = 3237,
  CODE_FOR_avx512vl_rorvv8si_mask = 3238,
  CODE_FOR_avx512vl_rolvv4si = 3239,
  CODE_FOR_avx512vl_rolvv4si_mask = 3240,
  CODE_FOR_avx512vl_rorvv4si = 3241,
  CODE_FOR_avx512vl_rorvv4si_mask = 3242,
  CODE_FOR_avx512f_rolvv8di = 3243,
  CODE_FOR_avx512f_rolvv8di_mask = 3244,
  CODE_FOR_avx512f_rorvv8di = 3245,
  CODE_FOR_avx512f_rorvv8di_mask = 3246,
  CODE_FOR_avx512vl_rolvv4di = 3247,
  CODE_FOR_avx512vl_rolvv4di_mask = 3248,
  CODE_FOR_avx512vl_rorvv4di = 3249,
  CODE_FOR_avx512vl_rorvv4di_mask = 3250,
  CODE_FOR_avx512vl_rolvv2di = 3251,
  CODE_FOR_avx512vl_rolvv2di_mask = 3252,
  CODE_FOR_avx512vl_rorvv2di = 3253,
  CODE_FOR_avx512vl_rorvv2di_mask = 3254,
  CODE_FOR_avx512f_rolv16si = 3255,
  CODE_FOR_avx512f_rolv16si_mask = 3256,
  CODE_FOR_avx512f_rorv16si = 3257,
  CODE_FOR_avx512f_rorv16si_mask = 3258,
  CODE_FOR_avx512vl_rolv8si = 3259,
  CODE_FOR_avx512vl_rolv8si_mask = 3260,
  CODE_FOR_avx512vl_rorv8si = 3261,
  CODE_FOR_avx512vl_rorv8si_mask = 3262,
  CODE_FOR_avx512vl_rolv4si = 3263,
  CODE_FOR_avx512vl_rolv4si_mask = 3264,
  CODE_FOR_avx512vl_rorv4si = 3265,
  CODE_FOR_avx512vl_rorv4si_mask = 3266,
  CODE_FOR_avx512f_rolv8di = 3267,
  CODE_FOR_avx512f_rolv8di_mask = 3268,
  CODE_FOR_avx512f_rorv8di = 3269,
  CODE_FOR_avx512f_rorv8di_mask = 3270,
  CODE_FOR_avx512vl_rolv4di = 3271,
  CODE_FOR_avx512vl_rolv4di_mask = 3272,
  CODE_FOR_avx512vl_rorv4di = 3273,
  CODE_FOR_avx512vl_rorv4di_mask = 3274,
  CODE_FOR_avx512vl_rolv2di = 3275,
  CODE_FOR_avx512vl_rolv2di_mask = 3276,
  CODE_FOR_avx512vl_rorv2di = 3277,
  CODE_FOR_avx512vl_rorv2di_mask = 3278,
  CODE_FOR_smaxv64qi3_mask = 3340,
  CODE_FOR_sminv64qi3_mask = 3342,
  CODE_FOR_umaxv64qi3_mask = 3344,
  CODE_FOR_uminv64qi3_mask = 3346,
  CODE_FOR_smaxv16qi3_mask = 3348,
  CODE_FOR_sminv16qi3_mask = 3350,
  CODE_FOR_umaxv16qi3_mask = 3352,
  CODE_FOR_uminv16qi3_mask = 3354,
  CODE_FOR_smaxv32qi3_mask = 3356,
  CODE_FOR_sminv32qi3_mask = 3358,
  CODE_FOR_umaxv32qi3_mask = 3360,
  CODE_FOR_uminv32qi3_mask = 3362,
  CODE_FOR_smaxv32hi3_mask = 3364,
  CODE_FOR_sminv32hi3_mask = 3366,
  CODE_FOR_umaxv32hi3_mask = 3368,
  CODE_FOR_uminv32hi3_mask = 3370,
  CODE_FOR_smaxv16hi3_mask = 3372,
  CODE_FOR_sminv16hi3_mask = 3374,
  CODE_FOR_umaxv16hi3_mask = 3376,
  CODE_FOR_uminv16hi3_mask = 3378,
  CODE_FOR_smaxv8hi3_mask = 3380,
  CODE_FOR_sminv8hi3_mask = 3382,
  CODE_FOR_umaxv8hi3_mask = 3384,
  CODE_FOR_uminv8hi3_mask = 3386,
  CODE_FOR_avx512bw_eqv64qi3_1 = 3411,
  CODE_FOR_avx512bw_eqv64qi3_mask_1 = 3412,
  CODE_FOR_avx512vl_eqv16qi3_1 = 3413,
  CODE_FOR_avx512vl_eqv16qi3_mask_1 = 3414,
  CODE_FOR_avx512vl_eqv32qi3_1 = 3415,
  CODE_FOR_avx512vl_eqv32qi3_mask_1 = 3416,
  CODE_FOR_avx512bw_eqv32hi3_1 = 3417,
  CODE_FOR_avx512bw_eqv32hi3_mask_1 = 3418,
  CODE_FOR_avx512vl_eqv16hi3_1 = 3419,
  CODE_FOR_avx512vl_eqv16hi3_mask_1 = 3420,
  CODE_FOR_avx512vl_eqv8hi3_1 = 3421,
  CODE_FOR_avx512vl_eqv8hi3_mask_1 = 3422,
  CODE_FOR_avx512f_eqv16si3_1 = 3423,
  CODE_FOR_avx512f_eqv16si3_mask_1 = 3424,
  CODE_FOR_avx512vl_eqv8si3_1 = 3425,
  CODE_FOR_avx512vl_eqv8si3_mask_1 = 3426,
  CODE_FOR_avx512vl_eqv4si3_1 = 3427,
  CODE_FOR_avx512vl_eqv4si3_mask_1 = 3428,
  CODE_FOR_avx512f_eqv8di3_1 = 3429,
  CODE_FOR_avx512f_eqv8di3_mask_1 = 3430,
  CODE_FOR_avx512vl_eqv4di3_1 = 3431,
  CODE_FOR_avx512vl_eqv4di3_mask_1 = 3432,
  CODE_FOR_avx512vl_eqv2di3_1 = 3433,
  CODE_FOR_avx512vl_eqv2di3_mask_1 = 3434,
  CODE_FOR_sse4_2_gtv2di3 = 3439,
  CODE_FOR_avx2_gtv32qi3 = 3440,
  CODE_FOR_avx2_gtv16hi3 = 3441,
  CODE_FOR_avx2_gtv8si3 = 3442,
  CODE_FOR_avx2_gtv4di3 = 3443,
  CODE_FOR_avx512f_gtv16si3 = 3444,
  CODE_FOR_avx512f_gtv16si3_mask = 3445,
  CODE_FOR_avx512vl_gtv8si3 = 3446,
  CODE_FOR_avx512vl_gtv8si3_mask = 3447,
  CODE_FOR_avx512vl_gtv4si3 = 3448,
  CODE_FOR_avx512vl_gtv4si3_mask = 3449,
  CODE_FOR_avx512f_gtv8di3 = 3450,
  CODE_FOR_avx512f_gtv8di3_mask = 3451,
  CODE_FOR_avx512vl_gtv4di3 = 3452,
  CODE_FOR_avx512vl_gtv4di3_mask = 3453,
  CODE_FOR_avx512vl_gtv2di3 = 3454,
  CODE_FOR_avx512vl_gtv2di3_mask = 3455,
  CODE_FOR_avx512bw_gtv64qi3 = 3456,
  CODE_FOR_avx512bw_gtv64qi3_mask = 3457,
  CODE_FOR_avx512vl_gtv16qi3 = 3458,
  CODE_FOR_avx512vl_gtv16qi3_mask = 3459,
  CODE_FOR_avx512vl_gtv32qi3 = 3460,
  CODE_FOR_avx512vl_gtv32qi3_mask = 3461,
  CODE_FOR_avx512bw_gtv32hi3 = 3462,
  CODE_FOR_avx512bw_gtv32hi3_mask = 3463,
  CODE_FOR_avx512vl_gtv16hi3 = 3464,
  CODE_FOR_avx512vl_gtv16hi3_mask = 3465,
  CODE_FOR_avx512vl_gtv8hi3 = 3466,
  CODE_FOR_avx512vl_gtv8hi3_mask = 3467,
  CODE_FOR_sse2_gtv16qi3 = 3468,
  CODE_FOR_sse2_gtv8hi3 = 3469,
  CODE_FOR_sse2_gtv4si3 = 3470,
  CODE_FOR_andv16si3_mask = 3490,
  CODE_FOR_iorv16si3_mask = 3492,
  CODE_FOR_xorv16si3_mask = 3494,
  CODE_FOR_andv8si3_mask = 3496,
  CODE_FOR_iorv8si3_mask = 3498,
  CODE_FOR_xorv8si3_mask = 3500,
  CODE_FOR_andv4si3_mask = 3502,
  CODE_FOR_iorv4si3_mask = 3504,
  CODE_FOR_xorv4si3_mask = 3506,
  CODE_FOR_andv8di3_mask = 3508,
  CODE_FOR_iorv8di3_mask = 3510,
  CODE_FOR_xorv8di3_mask = 3512,
  CODE_FOR_andv4di3_mask = 3514,
  CODE_FOR_iorv4di3_mask = 3516,
  CODE_FOR_xorv4di3_mask = 3518,
  CODE_FOR_andv2di3_mask = 3520,
  CODE_FOR_iorv2di3_mask = 3522,
  CODE_FOR_xorv2di3_mask = 3524,
  CODE_FOR_avx512bw_testmv64qi3 = 3543,
  CODE_FOR_avx512bw_testmv64qi3_mask = 3544,
  CODE_FOR_avx512vl_testmv16qi3 = 3545,
  CODE_FOR_avx512vl_testmv16qi3_mask = 3546,
  CODE_FOR_avx512vl_testmv32qi3 = 3547,
  CODE_FOR_avx512vl_testmv32qi3_mask = 3548,
  CODE_FOR_avx512bw_testmv32hi3 = 3549,
  CODE_FOR_avx512bw_testmv32hi3_mask = 3550,
  CODE_FOR_avx512vl_testmv16hi3 = 3551,
  CODE_FOR_avx512vl_testmv16hi3_mask = 3552,
  CODE_FOR_avx512vl_testmv8hi3 = 3553,
  CODE_FOR_avx512vl_testmv8hi3_mask = 3554,
  CODE_FOR_avx512f_testmv16si3 = 3555,
  CODE_FOR_avx512f_testmv16si3_mask = 3556,
  CODE_FOR_avx512vl_testmv8si3 = 3557,
  CODE_FOR_avx512vl_testmv8si3_mask = 3558,
  CODE_FOR_avx512vl_testmv4si3 = 3559,
  CODE_FOR_avx512vl_testmv4si3_mask = 3560,
  CODE_FOR_avx512f_testmv8di3 = 3561,
  CODE_FOR_avx512f_testmv8di3_mask = 3562,
  CODE_FOR_avx512vl_testmv4di3 = 3563,
  CODE_FOR_avx512vl_testmv4di3_mask = 3564,
  CODE_FOR_avx512vl_testmv2di3 = 3565,
  CODE_FOR_avx512vl_testmv2di3_mask = 3566,
  CODE_FOR_avx512bw_testnmv64qi3 = 3567,
  CODE_FOR_avx512bw_testnmv64qi3_mask = 3568,
  CODE_FOR_avx512vl_testnmv16qi3 = 3569,
  CODE_FOR_avx512vl_testnmv16qi3_mask = 3570,
  CODE_FOR_avx512vl_testnmv32qi3 = 3571,
  CODE_FOR_avx512vl_testnmv32qi3_mask = 3572,
  CODE_FOR_avx512bw_testnmv32hi3 = 3573,
  CODE_FOR_avx512bw_testnmv32hi3_mask = 3574,
  CODE_FOR_avx512vl_testnmv16hi3 = 3575,
  CODE_FOR_avx512vl_testnmv16hi3_mask = 3576,
  CODE_FOR_avx512vl_testnmv8hi3 = 3577,
  CODE_FOR_avx512vl_testnmv8hi3_mask = 3578,
  CODE_FOR_avx512f_testnmv16si3 = 3579,
  CODE_FOR_avx512f_testnmv16si3_mask = 3580,
  CODE_FOR_avx512vl_testnmv8si3 = 3581,
  CODE_FOR_avx512vl_testnmv8si3_mask = 3582,
  CODE_FOR_avx512vl_testnmv4si3 = 3583,
  CODE_FOR_avx512vl_testnmv4si3_mask = 3584,
  CODE_FOR_avx512f_testnmv8di3 = 3585,
  CODE_FOR_avx512f_testnmv8di3_mask = 3586,
  CODE_FOR_avx512vl_testnmv4di3 = 3587,
  CODE_FOR_avx512vl_testnmv4di3_mask = 3588,
  CODE_FOR_avx512vl_testnmv2di3 = 3589,
  CODE_FOR_avx512vl_testnmv2di3_mask = 3590,
  CODE_FOR_avx512bw_packsswb = 3591,
  CODE_FOR_avx512bw_packsswb_mask = 3592,
  CODE_FOR_avx2_packsswb = 3593,
  CODE_FOR_avx2_packsswb_mask = 3594,
  CODE_FOR_sse2_packsswb = 3595,
  CODE_FOR_sse2_packsswb_mask = 3596,
  CODE_FOR_avx512bw_packssdw = 3597,
  CODE_FOR_avx512bw_packssdw_mask = 3598,
  CODE_FOR_avx2_packssdw = 3599,
  CODE_FOR_avx2_packssdw_mask = 3600,
  CODE_FOR_sse2_packssdw = 3601,
  CODE_FOR_sse2_packssdw_mask = 3602,
  CODE_FOR_avx512bw_packuswb = 3603,
  CODE_FOR_avx512bw_packuswb_mask = 3604,
  CODE_FOR_avx2_packuswb = 3605,
  CODE_FOR_avx2_packuswb_mask = 3606,
  CODE_FOR_sse2_packuswb = 3607,
  CODE_FOR_sse2_packuswb_mask = 3608,
  CODE_FOR_avx512bw_interleave_highv64qi = 3609,
  CODE_FOR_avx512bw_interleave_highv64qi_mask = 3610,
  CODE_FOR_avx2_interleave_highv32qi = 3611,
  CODE_FOR_avx2_interleave_highv32qi_mask = 3612,
  CODE_FOR_vec_interleave_highv16qi = 3613,
  CODE_FOR_vec_interleave_highv16qi_mask = 3614,
  CODE_FOR_avx512bw_interleave_lowv64qi = 3615,
  CODE_FOR_avx512bw_interleave_lowv64qi_mask = 3616,
  CODE_FOR_avx2_interleave_lowv32qi = 3617,
  CODE_FOR_avx2_interleave_lowv32qi_mask = 3618,
  CODE_FOR_vec_interleave_lowv16qi = 3619,
  CODE_FOR_vec_interleave_lowv16qi_mask = 3620,
  CODE_FOR_avx512bw_interleave_highv32hi = 3621,
  CODE_FOR_avx512bw_interleave_highv32hi_mask = 3622,
  CODE_FOR_avx2_interleave_highv16hi = 3623,
  CODE_FOR_avx2_interleave_highv16hi_mask = 3624,
  CODE_FOR_vec_interleave_highv8hi = 3625,
  CODE_FOR_vec_interleave_highv8hi_mask = 3626,
  CODE_FOR_avx512bw_interleave_lowv32hi_mask = 3628,
  CODE_FOR_avx2_interleave_lowv16hi = 3629,
  CODE_FOR_avx2_interleave_lowv16hi_mask = 3630,
  CODE_FOR_vec_interleave_lowv8hi = 3631,
  CODE_FOR_vec_interleave_lowv8hi_mask = 3632,
  CODE_FOR_avx2_interleave_highv8si = 3633,
  CODE_FOR_avx2_interleave_highv8si_mask = 3634,
  CODE_FOR_avx512f_interleave_highv16si_mask = 3636,
  CODE_FOR_vec_interleave_highv4si = 3637,
  CODE_FOR_vec_interleave_highv4si_mask = 3638,
  CODE_FOR_avx2_interleave_lowv8si = 3639,
  CODE_FOR_avx2_interleave_lowv8si_mask = 3640,
  CODE_FOR_avx512f_interleave_lowv16si_mask = 3642,
  CODE_FOR_vec_interleave_lowv4si = 3643,
  CODE_FOR_vec_interleave_lowv4si_mask = 3644,
  CODE_FOR_sse4_1_pinsrb = 3645,
  CODE_FOR_sse2_pinsrw = 3646,
  CODE_FOR_sse4_1_pinsrd = 3647,
  CODE_FOR_sse4_1_pinsrq = 3648,
  CODE_FOR_avx512dq_vinsertf64x2_1_mask = 3650,
  CODE_FOR_avx512dq_vinserti64x2_1_mask = 3652,
  CODE_FOR_avx512f_vinsertf32x4_1_mask = 3654,
  CODE_FOR_avx512f_vinserti32x4_1_mask = 3656,
  CODE_FOR_vec_set_lo_v16sf = 3657,
  CODE_FOR_vec_set_lo_v16sf_mask = 3658,
  CODE_FOR_vec_set_lo_v16si = 3659,
  CODE_FOR_vec_set_lo_v16si_mask = 3660,
  CODE_FOR_vec_set_hi_v16sf = 3661,
  CODE_FOR_vec_set_hi_v16sf_mask = 3662,
  CODE_FOR_vec_set_hi_v16si = 3663,
  CODE_FOR_vec_set_hi_v16si_mask = 3664,
  CODE_FOR_vec_set_lo_v8df = 3665,
  CODE_FOR_vec_set_lo_v8df_mask = 3666,
  CODE_FOR_vec_set_lo_v8di = 3667,
  CODE_FOR_vec_set_lo_v8di_mask = 3668,
  CODE_FOR_vec_set_hi_v8df = 3669,
  CODE_FOR_vec_set_hi_v8df_mask = 3670,
  CODE_FOR_vec_set_hi_v8di = 3671,
  CODE_FOR_vec_set_hi_v8di_mask = 3672,
  CODE_FOR_avx512dq_shuf_i64x2_1_mask = 3674,
  CODE_FOR_avx512dq_shuf_f64x2_1_mask = 3676,
  CODE_FOR_avx512f_shuf_f64x2_1 = 3677,
  CODE_FOR_avx512f_shuf_f64x2_1_mask = 3678,
  CODE_FOR_avx512f_shuf_i64x2_1 = 3679,
  CODE_FOR_avx512f_shuf_i64x2_1_mask = 3680,
  CODE_FOR_avx512vl_shuf_i32x4_1 = 3681,
  CODE_FOR_avx512vl_shuf_i32x4_1_mask = 3682,
  CODE_FOR_avx512vl_shuf_f32x4_1 = 3683,
  CODE_FOR_avx512vl_shuf_f32x4_1_mask = 3684,
  CODE_FOR_avx512f_shuf_f32x4_1 = 3685,
  CODE_FOR_avx512f_shuf_f32x4_1_mask = 3686,
  CODE_FOR_avx512f_shuf_i32x4_1 = 3687,
  CODE_FOR_avx512f_shuf_i32x4_1_mask = 3688,
  CODE_FOR_avx512f_pshufd_1 = 3689,
  CODE_FOR_avx512f_pshufd_1_mask = 3690,
  CODE_FOR_avx2_pshufd_1 = 3691,
  CODE_FOR_avx2_pshufd_1_mask = 3692,
  CODE_FOR_sse2_pshufd_1 = 3693,
  CODE_FOR_sse2_pshufd_1_mask = 3694,
  CODE_FOR_avx512bw_pshuflwv32hi_mask = 3696,
  CODE_FOR_avx2_pshuflw_1 = 3697,
  CODE_FOR_avx2_pshuflw_1_mask = 3698,
  CODE_FOR_sse2_pshuflw_1 = 3699,
  CODE_FOR_sse2_pshuflw_1_mask = 3700,
  CODE_FOR_avx512bw_pshufhwv32hi_mask = 3702,
  CODE_FOR_avx2_pshufhw_1 = 3703,
  CODE_FOR_avx2_pshufhw_1_mask = 3704,
  CODE_FOR_sse2_pshufhw_1 = 3705,
  CODE_FOR_sse2_pshufhw_1_mask = 3706,
  CODE_FOR_sse2_loadld = 3707,
  CODE_FOR_vec_concatv2di = 3731,
  CODE_FOR_vec_setv8di_0 = 3732,
  CODE_FOR_vec_setv4di_0 = 3733,
  CODE_FOR_avx512f_psadbw = 3746,
  CODE_FOR_avx2_psadbw = 3747,
  CODE_FOR_sse2_psadbw = 3748,
  CODE_FOR_avx_movmskps256 = 3749,
  CODE_FOR_sse_movmskps = 3750,
  CODE_FOR_avx_movmskpd256 = 3751,
  CODE_FOR_sse2_movmskpd = 3752,
  CODE_FOR_avx2_pmovmskb = 3757,
  CODE_FOR_sse2_pmovmskb = 3758,
  CODE_FOR_sse_ldmxcsr = 3763,
  CODE_FOR_sse_stmxcsr = 3764,
  CODE_FOR_sse2_clflush = 3765,
  CODE_FOR_sse3_mwait = 3766,
  CODE_FOR_sse3_monitor_si = 3767,
  CODE_FOR_sse3_monitor_di = 3768,
  CODE_FOR_avx2_phaddwv16hi3 = 3769,
  CODE_FOR_avx2_phaddswv16hi3 = 3770,
  CODE_FOR_avx2_phsubwv16hi3 = 3771,
  CODE_FOR_avx2_phsubswv16hi3 = 3772,
  CODE_FOR_ssse3_phaddwv8hi3 = 3773,
  CODE_FOR_ssse3_phaddswv8hi3 = 3774,
  CODE_FOR_ssse3_phsubwv8hi3 = 3775,
  CODE_FOR_ssse3_phsubswv8hi3 = 3776,
  CODE_FOR_ssse3_phaddwv4hi3 = 3777,
  CODE_FOR_ssse3_phaddswv4hi3 = 3778,
  CODE_FOR_ssse3_phsubwv4hi3 = 3779,
  CODE_FOR_ssse3_phsubswv4hi3 = 3780,
  CODE_FOR_avx2_phadddv8si3 = 3781,
  CODE_FOR_avx2_phsubdv8si3 = 3782,
  CODE_FOR_ssse3_phadddv4si3 = 3783,
  CODE_FOR_ssse3_phsubdv4si3 = 3784,
  CODE_FOR_ssse3_phadddv2si3 = 3785,
  CODE_FOR_ssse3_phsubdv2si3 = 3786,
  CODE_FOR_avx2_pmaddubsw256 = 3787,
  CODE_FOR_avx512bw_pmaddubsw512v8hi = 3788,
  CODE_FOR_avx512bw_pmaddubsw512v8hi_mask = 3789,
  CODE_FOR_avx512bw_pmaddubsw512v16hi = 3790,
  CODE_FOR_avx512bw_pmaddubsw512v16hi_mask = 3791,
  CODE_FOR_avx512bw_pmaddubsw512v32hi = 3792,
  CODE_FOR_avx512bw_pmaddubsw512v32hi_mask = 3793,
  CODE_FOR_avx512bw_umulhrswv32hi3 = 3794,
  CODE_FOR_avx512bw_umulhrswv32hi3_mask = 3795,
  CODE_FOR_ssse3_pmaddubsw128 = 3796,
  CODE_FOR_ssse3_pmaddubsw = 3797,
  CODE_FOR_avx512bw_pshufbv64qi3 = 3805,
  CODE_FOR_avx512bw_pshufbv64qi3_mask = 3806,
  CODE_FOR_avx2_pshufbv32qi3 = 3807,
  CODE_FOR_avx2_pshufbv32qi3_mask = 3808,
  CODE_FOR_ssse3_pshufbv16qi3 = 3809,
  CODE_FOR_ssse3_pshufbv16qi3_mask = 3810,
  CODE_FOR_ssse3_pshufbv8qi3 = 3811,
  CODE_FOR_avx2_psignv32qi3 = 3812,
  CODE_FOR_ssse3_psignv16qi3 = 3813,
  CODE_FOR_avx2_psignv16hi3 = 3814,
  CODE_FOR_ssse3_psignv8hi3 = 3815,
  CODE_FOR_avx2_psignv8si3 = 3816,
  CODE_FOR_ssse3_psignv4si3 = 3817,
  CODE_FOR_ssse3_psignv8qi3 = 3818,
  CODE_FOR_ssse3_psignv4hi3 = 3819,
  CODE_FOR_ssse3_psignv2si3 = 3820,
  CODE_FOR_avx512bw_palignrv64qi_mask = 3821,
  CODE_FOR_avx2_palignrv32qi_mask = 3822,
  CODE_FOR_ssse3_palignrv16qi_mask = 3823,
  CODE_FOR_avx512bw_palignrv4ti = 3824,
  CODE_FOR_avx2_palignrv2ti = 3825,
  CODE_FOR_ssse3_palignrti = 3826,
  CODE_FOR_ssse3_palignrdi = 3827,
  CODE_FOR_absv16si2_mask = 3840,
  CODE_FOR_absv8si2_mask = 3841,
  CODE_FOR_absv4si2_mask = 3842,
  CODE_FOR_absv8di2_mask = 3843,
  CODE_FOR_absv4di2_mask = 3844,
  CODE_FOR_absv2di2_mask = 3845,
  CODE_FOR_absv64qi2_mask = 3846,
  CODE_FOR_absv16qi2_mask = 3847,
  CODE_FOR_absv32qi2_mask = 3848,
  CODE_FOR_absv32hi2_mask = 3849,
  CODE_FOR_absv16hi2_mask = 3850,
  CODE_FOR_absv8hi2_mask = 3851,
  CODE_FOR_ssse3_absv8qi2 = 3852,
  CODE_FOR_ssse3_absv4hi2 = 3853,
  CODE_FOR_ssse3_absv2si2 = 3854,
  CODE_FOR_sse4a_movntsf = 3855,
  CODE_FOR_sse4a_movntdf = 3856,
  CODE_FOR_sse4a_vmmovntv4sf = 3857,
  CODE_FOR_sse4a_vmmovntv2df = 3858,
  CODE_FOR_sse4a_extrqi = 3859,
  CODE_FOR_sse4a_extrq = 3860,
  CODE_FOR_sse4a_insertqi = 3861,
  CODE_FOR_sse4a_insertq = 3862,
  CODE_FOR_avx_blendps256 = 3863,
  CODE_FOR_sse4_1_blendps = 3864,
  CODE_FOR_avx_blendpd256 = 3865,
  CODE_FOR_sse4_1_blendpd = 3866,
  CODE_FOR_avx_blendvps256 = 3867,
  CODE_FOR_sse4_1_blendvps = 3868,
  CODE_FOR_avx_blendvpd256 = 3869,
  CODE_FOR_sse4_1_blendvpd = 3870,
  CODE_FOR_avx_dpps256 = 3871,
  CODE_FOR_sse4_1_dpps = 3872,
  CODE_FOR_avx_dppd256 = 3873,
  CODE_FOR_sse4_1_dppd = 3874,
  CODE_FOR_avx512f_movntdqa = 3875,
  CODE_FOR_avx2_movntdqa = 3876,
  CODE_FOR_sse4_1_movntdqa = 3877,
  CODE_FOR_avx2_mpsadbw = 3878,
  CODE_FOR_sse4_1_mpsadbw = 3879,
  CODE_FOR_avx512bw_packusdw = 3880,
  CODE_FOR_avx512bw_packusdw_mask = 3881,
  CODE_FOR_avx2_packusdw = 3882,
  CODE_FOR_avx2_packusdw_mask = 3883,
  CODE_FOR_sse4_1_packusdw = 3884,
  CODE_FOR_sse4_1_packusdw_mask = 3885,
  CODE_FOR_avx2_pblendvb = 3886,
  CODE_FOR_sse4_1_pblendvb = 3887,
  CODE_FOR_sse4_1_pblendw = 3888,
  CODE_FOR_avx2_pblenddv8si = 3890,
  CODE_FOR_avx2_pblenddv4si = 3891,
  CODE_FOR_sse4_1_phminposuw = 3892,
  CODE_FOR_avx2_sign_extendv16qiv16hi2 = 3893,
  CODE_FOR_avx2_sign_extendv16qiv16hi2_mask = 3894,
  CODE_FOR_avx2_zero_extendv16qiv16hi2 = 3895,
  CODE_FOR_avx2_zero_extendv16qiv16hi2_mask = 3896,
  CODE_FOR_avx512bw_sign_extendv32qiv32hi2 = 3897,
  CODE_FOR_avx512bw_sign_extendv32qiv32hi2_mask = 3898,
  CODE_FOR_avx512bw_zero_extendv32qiv32hi2 = 3899,
  CODE_FOR_avx512bw_zero_extendv32qiv32hi2_mask = 3900,
  CODE_FOR_sse4_1_sign_extendv8qiv8hi2 = 3901,
  CODE_FOR_sse4_1_sign_extendv8qiv8hi2_mask = 3902,
  CODE_FOR_sse4_1_zero_extendv8qiv8hi2 = 3903,
  CODE_FOR_sse4_1_zero_extendv8qiv8hi2_mask = 3904,
  CODE_FOR_avx512f_sign_extendv16qiv16si2_mask = 3906,
  CODE_FOR_avx512f_zero_extendv16qiv16si2_mask = 3908,
  CODE_FOR_avx2_sign_extendv8qiv8si2 = 3909,
  CODE_FOR_avx2_sign_extendv8qiv8si2_mask = 3910,
  CODE_FOR_avx2_zero_extendv8qiv8si2 = 3911,
  CODE_FOR_avx2_zero_extendv8qiv8si2_mask = 3912,
  CODE_FOR_sse4_1_sign_extendv4qiv4si2 = 3913,
  CODE_FOR_sse4_1_sign_extendv4qiv4si2_mask = 3914,
  CODE_FOR_sse4_1_zero_extendv4qiv4si2 = 3915,
  CODE_FOR_sse4_1_zero_extendv4qiv4si2_mask = 3916,
  CODE_FOR_avx512f_sign_extendv16hiv16si2 = 3917,
  CODE_FOR_avx512f_sign_extendv16hiv16si2_mask = 3918,
  CODE_FOR_avx512f_zero_extendv16hiv16si2 = 3919,
  CODE_FOR_avx512f_zero_extendv16hiv16si2_mask = 3920,
  CODE_FOR_avx2_sign_extendv8hiv8si2 = 3921,
  CODE_FOR_avx2_sign_extendv8hiv8si2_mask = 3922,
  CODE_FOR_avx2_zero_extendv8hiv8si2 = 3923,
  CODE_FOR_avx2_zero_extendv8hiv8si2_mask = 3924,
  CODE_FOR_sse4_1_sign_extendv4hiv4si2 = 3925,
  CODE_FOR_sse4_1_sign_extendv4hiv4si2_mask = 3926,
  CODE_FOR_sse4_1_zero_extendv4hiv4si2 = 3927,
  CODE_FOR_sse4_1_zero_extendv4hiv4si2_mask = 3928,
  CODE_FOR_avx512f_sign_extendv8qiv8di2 = 3929,
  CODE_FOR_avx512f_sign_extendv8qiv8di2_mask = 3930,
  CODE_FOR_avx512f_zero_extendv8qiv8di2 = 3931,
  CODE_FOR_avx512f_zero_extendv8qiv8di2_mask = 3932,
  CODE_FOR_avx2_sign_extendv4qiv4di2 = 3933,
  CODE_FOR_avx2_sign_extendv4qiv4di2_mask = 3934,
  CODE_FOR_avx2_zero_extendv4qiv4di2 = 3935,
  CODE_FOR_avx2_zero_extendv4qiv4di2_mask = 3936,
  CODE_FOR_sse4_1_sign_extendv2qiv2di2 = 3937,
  CODE_FOR_sse4_1_sign_extendv2qiv2di2_mask = 3938,
  CODE_FOR_sse4_1_zero_extendv2qiv2di2 = 3939,
  CODE_FOR_sse4_1_zero_extendv2qiv2di2_mask = 3940,
  CODE_FOR_avx512f_sign_extendv8hiv8di2 = 3941,
  CODE_FOR_avx512f_sign_extendv8hiv8di2_mask = 3942,
  CODE_FOR_avx512f_zero_extendv8hiv8di2 = 3943,
  CODE_FOR_avx512f_zero_extendv8hiv8di2_mask = 3944,
  CODE_FOR_avx2_sign_extendv4hiv4di2 = 3945,
  CODE_FOR_avx2_sign_extendv4hiv4di2_mask = 3946,
  CODE_FOR_avx2_zero_extendv4hiv4di2 = 3947,
  CODE_FOR_avx2_zero_extendv4hiv4di2_mask = 3948,
  CODE_FOR_sse4_1_sign_extendv2hiv2di2 = 3949,
  CODE_FOR_sse4_1_sign_extendv2hiv2di2_mask = 3950,
  CODE_FOR_sse4_1_zero_extendv2hiv2di2 = 3951,
  CODE_FOR_sse4_1_zero_extendv2hiv2di2_mask = 3952,
  CODE_FOR_avx512f_sign_extendv8siv8di2 = 3953,
  CODE_FOR_avx512f_sign_extendv8siv8di2_mask = 3954,
  CODE_FOR_avx512f_zero_extendv8siv8di2 = 3955,
  CODE_FOR_avx512f_zero_extendv8siv8di2_mask = 3956,
  CODE_FOR_avx2_sign_extendv4siv4di2 = 3957,
  CODE_FOR_avx2_sign_extendv4siv4di2_mask = 3958,
  CODE_FOR_avx2_zero_extendv4siv4di2 = 3959,
  CODE_FOR_avx2_zero_extendv4siv4di2_mask = 3960,
  CODE_FOR_sse4_1_sign_extendv2siv2di2 = 3961,
  CODE_FOR_sse4_1_sign_extendv2siv2di2_mask = 3962,
  CODE_FOR_sse4_1_zero_extendv2siv2di2 = 3963,
  CODE_FOR_sse4_1_zero_extendv2siv2di2_mask = 3964,
  CODE_FOR_avx_vtestps256 = 3965,
  CODE_FOR_avx_vtestps = 3966,
  CODE_FOR_avx_vtestpd256 = 3967,
  CODE_FOR_avx_vtestpd = 3968,
  CODE_FOR_sse4_1_ptestv16qi = 3969,
  CODE_FOR_sse4_1_ptestv8hi = 3970,
  CODE_FOR_sse4_1_ptestv4si = 3971,
  CODE_FOR_sse4_1_ptestv2di = 3972,
  CODE_FOR_sse4_1_ptestv4sf = 3973,
  CODE_FOR_sse4_1_ptestv2df = 3974,
  CODE_FOR_avx_ptestv32qi = 3975,
  CODE_FOR_avx_ptestv16hi = 3976,
  CODE_FOR_avx_ptestv8si = 3977,
  CODE_FOR_avx_ptestv4di = 3978,
  CODE_FOR_avx_ptestv8sf = 3979,
  CODE_FOR_avx_ptestv4df = 3980,
  CODE_FOR_ptesttf2 = 3981,
  CODE_FOR_avx_roundps256 = 3982,
  CODE_FOR_sse4_1_roundps = 3983,
  CODE_FOR_avx_roundpd256 = 3984,
  CODE_FOR_sse4_1_roundpd = 3985,
  CODE_FOR_sse4_1_roundss = 3986,
  CODE_FOR_sse4_1_roundsd = 3987,
  CODE_FOR_sse4_2_pcmpestr = 3988,
  CODE_FOR_sse4_2_pcmpestri = 3989,
  CODE_FOR_sse4_2_pcmpestrm = 3990,
  CODE_FOR_sse4_2_pcmpestr_cconly = 3991,
  CODE_FOR_sse4_2_pcmpistr = 3992,
  CODE_FOR_sse4_2_pcmpistri = 3993,
  CODE_FOR_sse4_2_pcmpistrm = 3994,
  CODE_FOR_sse4_2_pcmpistr_cconly = 3995,
  CODE_FOR_avx512er_exp2v16sf = 4012,
  CODE_FOR_avx512er_exp2v16sf_round = 4013,
  CODE_FOR_avx512er_exp2v16sf_mask = 4014,
  CODE_FOR_avx512er_exp2v16sf_mask_round = 4015,
  CODE_FOR_avx512er_exp2v8df = 4016,
  CODE_FOR_avx512er_exp2v8df_round = 4017,
  CODE_FOR_avx512er_exp2v8df_mask = 4018,
  CODE_FOR_avx512er_exp2v8df_mask_round = 4019,
  CODE_FOR_avx512er_rcp28v16sf_mask = 4022,
  CODE_FOR_avx512er_rcp28v16sf_mask_round = 4023,
  CODE_FOR_avx512er_rcp28v8df_mask = 4026,
  CODE_FOR_avx512er_rcp28v8df_mask_round = 4027,
  CODE_FOR_avx512er_vmrcp28v4sf = 4028,
  CODE_FOR_avx512er_vmrcp28v4sf_round = 4029,
  CODE_FOR_avx512er_vmrcp28v2df = 4030,
  CODE_FOR_avx512er_vmrcp28v2df_round = 4031,
  CODE_FOR_avx512er_rsqrt28v16sf_mask = 4034,
  CODE_FOR_avx512er_rsqrt28v16sf_mask_round = 4035,
  CODE_FOR_avx512er_rsqrt28v8df_mask = 4038,
  CODE_FOR_avx512er_rsqrt28v8df_mask_round = 4039,
  CODE_FOR_avx512er_vmrsqrt28v4sf = 4040,
  CODE_FOR_avx512er_vmrsqrt28v4sf_round = 4041,
  CODE_FOR_avx512er_vmrsqrt28v2df = 4042,
  CODE_FOR_avx512er_vmrsqrt28v2df_round = 4043,
  CODE_FOR_xop_pmacsww = 4044,
  CODE_FOR_xop_pmacssww = 4045,
  CODE_FOR_xop_pmacsdd = 4046,
  CODE_FOR_xop_pmacssdd = 4047,
  CODE_FOR_xop_pmacsdql = 4048,
  CODE_FOR_xop_pmacssdql = 4049,
  CODE_FOR_xop_pmacsdqh = 4050,
  CODE_FOR_xop_pmacssdqh = 4051,
  CODE_FOR_xop_pmacswd = 4052,
  CODE_FOR_xop_pmacsswd = 4053,
  CODE_FOR_xop_pmadcswd = 4054,
  CODE_FOR_xop_pmadcsswd = 4055,
  CODE_FOR_xop_pcmov_v32qi256 = 4056,
  CODE_FOR_xop_pcmov_v16qi = 4057,
  CODE_FOR_xop_pcmov_v16hi256 = 4058,
  CODE_FOR_xop_pcmov_v8hi = 4059,
  CODE_FOR_xop_pcmov_v8si256 = 4060,
  CODE_FOR_xop_pcmov_v4si = 4061,
  CODE_FOR_xop_pcmov_v4di256 = 4062,
  CODE_FOR_xop_pcmov_v2di = 4063,
  CODE_FOR_xop_pcmov_v8sf256 = 4064,
  CODE_FOR_xop_pcmov_v4sf = 4065,
  CODE_FOR_xop_pcmov_v4df256 = 4066,
  CODE_FOR_xop_pcmov_v2df = 4067,
  CODE_FOR_xop_phaddbw = 4068,
  CODE_FOR_xop_phaddubw = 4069,
  CODE_FOR_xop_phaddbd = 4070,
  CODE_FOR_xop_phaddubd = 4071,
  CODE_FOR_xop_phaddbq = 4072,
  CODE_FOR_xop_phaddubq = 4073,
  CODE_FOR_xop_phaddwd = 4074,
  CODE_FOR_xop_phadduwd = 4075,
  CODE_FOR_xop_phaddwq = 4076,
  CODE_FOR_xop_phadduwq = 4077,
  CODE_FOR_xop_phadddq = 4078,
  CODE_FOR_xop_phaddudq = 4079,
  CODE_FOR_xop_phsubbw = 4080,
  CODE_FOR_xop_phsubwd = 4081,
  CODE_FOR_xop_phsubdq = 4082,
  CODE_FOR_xop_pperm = 4083,
  CODE_FOR_xop_pperm_pack_v2di_v4si = 4084,
  CODE_FOR_xop_pperm_pack_v4si_v8hi = 4085,
  CODE_FOR_xop_pperm_pack_v8hi_v16qi = 4086,
  CODE_FOR_xop_rotlv16qi3 = 4087,
  CODE_FOR_xop_rotlv8hi3 = 4088,
  CODE_FOR_xop_rotlv4si3 = 4089,
  CODE_FOR_xop_rotlv2di3 = 4090,
  CODE_FOR_xop_rotrv16qi3 = 4091,
  CODE_FOR_xop_rotrv8hi3 = 4092,
  CODE_FOR_xop_rotrv4si3 = 4093,
  CODE_FOR_xop_rotrv2di3 = 4094,
  CODE_FOR_xop_vrotlv16qi3 = 4095,
  CODE_FOR_xop_vrotlv8hi3 = 4096,
  CODE_FOR_xop_vrotlv4si3 = 4097,
  CODE_FOR_xop_vrotlv2di3 = 4098,
  CODE_FOR_xop_shav16qi3 = 4099,
  CODE_FOR_xop_shav8hi3 = 4100,
  CODE_FOR_xop_shav4si3 = 4101,
  CODE_FOR_xop_shav2di3 = 4102,
  CODE_FOR_xop_shlv16qi3 = 4103,
  CODE_FOR_xop_shlv8hi3 = 4104,
  CODE_FOR_xop_shlv4si3 = 4105,
  CODE_FOR_xop_shlv2di3 = 4106,
  CODE_FOR_xop_frczsf2 = 4107,
  CODE_FOR_xop_frczdf2 = 4108,
  CODE_FOR_xop_frczv4sf2 = 4109,
  CODE_FOR_xop_frczv2df2 = 4110,
  CODE_FOR_xop_frczv8sf2 = 4111,
  CODE_FOR_xop_frczv4df2 = 4112,
  CODE_FOR_xop_maskcmpv16qi3 = 4115,
  CODE_FOR_xop_maskcmpv8hi3 = 4116,
  CODE_FOR_xop_maskcmpv4si3 = 4117,
  CODE_FOR_xop_maskcmpv2di3 = 4118,
  CODE_FOR_xop_maskcmp_unsv16qi3 = 4119,
  CODE_FOR_xop_maskcmp_unsv8hi3 = 4120,
  CODE_FOR_xop_maskcmp_unsv4si3 = 4121,
  CODE_FOR_xop_maskcmp_unsv2di3 = 4122,
  CODE_FOR_xop_maskcmp_uns2v16qi3 = 4123,
  CODE_FOR_xop_maskcmp_uns2v8hi3 = 4124,
  CODE_FOR_xop_maskcmp_uns2v4si3 = 4125,
  CODE_FOR_xop_maskcmp_uns2v2di3 = 4126,
  CODE_FOR_xop_pcom_tfv16qi3 = 4127,
  CODE_FOR_xop_pcom_tfv8hi3 = 4128,
  CODE_FOR_xop_pcom_tfv4si3 = 4129,
  CODE_FOR_xop_pcom_tfv2di3 = 4130,
  CODE_FOR_xop_vpermil2v8sf3 = 4131,
  CODE_FOR_xop_vpermil2v4sf3 = 4132,
  CODE_FOR_xop_vpermil2v4df3 = 4133,
  CODE_FOR_xop_vpermil2v2df3 = 4134,
  CODE_FOR_aesenc = 4135,
  CODE_FOR_aesenclast = 4136,
  CODE_FOR_aesdec = 4137,
  CODE_FOR_aesdeclast = 4138,
  CODE_FOR_aesimc = 4139,
  CODE_FOR_aeskeygenassist = 4140,
  CODE_FOR_pclmulqdq = 4141,
  CODE_FOR_avx_vzeroupper = 4143,
  CODE_FOR_avx2_pbroadcastv16si = 4144,
  CODE_FOR_avx2_pbroadcastv8di = 4145,
  CODE_FOR_avx2_pbroadcastv64qi = 4146,
  CODE_FOR_avx2_pbroadcastv32qi = 4147,
  CODE_FOR_avx2_pbroadcastv16qi = 4148,
  CODE_FOR_avx2_pbroadcastv32hi = 4149,
  CODE_FOR_avx2_pbroadcastv16hi = 4150,
  CODE_FOR_avx2_pbroadcastv8hi = 4151,
  CODE_FOR_avx2_pbroadcastv8si = 4152,
  CODE_FOR_avx2_pbroadcastv4si = 4153,
  CODE_FOR_avx2_pbroadcastv4di = 4154,
  CODE_FOR_avx2_pbroadcastv2di = 4155,
  CODE_FOR_avx2_pbroadcastv32qi_1 = 4156,
  CODE_FOR_avx2_pbroadcastv16hi_1 = 4157,
  CODE_FOR_avx2_pbroadcastv8si_1 = 4158,
  CODE_FOR_avx2_pbroadcastv4di_1 = 4159,
  CODE_FOR_avx2_permvarv8si = 4160,
  CODE_FOR_avx2_permvarv8si_mask = 4161,
  CODE_FOR_avx2_permvarv8sf = 4162,
  CODE_FOR_avx2_permvarv8sf_mask = 4163,
  CODE_FOR_avx512f_permvarv16si = 4164,
  CODE_FOR_avx512f_permvarv16si_mask = 4165,
  CODE_FOR_avx512f_permvarv16sf = 4166,
  CODE_FOR_avx512f_permvarv16sf_mask = 4167,
  CODE_FOR_avx512f_permvarv8di = 4168,
  CODE_FOR_avx512f_permvarv8di_mask = 4169,
  CODE_FOR_avx512f_permvarv8df = 4170,
  CODE_FOR_avx512f_permvarv8df_mask = 4171,
  CODE_FOR_avx2_permvarv4di = 4172,
  CODE_FOR_avx2_permvarv4di_mask = 4173,
  CODE_FOR_avx2_permvarv4df = 4174,
  CODE_FOR_avx2_permvarv4df_mask = 4175,
  CODE_FOR_avx512bw_permvarv64qi = 4176,
  CODE_FOR_avx512bw_permvarv64qi_mask = 4177,
  CODE_FOR_avx512vl_permvarv16qi = 4178,
  CODE_FOR_avx512vl_permvarv16qi_mask = 4179,
  CODE_FOR_avx512vl_permvarv32qi = 4180,
  CODE_FOR_avx512vl_permvarv32qi_mask = 4181,
  CODE_FOR_avx512vl_permvarv8hi = 4182,
  CODE_FOR_avx512vl_permvarv8hi_mask = 4183,
  CODE_FOR_avx512vl_permvarv16hi = 4184,
  CODE_FOR_avx512vl_permvarv16hi_mask = 4185,
  CODE_FOR_avx512bw_permvarv32hi = 4186,
  CODE_FOR_avx512bw_permvarv32hi_mask = 4187,
  CODE_FOR_avx2_permv4di_1 = 4188,
  CODE_FOR_avx2_permv4di_1_mask = 4189,
  CODE_FOR_avx2_permv4df_1 = 4190,
  CODE_FOR_avx2_permv4df_1_mask = 4191,
  CODE_FOR_avx512f_permv8df_1 = 4192,
  CODE_FOR_avx512f_permv8df_1_mask = 4193,
  CODE_FOR_avx512f_permv8di_1 = 4194,
  CODE_FOR_avx512f_permv8di_1_mask = 4195,
  CODE_FOR_avx2_permv2ti = 4196,
  CODE_FOR_avx2_vec_dupv4df = 4197,
  CODE_FOR_avx512f_vec_dupv16si_1 = 4198,
  CODE_FOR_avx512f_vec_dupv8di_1 = 4199,
  CODE_FOR_avx512bw_vec_dupv32hi_1 = 4200,
  CODE_FOR_avx512bw_vec_dupv64qi_1 = 4201,
  CODE_FOR_avx512f_vec_dupv16si = 4202,
  CODE_FOR_avx512f_vec_dupv16si_mask = 4203,
  CODE_FOR_avx512vl_vec_dupv8si = 4204,
  CODE_FOR_avx512vl_vec_dupv8si_mask = 4205,
  CODE_FOR_avx512vl_vec_dupv4si = 4206,
  CODE_FOR_avx512vl_vec_dupv4si_mask = 4207,
  CODE_FOR_avx512f_vec_dupv8di = 4208,
  CODE_FOR_avx512f_vec_dupv8di_mask = 4209,
  CODE_FOR_avx512vl_vec_dupv4di = 4210,
  CODE_FOR_avx512vl_vec_dupv4di_mask = 4211,
  CODE_FOR_avx512vl_vec_dupv2di = 4212,
  CODE_FOR_avx512vl_vec_dupv2di_mask = 4213,
  CODE_FOR_avx512f_vec_dupv16sf = 4214,
  CODE_FOR_avx512f_vec_dupv16sf_mask = 4215,
  CODE_FOR_avx512vl_vec_dupv8sf = 4216,
  CODE_FOR_avx512vl_vec_dupv8sf_mask = 4217,
  CODE_FOR_avx512vl_vec_dupv4sf = 4218,
  CODE_FOR_avx512vl_vec_dupv4sf_mask = 4219,
  CODE_FOR_avx512f_vec_dupv8df = 4220,
  CODE_FOR_avx512f_vec_dupv8df_mask = 4221,
  CODE_FOR_avx512vl_vec_dupv4df = 4222,
  CODE_FOR_avx512vl_vec_dupv4df_mask = 4223,
  CODE_FOR_avx512vl_vec_dupv2df = 4224,
  CODE_FOR_avx512vl_vec_dupv2df_mask = 4225,
  CODE_FOR_avx512bw_vec_dupv64qi = 4226,
  CODE_FOR_avx512bw_vec_dupv64qi_mask = 4227,
  CODE_FOR_avx512vl_vec_dupv16qi = 4228,
  CODE_FOR_avx512vl_vec_dupv16qi_mask = 4229,
  CODE_FOR_avx512vl_vec_dupv32qi = 4230,
  CODE_FOR_avx512vl_vec_dupv32qi_mask = 4231,
  CODE_FOR_avx512bw_vec_dupv32hi = 4232,
  CODE_FOR_avx512bw_vec_dupv32hi_mask = 4233,
  CODE_FOR_avx512vl_vec_dupv16hi = 4234,
  CODE_FOR_avx512vl_vec_dupv16hi_mask = 4235,
  CODE_FOR_avx512vl_vec_dupv8hi = 4236,
  CODE_FOR_avx512vl_vec_dupv8hi_mask = 4237,
  CODE_FOR_avx512f_broadcastv16sf_mask = 4239,
  CODE_FOR_avx512f_broadcastv16si_mask = 4241,
  CODE_FOR_avx512f_broadcastv8df_mask = 4243,
  CODE_FOR_avx512f_broadcastv8di_mask = 4245,
  CODE_FOR_avx512bw_vec_dup_gprv64qi_mask = 4247,
  CODE_FOR_avx512vl_vec_dup_gprv16qi_mask = 4249,
  CODE_FOR_avx512vl_vec_dup_gprv32qi_mask = 4251,
  CODE_FOR_avx512bw_vec_dup_gprv32hi_mask = 4253,
  CODE_FOR_avx512vl_vec_dup_gprv16hi_mask = 4255,
  CODE_FOR_avx512vl_vec_dup_gprv8hi_mask = 4257,
  CODE_FOR_avx512f_vec_dup_gprv16si_mask = 4259,
  CODE_FOR_avx512vl_vec_dup_gprv8si_mask = 4261,
  CODE_FOR_avx512vl_vec_dup_gprv4si_mask = 4263,
  CODE_FOR_avx512f_vec_dup_gprv8di_mask = 4265,
  CODE_FOR_avx512vl_vec_dup_gprv4di_mask = 4267,
  CODE_FOR_avx512vl_vec_dup_gprv2di_mask = 4269,
  CODE_FOR_avx512f_vec_dup_gprv16sf_mask = 4271,
  CODE_FOR_avx512vl_vec_dup_gprv8sf_mask = 4273,
  CODE_FOR_avx512vl_vec_dup_gprv4sf_mask = 4275,
  CODE_FOR_avx512f_vec_dup_gprv8df_mask = 4277,
  CODE_FOR_avx512vl_vec_dup_gprv4df_mask = 4279,
  CODE_FOR_avx512vl_vec_dup_gprv2df_mask = 4281,
  CODE_FOR_vec_dupv4sf = 4282,
  CODE_FOR_avx2_vbroadcasti128_v32qi = 4285,
  CODE_FOR_avx2_vbroadcasti128_v16hi = 4286,
  CODE_FOR_avx2_vbroadcasti128_v8si = 4287,
  CODE_FOR_avx2_vbroadcasti128_v4di = 4288,
  CODE_FOR_vec_dupv8si = 4295,
  CODE_FOR_vec_dupv8sf = 4296,
  CODE_FOR_vec_dupv4di = 4297,
  CODE_FOR_vec_dupv4df = 4298,
  CODE_FOR_avx_vbroadcastf128_v32qi = 4299,
  CODE_FOR_avx_vbroadcastf128_v16hi = 4300,
  CODE_FOR_avx_vbroadcastf128_v8si = 4301,
  CODE_FOR_avx_vbroadcastf128_v4di = 4302,
  CODE_FOR_avx_vbroadcastf128_v8sf = 4303,
  CODE_FOR_avx_vbroadcastf128_v4df = 4304,
  CODE_FOR_avx512dq_broadcastv16si_mask = 4306,
  CODE_FOR_avx512dq_broadcastv8si_mask = 4308,
  CODE_FOR_avx512dq_broadcastv4si_mask = 4310,
  CODE_FOR_avx512dq_broadcastv16sf_mask = 4312,
  CODE_FOR_avx512dq_broadcastv8sf_mask = 4314,
  CODE_FOR_avx512vl_broadcastv8si_mask_1 = 4316,
  CODE_FOR_avx512vl_broadcastv8sf_mask_1 = 4318,
  CODE_FOR_avx512dq_broadcastv16sf_mask_1 = 4320,
  CODE_FOR_avx512dq_broadcastv16si_mask_1 = 4322,
  CODE_FOR_avx512dq_broadcastv8di_mask_1 = 4324,
  CODE_FOR_avx512dq_broadcastv8df_mask_1 = 4326,
  CODE_FOR_avx512dq_broadcastv4di_mask_1 = 4328,
  CODE_FOR_avx512dq_broadcastv4df_mask_1 = 4330,
  CODE_FOR_avx512cd_maskb_vec_dupv8di = 4331,
  CODE_FOR_avx512cd_maskb_vec_dupv4di = 4332,
  CODE_FOR_avx512cd_maskb_vec_dupv2di = 4333,
  CODE_FOR_avx512cd_maskw_vec_dupv16si = 4334,
  CODE_FOR_avx512cd_maskw_vec_dupv8si = 4335,
  CODE_FOR_avx512cd_maskw_vec_dupv4si = 4336,
  CODE_FOR_avx512f_vpermilvarv16sf3 = 4352,
  CODE_FOR_avx512f_vpermilvarv16sf3_mask = 4353,
  CODE_FOR_avx_vpermilvarv8sf3 = 4354,
  CODE_FOR_avx_vpermilvarv8sf3_mask = 4355,
  CODE_FOR_avx_vpermilvarv4sf3 = 4356,
  CODE_FOR_avx_vpermilvarv4sf3_mask = 4357,
  CODE_FOR_avx512f_vpermilvarv8df3 = 4358,
  CODE_FOR_avx512f_vpermilvarv8df3_mask = 4359,
  CODE_FOR_avx_vpermilvarv4df3 = 4360,
  CODE_FOR_avx_vpermilvarv4df3_mask = 4361,
  CODE_FOR_avx_vpermilvarv2df3 = 4362,
  CODE_FOR_avx_vpermilvarv2df3_mask = 4363,
  CODE_FOR_avx512f_vpermt2varv16si3 = 4382,
  CODE_FOR_avx512f_vpermt2varv16si3_maskz_1 = 4383,
  CODE_FOR_avx512f_vpermt2varv16sf3 = 4384,
  CODE_FOR_avx512f_vpermt2varv16sf3_maskz_1 = 4385,
  CODE_FOR_avx512f_vpermt2varv8di3 = 4386,
  CODE_FOR_avx512f_vpermt2varv8di3_maskz_1 = 4387,
  CODE_FOR_avx512f_vpermt2varv8df3 = 4388,
  CODE_FOR_avx512f_vpermt2varv8df3_maskz_1 = 4389,
  CODE_FOR_avx512vl_vpermt2varv8si3 = 4390,
  CODE_FOR_avx512vl_vpermt2varv8si3_maskz_1 = 4391,
  CODE_FOR_avx512vl_vpermt2varv8sf3 = 4392,
  CODE_FOR_avx512vl_vpermt2varv8sf3_maskz_1 = 4393,
  CODE_FOR_avx512vl_vpermt2varv4di3 = 4394,
  CODE_FOR_avx512vl_vpermt2varv4di3_maskz_1 = 4395,
  CODE_FOR_avx512vl_vpermt2varv4df3 = 4396,
  CODE_FOR_avx512vl_vpermt2varv4df3_maskz_1 = 4397,
  CODE_FOR_avx512vl_vpermt2varv4si3 = 4398,
  CODE_FOR_avx512vl_vpermt2varv4si3_maskz_1 = 4399,
  CODE_FOR_avx512vl_vpermt2varv4sf3 = 4400,
  CODE_FOR_avx512vl_vpermt2varv4sf3_maskz_1 = 4401,
  CODE_FOR_avx512vl_vpermt2varv2di3 = 4402,
  CODE_FOR_avx512vl_vpermt2varv2di3_maskz_1 = 4403,
  CODE_FOR_avx512vl_vpermt2varv2df3 = 4404,
  CODE_FOR_avx512vl_vpermt2varv2df3_maskz_1 = 4405,
  CODE_FOR_avx512bw_vpermt2varv32hi3 = 4406,
  CODE_FOR_avx512bw_vpermt2varv32hi3_maskz_1 = 4407,
  CODE_FOR_avx512vl_vpermt2varv16hi3 = 4408,
  CODE_FOR_avx512vl_vpermt2varv16hi3_maskz_1 = 4409,
  CODE_FOR_avx512vl_vpermt2varv8hi3 = 4410,
  CODE_FOR_avx512vl_vpermt2varv8hi3_maskz_1 = 4411,
  CODE_FOR_avx512bw_vpermt2varv64qi3 = 4412,
  CODE_FOR_avx512bw_vpermt2varv64qi3_maskz_1 = 4413,
  CODE_FOR_avx512vl_vpermt2varv32qi3 = 4414,
  CODE_FOR_avx512vl_vpermt2varv32qi3_maskz_1 = 4415,
  CODE_FOR_avx512vl_vpermt2varv16qi3 = 4416,
  CODE_FOR_avx512vl_vpermt2varv16qi3_maskz_1 = 4417,
  CODE_FOR_avx512f_vpermt2varv16si3_mask = 4418,
  CODE_FOR_avx512f_vpermt2varv16sf3_mask = 4419,
  CODE_FOR_avx512f_vpermt2varv8di3_mask = 4420,
  CODE_FOR_avx512f_vpermt2varv8df3_mask = 4421,
  CODE_FOR_avx512vl_vpermt2varv8si3_mask = 4422,
  CODE_FOR_avx512vl_vpermt2varv8sf3_mask = 4423,
  CODE_FOR_avx512vl_vpermt2varv4di3_mask = 4424,
  CODE_FOR_avx512vl_vpermt2varv4df3_mask = 4425,
  CODE_FOR_avx512vl_vpermt2varv4si3_mask = 4426,
  CODE_FOR_avx512vl_vpermt2varv4sf3_mask = 4427,
  CODE_FOR_avx512vl_vpermt2varv2di3_mask = 4428,
  CODE_FOR_avx512vl_vpermt2varv2df3_mask = 4429,
  CODE_FOR_avx512bw_vpermt2varv32hi3_mask = 4430,
  CODE_FOR_avx512vl_vpermt2varv16hi3_mask = 4431,
  CODE_FOR_avx512vl_vpermt2varv8hi3_mask = 4432,
  CODE_FOR_avx512bw_vpermt2varv64qi3_mask = 4433,
  CODE_FOR_avx512vl_vpermt2varv32qi3_mask = 4434,
  CODE_FOR_avx512vl_vpermt2varv16qi3_mask = 4435,
  CODE_FOR_vec_set_lo_v4di = 4448,
  CODE_FOR_vec_set_lo_v4di_mask = 4449,
  CODE_FOR_vec_set_lo_v4df = 4450,
  CODE_FOR_vec_set_lo_v4df_mask = 4451,
  CODE_FOR_vec_set_hi_v4di = 4452,
  CODE_FOR_vec_set_hi_v4di_mask = 4453,
  CODE_FOR_vec_set_hi_v4df = 4454,
  CODE_FOR_vec_set_hi_v4df_mask = 4455,
  CODE_FOR_vec_set_lo_v8si = 4456,
  CODE_FOR_vec_set_lo_v8si_mask = 4457,
  CODE_FOR_vec_set_lo_v8sf = 4458,
  CODE_FOR_vec_set_lo_v8sf_mask = 4459,
  CODE_FOR_vec_set_hi_v8si = 4460,
  CODE_FOR_vec_set_hi_v8si_mask = 4461,
  CODE_FOR_vec_set_hi_v8sf = 4462,
  CODE_FOR_vec_set_hi_v8sf_mask = 4463,
  CODE_FOR_vec_set_lo_v16hi = 4464,
  CODE_FOR_vec_set_hi_v16hi = 4465,
  CODE_FOR_vec_set_lo_v32qi = 4466,
  CODE_FOR_vec_set_hi_v32qi = 4467,
  CODE_FOR_avx_maskloadps = 4468,
  CODE_FOR_avx_maskloadpd = 4469,
  CODE_FOR_avx_maskloadps256 = 4470,
  CODE_FOR_avx_maskloadpd256 = 4471,
  CODE_FOR_avx2_maskloadd = 4472,
  CODE_FOR_avx2_maskloadq = 4473,
  CODE_FOR_avx2_maskloadd256 = 4474,
  CODE_FOR_avx2_maskloadq256 = 4475,
  CODE_FOR_avx_maskstoreps = 4476,
  CODE_FOR_avx_maskstorepd = 4477,
  CODE_FOR_avx_maskstoreps256 = 4478,
  CODE_FOR_avx_maskstorepd256 = 4479,
  CODE_FOR_avx2_maskstored = 4480,
  CODE_FOR_avx2_maskstoreq = 4481,
  CODE_FOR_avx2_maskstored256 = 4482,
  CODE_FOR_avx2_maskstoreq256 = 4483,
  CODE_FOR_avx_si256_si = 4484,
  CODE_FOR_avx_ps256_ps = 4485,
  CODE_FOR_avx_pd256_pd = 4486,
  CODE_FOR_avx2_ashrvv4si = 4487,
  CODE_FOR_avx2_ashrvv4si_mask = 4488,
  CODE_FOR_avx2_ashrvv8si = 4489,
  CODE_FOR_avx2_ashrvv8si_mask = 4490,
  CODE_FOR_avx512f_ashrvv16si = 4491,
  CODE_FOR_avx512f_ashrvv16si_mask = 4492,
  CODE_FOR_avx2_ashrvv2di = 4493,
  CODE_FOR_avx2_ashrvv2di_mask = 4494,
  CODE_FOR_avx2_ashrvv4di = 4495,
  CODE_FOR_avx2_ashrvv4di_mask = 4496,
  CODE_FOR_avx512f_ashrvv8di = 4497,
  CODE_FOR_avx512f_ashrvv8di_mask = 4498,
  CODE_FOR_avx512vl_ashrvv8hi = 4499,
  CODE_FOR_avx512vl_ashrvv8hi_mask = 4500,
  CODE_FOR_avx512vl_ashrvv16hi = 4501,
  CODE_FOR_avx512vl_ashrvv16hi_mask = 4502,
  CODE_FOR_avx512bw_ashrvv32hi = 4503,
  CODE_FOR_avx512bw_ashrvv32hi_mask = 4504,
  CODE_FOR_avx512f_ashlvv16si = 4505,
  CODE_FOR_avx512f_ashlvv16si_mask = 4506,
  CODE_FOR_avx512f_lshrvv16si = 4507,
  CODE_FOR_avx512f_lshrvv16si_mask = 4508,
  CODE_FOR_avx2_ashlvv8si = 4509,
  CODE_FOR_avx2_ashlvv8si_mask = 4510,
  CODE_FOR_avx2_lshrvv8si = 4511,
  CODE_FOR_avx2_lshrvv8si_mask = 4512,
  CODE_FOR_avx2_ashlvv4si = 4513,
  CODE_FOR_avx2_ashlvv4si_mask = 4514,
  CODE_FOR_avx2_lshrvv4si = 4515,
  CODE_FOR_avx2_lshrvv4si_mask = 4516,
  CODE_FOR_avx512f_ashlvv8di = 4517,
  CODE_FOR_avx512f_ashlvv8di_mask = 4518,
  CODE_FOR_avx512f_lshrvv8di = 4519,
  CODE_FOR_avx512f_lshrvv8di_mask = 4520,
  CODE_FOR_avx2_ashlvv4di = 4521,
  CODE_FOR_avx2_ashlvv4di_mask = 4522,
  CODE_FOR_avx2_lshrvv4di = 4523,
  CODE_FOR_avx2_lshrvv4di_mask = 4524,
  CODE_FOR_avx2_ashlvv2di = 4525,
  CODE_FOR_avx2_ashlvv2di_mask = 4526,
  CODE_FOR_avx2_lshrvv2di = 4527,
  CODE_FOR_avx2_lshrvv2di_mask = 4528,
  CODE_FOR_avx512vl_ashlvv8hi = 4529,
  CODE_FOR_avx512vl_ashlvv8hi_mask = 4530,
  CODE_FOR_avx512vl_lshrvv8hi = 4531,
  CODE_FOR_avx512vl_lshrvv8hi_mask = 4532,
  CODE_FOR_avx512vl_ashlvv16hi = 4533,
  CODE_FOR_avx512vl_ashlvv16hi_mask = 4534,
  CODE_FOR_avx512vl_lshrvv16hi = 4535,
  CODE_FOR_avx512vl_lshrvv16hi_mask = 4536,
  CODE_FOR_avx512bw_ashlvv32hi = 4537,
  CODE_FOR_avx512bw_ashlvv32hi_mask = 4538,
  CODE_FOR_avx512bw_lshrvv32hi = 4539,
  CODE_FOR_avx512bw_lshrvv32hi_mask = 4540,
  CODE_FOR_avx_vec_concatv32qi = 4541,
  CODE_FOR_avx_vec_concatv16hi = 4542,
  CODE_FOR_avx_vec_concatv8si = 4543,
  CODE_FOR_avx_vec_concatv4di = 4544,
  CODE_FOR_avx_vec_concatv8sf = 4545,
  CODE_FOR_avx_vec_concatv4df = 4546,
  CODE_FOR_avx_vec_concatv64qi = 4547,
  CODE_FOR_avx_vec_concatv32hi = 4548,
  CODE_FOR_avx_vec_concatv16si = 4549,
  CODE_FOR_avx_vec_concatv8di = 4550,
  CODE_FOR_avx_vec_concatv16sf = 4551,
  CODE_FOR_avx_vec_concatv8df = 4552,
  CODE_FOR_vcvtph2ps = 4553,
  CODE_FOR_vcvtph2ps_mask = 4554,
  CODE_FOR_vcvtph2ps256 = 4557,
  CODE_FOR_vcvtph2ps256_mask = 4558,
  CODE_FOR_avx512f_vcvtph2ps512_mask = 4561,
  CODE_FOR_avx512f_vcvtph2ps512_mask_round = 4562,
  CODE_FOR_vcvtps2ph256 = 4566,
  CODE_FOR_vcvtps2ph256_mask = 4567,
  CODE_FOR_avx512f_vcvtps2ph512_mask = 4569,
  CODE_FOR_avx512f_compressv16si_mask = 4786,
  CODE_FOR_avx512f_compressv16sf_mask = 4787,
  CODE_FOR_avx512f_compressv8di_mask = 4788,
  CODE_FOR_avx512f_compressv8df_mask = 4789,
  CODE_FOR_avx512vl_compressv8si_mask = 4790,
  CODE_FOR_avx512vl_compressv8sf_mask = 4791,
  CODE_FOR_avx512vl_compressv4di_mask = 4792,
  CODE_FOR_avx512vl_compressv4df_mask = 4793,
  CODE_FOR_avx512vl_compressv4si_mask = 4794,
  CODE_FOR_avx512vl_compressv4sf_mask = 4795,
  CODE_FOR_avx512vl_compressv2di_mask = 4796,
  CODE_FOR_avx512vl_compressv2df_mask = 4797,
  CODE_FOR_compressv64qi_mask = 4798,
  CODE_FOR_compressv16qi_mask = 4799,
  CODE_FOR_compressv32qi_mask = 4800,
  CODE_FOR_compressv32hi_mask = 4801,
  CODE_FOR_compressv16hi_mask = 4802,
  CODE_FOR_compressv8hi_mask = 4803,
  CODE_FOR_avx512f_compressstorev16si_mask = 4804,
  CODE_FOR_avx512f_compressstorev16sf_mask = 4805,
  CODE_FOR_avx512f_compressstorev8di_mask = 4806,
  CODE_FOR_avx512f_compressstorev8df_mask = 4807,
  CODE_FOR_avx512vl_compressstorev8si_mask = 4808,
  CODE_FOR_avx512vl_compressstorev8sf_mask = 4809,
  CODE_FOR_avx512vl_compressstorev4di_mask = 4810,
  CODE_FOR_avx512vl_compressstorev4df_mask = 4811,
  CODE_FOR_avx512vl_compressstorev4si_mask = 4812,
  CODE_FOR_avx512vl_compressstorev4sf_mask = 4813,
  CODE_FOR_avx512vl_compressstorev2di_mask = 4814,
  CODE_FOR_avx512vl_compressstorev2df_mask = 4815,
  CODE_FOR_compressstorev64qi_mask = 4816,
  CODE_FOR_compressstorev16qi_mask = 4817,
  CODE_FOR_compressstorev32qi_mask = 4818,
  CODE_FOR_compressstorev32hi_mask = 4819,
  CODE_FOR_compressstorev16hi_mask = 4820,
  CODE_FOR_compressstorev8hi_mask = 4821,
  CODE_FOR_avx512f_expandv16si_mask = 4822,
  CODE_FOR_avx512f_expandv16sf_mask = 4823,
  CODE_FOR_avx512f_expandv8di_mask = 4824,
  CODE_FOR_avx512f_expandv8df_mask = 4825,
  CODE_FOR_avx512vl_expandv8si_mask = 4826,
  CODE_FOR_avx512vl_expandv8sf_mask = 4827,
  CODE_FOR_avx512vl_expandv4di_mask = 4828,
  CODE_FOR_avx512vl_expandv4df_mask = 4829,
  CODE_FOR_avx512vl_expandv4si_mask = 4830,
  CODE_FOR_avx512vl_expandv4sf_mask = 4831,
  CODE_FOR_avx512vl_expandv2di_mask = 4832,
  CODE_FOR_avx512vl_expandv2df_mask = 4833,
  CODE_FOR_expandv64qi_mask = 4834,
  CODE_FOR_expandv16qi_mask = 4835,
  CODE_FOR_expandv32qi_mask = 4836,
  CODE_FOR_expandv32hi_mask = 4837,
  CODE_FOR_expandv16hi_mask = 4838,
  CODE_FOR_expandv8hi_mask = 4839,
  CODE_FOR_avx512dq_rangepv16sf = 4840,
  CODE_FOR_avx512dq_rangepv16sf_round = 4841,
  CODE_FOR_avx512dq_rangepv16sf_mask = 4842,
  CODE_FOR_avx512dq_rangepv16sf_mask_round = 4843,
  CODE_FOR_avx512dq_rangepv8sf = 4844,
   CODE_FOR_avx512dq_rangepv8sf_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv8sf_mask = 4845,
   CODE_FOR_avx512dq_rangepv8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4sf = 4846,
   CODE_FOR_avx512dq_rangepv4sf_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4sf_mask = 4847,
   CODE_FOR_avx512dq_rangepv4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv8df = 4848,
  CODE_FOR_avx512dq_rangepv8df_round = 4849,
  CODE_FOR_avx512dq_rangepv8df_mask = 4850,
  CODE_FOR_avx512dq_rangepv8df_mask_round = 4851,
  CODE_FOR_avx512dq_rangepv4df = 4852,
   CODE_FOR_avx512dq_rangepv4df_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4df_mask = 4853,
   CODE_FOR_avx512dq_rangepv4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv2df = 4854,
   CODE_FOR_avx512dq_rangepv2df_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv2df_mask = 4855,
   CODE_FOR_avx512dq_rangepv2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangesv4sf = 4856,
  CODE_FOR_avx512dq_rangesv4sf_mask = 4857,
  CODE_FOR_avx512dq_rangesv4sf_round = 4858,
  CODE_FOR_avx512dq_rangesv4sf_mask_round = 4859,
  CODE_FOR_avx512dq_rangesv2df = 4860,
  CODE_FOR_avx512dq_rangesv2df_mask = 4861,
  CODE_FOR_avx512dq_rangesv2df_round = 4862,
  CODE_FOR_avx512dq_rangesv2df_mask_round = 4863,
  CODE_FOR_avx512dq_fpclassv16sf = 4864,
  CODE_FOR_avx512dq_fpclassv16sf_mask = 4865,
  CODE_FOR_avx512dq_fpclassv8sf = 4866,
  CODE_FOR_avx512dq_fpclassv8sf_mask = 4867,
  CODE_FOR_avx512dq_fpclassv4sf = 4868,
  CODE_FOR_avx512dq_fpclassv4sf_mask = 4869,
  CODE_FOR_avx512dq_fpclassv8df = 4870,
  CODE_FOR_avx512dq_fpclassv8df_mask = 4871,
  CODE_FOR_avx512dq_fpclassv4df = 4872,
  CODE_FOR_avx512dq_fpclassv4df_mask = 4873,
  CODE_FOR_avx512dq_fpclassv2df = 4874,
  CODE_FOR_avx512dq_fpclassv2df_mask = 4875,
  CODE_FOR_avx512dq_vmfpclassv4sf = 4876,
  CODE_FOR_avx512dq_vmfpclassv2df = 4877,
  CODE_FOR_avx512f_getmantv16sf = 4878,
  CODE_FOR_avx512f_getmantv16sf_round = 4879,
  CODE_FOR_avx512f_getmantv16sf_mask = 4880,
  CODE_FOR_avx512f_getmantv16sf_mask_round = 4881,
  CODE_FOR_avx512vl_getmantv8sf = 4882,
  CODE_FOR_avx512vl_getmantv8sf_round = 4883,
  CODE_FOR_avx512vl_getmantv8sf_mask = 4884,
  CODE_FOR_avx512vl_getmantv8sf_mask_round = 4885,
  CODE_FOR_avx512vl_getmantv4sf = 4886,
  CODE_FOR_avx512vl_getmantv4sf_round = 4887,
  CODE_FOR_avx512vl_getmantv4sf_mask = 4888,
  CODE_FOR_avx512vl_getmantv4sf_mask_round = 4889,
  CODE_FOR_avx512f_getmantv8df = 4890,
  CODE_FOR_avx512f_getmantv8df_round = 4891,
  CODE_FOR_avx512f_getmantv8df_mask = 4892,
  CODE_FOR_avx512f_getmantv8df_mask_round = 4893,
  CODE_FOR_avx512vl_getmantv4df = 4894,
  CODE_FOR_avx512vl_getmantv4df_round = 4895,
  CODE_FOR_avx512vl_getmantv4df_mask = 4896,
  CODE_FOR_avx512vl_getmantv4df_mask_round = 4897,
  CODE_FOR_avx512vl_getmantv2df = 4898,
  CODE_FOR_avx512vl_getmantv2df_round = 4899,
  CODE_FOR_avx512vl_getmantv2df_mask = 4900,
  CODE_FOR_avx512vl_getmantv2df_mask_round = 4901,
  CODE_FOR_avx512f_vgetmantv4sf = 4902,
  CODE_FOR_avx512f_vgetmantv4sf_mask = 4903,
  CODE_FOR_avx512f_vgetmantv4sf_round = 4904,
  CODE_FOR_avx512f_vgetmantv4sf_mask_round = 4905,
  CODE_FOR_avx512f_vgetmantv2df = 4906,
  CODE_FOR_avx512f_vgetmantv2df_mask = 4907,
  CODE_FOR_avx512f_vgetmantv2df_round = 4908,
  CODE_FOR_avx512f_vgetmantv2df_mask_round = 4909,
  CODE_FOR_avx512bw_dbpsadbwv8hi_mask = 4911,
  CODE_FOR_avx512bw_dbpsadbwv16hi_mask = 4913,
  CODE_FOR_avx512bw_dbpsadbwv32hi_mask = 4915,
  CODE_FOR_clzv16si2 = 4916,
  CODE_FOR_clzv16si2_mask = 4917,
  CODE_FOR_clzv8si2 = 4918,
  CODE_FOR_clzv8si2_mask = 4919,
  CODE_FOR_clzv4si2 = 4920,
  CODE_FOR_clzv4si2_mask = 4921,
  CODE_FOR_clzv8di2 = 4922,
  CODE_FOR_clzv8di2_mask = 4923,
  CODE_FOR_clzv4di2 = 4924,
  CODE_FOR_clzv4di2_mask = 4925,
  CODE_FOR_clzv2di2 = 4926,
  CODE_FOR_clzv2di2_mask = 4927,
  CODE_FOR_conflictv16si_mask = 4929,
  CODE_FOR_conflictv8si_mask = 4931,
  CODE_FOR_conflictv4si_mask = 4933,
  CODE_FOR_conflictv8di_mask = 4935,
  CODE_FOR_conflictv4di_mask = 4937,
  CODE_FOR_conflictv2di_mask = 4939,
  CODE_FOR_sha1msg1 = 4940,
  CODE_FOR_sha1msg2 = 4941,
  CODE_FOR_sha1nexte = 4942,
  CODE_FOR_sha1rnds4 = 4943,
  CODE_FOR_sha256msg1 = 4944,
  CODE_FOR_sha256msg2 = 4945,
  CODE_FOR_sha256rnds2 = 4946,
  CODE_FOR_avx512f_si512_si = 4947,
  CODE_FOR_avx512f_ps512_ps = 4948,
  CODE_FOR_avx512f_pd512_pd = 4949,
  CODE_FOR_avx512f_si512_256si = 4950,
  CODE_FOR_avx512f_ps512_256ps = 4951,
  CODE_FOR_avx512f_pd512_256pd = 4952,
  CODE_FOR_vpamdd52luqv8di = 4953,
  CODE_FOR_vpamdd52luqv8di_maskz_1 = 4954,
  CODE_FOR_vpamdd52huqv8di = 4955,
  CODE_FOR_vpamdd52huqv8di_maskz_1 = 4956,
  CODE_FOR_vpamdd52luqv4di = 4957,
  CODE_FOR_vpamdd52luqv4di_maskz_1 = 4958,
  CODE_FOR_vpamdd52huqv4di = 4959,
  CODE_FOR_vpamdd52huqv4di_maskz_1 = 4960,
  CODE_FOR_vpamdd52luqv2di = 4961,
  CODE_FOR_vpamdd52luqv2di_maskz_1 = 4962,
  CODE_FOR_vpamdd52huqv2di = 4963,
  CODE_FOR_vpamdd52huqv2di_maskz_1 = 4964,
  CODE_FOR_vpamdd52luqv8di_mask = 4965,
  CODE_FOR_vpamdd52huqv8di_mask = 4966,
  CODE_FOR_vpamdd52luqv4di_mask = 4967,
  CODE_FOR_vpamdd52huqv4di_mask = 4968,
  CODE_FOR_vpamdd52luqv2di_mask = 4969,
  CODE_FOR_vpamdd52huqv2di_mask = 4970,
  CODE_FOR_vpmultishiftqbv64qi = 4971,
  CODE_FOR_vpmultishiftqbv64qi_mask = 4972,
  CODE_FOR_vpmultishiftqbv16qi = 4973,
  CODE_FOR_vpmultishiftqbv16qi_mask = 4974,
  CODE_FOR_vpmultishiftqbv32qi = 4975,
  CODE_FOR_vpmultishiftqbv32qi_mask = 4976,
  CODE_FOR_avx5124fmaddps_4fmaddps = 4979,
  CODE_FOR_avx5124fmaddps_4fmaddps_mask = 4980,
  CODE_FOR_avx5124fmaddps_4fmaddps_maskz = 4981,
  CODE_FOR_avx5124fmaddps_4fmaddss = 4982,
  CODE_FOR_avx5124fmaddps_4fmaddss_mask = 4983,
  CODE_FOR_avx5124fmaddps_4fmaddss_maskz = 4984,
  CODE_FOR_avx5124fmaddps_4fnmaddps = 4985,
  CODE_FOR_avx5124fmaddps_4fnmaddps_mask = 4986,
  CODE_FOR_avx5124fmaddps_4fnmaddps_maskz = 4987,
  CODE_FOR_avx5124fmaddps_4fnmaddss = 4988,
  CODE_FOR_avx5124fmaddps_4fnmaddss_mask = 4989,
  CODE_FOR_avx5124fmaddps_4fnmaddss_maskz = 4990,
  CODE_FOR_avx5124vnniw_vp4dpwssd = 4991,
  CODE_FOR_avx5124vnniw_vp4dpwssd_mask = 4992,
  CODE_FOR_avx5124vnniw_vp4dpwssd_maskz = 4993,
  CODE_FOR_avx5124vnniw_vp4dpwssds = 4994,
  CODE_FOR_avx5124vnniw_vp4dpwssds_mask = 4995,
  CODE_FOR_avx5124vnniw_vp4dpwssds_maskz = 4996,
  CODE_FOR_vpopcountv16si = 4997,
  CODE_FOR_vpopcountv16si_mask = 4998,
  CODE_FOR_vpopcountv8si = 4999,
  CODE_FOR_vpopcountv8si_mask = 5000,
  CODE_FOR_vpopcountv4si = 5001,
  CODE_FOR_vpopcountv4si_mask = 5002,
  CODE_FOR_vpopcountv8di = 5003,
  CODE_FOR_vpopcountv8di_mask = 5004,
  CODE_FOR_vpopcountv4di = 5005,
  CODE_FOR_vpopcountv4di_mask = 5006,
  CODE_FOR_vpopcountv2di = 5007,
  CODE_FOR_vpopcountv2di_mask = 5008,
  CODE_FOR_save_multiplesi = 5009,
  CODE_FOR_save_multipledi = 5010,
  CODE_FOR_restore_multiplesi = 5011,
  CODE_FOR_restore_multipledi = 5012,
  CODE_FOR_restore_multiple_and_returnsi = 5013,
  CODE_FOR_restore_multiple_and_returndi = 5014,
  CODE_FOR_restore_multiple_leave_returnsi = 5015,
  CODE_FOR_restore_multiple_leave_returndi = 5016,
  CODE_FOR_vpopcountv64qi = 5017,
  CODE_FOR_vpopcountv64qi_mask = 5018,
  CODE_FOR_vpopcountv16qi = 5019,
  CODE_FOR_vpopcountv16qi_mask = 5020,
  CODE_FOR_vpopcountv32qi = 5021,
  CODE_FOR_vpopcountv32qi_mask = 5022,
  CODE_FOR_vpopcountv32hi = 5023,
  CODE_FOR_vpopcountv32hi_mask = 5024,
  CODE_FOR_vpopcountv16hi = 5025,
  CODE_FOR_vpopcountv16hi_mask = 5026,
  CODE_FOR_vpopcountv8hi = 5027,
  CODE_FOR_vpopcountv8hi_mask = 5028,
  CODE_FOR_vgf2p8affineinvqb_v64qi = 5029,
  CODE_FOR_vgf2p8affineinvqb_v64qi_mask = 5030,
  CODE_FOR_vgf2p8affineinvqb_v32qi = 5031,
  CODE_FOR_vgf2p8affineinvqb_v32qi_mask = 5032,
  CODE_FOR_vgf2p8affineinvqb_v16qi = 5033,
  CODE_FOR_vgf2p8affineinvqb_v16qi_mask = 5034,
  CODE_FOR_vgf2p8affineqb_v64qi = 5035,
  CODE_FOR_vgf2p8affineqb_v64qi_mask = 5036,
  CODE_FOR_vgf2p8affineqb_v32qi = 5037,
  CODE_FOR_vgf2p8affineqb_v32qi_mask = 5038,
  CODE_FOR_vgf2p8affineqb_v16qi = 5039,
  CODE_FOR_vgf2p8affineqb_v16qi_mask = 5040,
  CODE_FOR_vgf2p8mulb_v64qi = 5041,
  CODE_FOR_vgf2p8mulb_v64qi_mask = 5042,
  CODE_FOR_vgf2p8mulb_v32qi = 5043,
  CODE_FOR_vgf2p8mulb_v32qi_mask = 5044,
  CODE_FOR_vgf2p8mulb_v16qi = 5045,
  CODE_FOR_vgf2p8mulb_v16qi_mask = 5046,
  CODE_FOR_vpshrd_v32hi = 5047,
  CODE_FOR_vpshrd_v32hi_mask = 5048,
  CODE_FOR_vpshrd_v16si = 5049,
  CODE_FOR_vpshrd_v16si_mask = 5050,
  CODE_FOR_vpshrd_v8di = 5051,
  CODE_FOR_vpshrd_v8di_mask = 5052,
  CODE_FOR_vpshrd_v16hi = 5053,
  CODE_FOR_vpshrd_v16hi_mask = 5054,
  CODE_FOR_vpshrd_v8si = 5055,
  CODE_FOR_vpshrd_v8si_mask = 5056,
  CODE_FOR_vpshrd_v4di = 5057,
  CODE_FOR_vpshrd_v4di_mask = 5058,
  CODE_FOR_vpshrd_v8hi = 5059,
  CODE_FOR_vpshrd_v8hi_mask = 5060,
  CODE_FOR_vpshrd_v4si = 5061,
  CODE_FOR_vpshrd_v4si_mask = 5062,
  CODE_FOR_vpshrd_v2di = 5063,
  CODE_FOR_vpshrd_v2di_mask = 5064,
  CODE_FOR_vpshld_v32hi = 5065,
  CODE_FOR_vpshld_v32hi_mask = 5066,
  CODE_FOR_vpshld_v16si = 5067,
  CODE_FOR_vpshld_v16si_mask = 5068,
  CODE_FOR_vpshld_v8di = 5069,
  CODE_FOR_vpshld_v8di_mask = 5070,
  CODE_FOR_vpshld_v16hi = 5071,
  CODE_FOR_vpshld_v16hi_mask = 5072,
  CODE_FOR_vpshld_v8si = 5073,
  CODE_FOR_vpshld_v8si_mask = 5074,
  CODE_FOR_vpshld_v4di = 5075,
  CODE_FOR_vpshld_v4di_mask = 5076,
  CODE_FOR_vpshld_v8hi = 5077,
  CODE_FOR_vpshld_v8hi_mask = 5078,
  CODE_FOR_vpshld_v4si = 5079,
  CODE_FOR_vpshld_v4si_mask = 5080,
  CODE_FOR_vpshld_v2di = 5081,
  CODE_FOR_vpshld_v2di_mask = 5082,
  CODE_FOR_vpshrdv_v32hi = 5083,
  CODE_FOR_vpshrdv_v16si = 5084,
  CODE_FOR_vpshrdv_v8di = 5085,
  CODE_FOR_vpshrdv_v16hi = 5086,
  CODE_FOR_vpshrdv_v8si = 5087,
  CODE_FOR_vpshrdv_v4di = 5088,
  CODE_FOR_vpshrdv_v8hi = 5089,
  CODE_FOR_vpshrdv_v4si = 5090,
  CODE_FOR_vpshrdv_v2di = 5091,
  CODE_FOR_vpshrdv_v32hi_mask = 5092,
  CODE_FOR_vpshrdv_v16si_mask = 5093,
  CODE_FOR_vpshrdv_v8di_mask = 5094,
  CODE_FOR_vpshrdv_v16hi_mask = 5095,
  CODE_FOR_vpshrdv_v8si_mask = 5096,
  CODE_FOR_vpshrdv_v4di_mask = 5097,
  CODE_FOR_vpshrdv_v8hi_mask = 5098,
  CODE_FOR_vpshrdv_v4si_mask = 5099,
  CODE_FOR_vpshrdv_v2di_mask = 5100,
  CODE_FOR_vpshrdv_v32hi_maskz_1 = 5101,
  CODE_FOR_vpshrdv_v16si_maskz_1 = 5102,
  CODE_FOR_vpshrdv_v8di_maskz_1 = 5103,
  CODE_FOR_vpshrdv_v16hi_maskz_1 = 5104,
  CODE_FOR_vpshrdv_v8si_maskz_1 = 5105,
  CODE_FOR_vpshrdv_v4di_maskz_1 = 5106,
  CODE_FOR_vpshrdv_v8hi_maskz_1 = 5107,
  CODE_FOR_vpshrdv_v4si_maskz_1 = 5108,
  CODE_FOR_vpshrdv_v2di_maskz_1 = 5109,
  CODE_FOR_vpshldv_v32hi = 5110,
  CODE_FOR_vpshldv_v16si = 5111,
  CODE_FOR_vpshldv_v8di = 5112,
  CODE_FOR_vpshldv_v16hi = 5113,
  CODE_FOR_vpshldv_v8si = 5114,
  CODE_FOR_vpshldv_v4di = 5115,
  CODE_FOR_vpshldv_v8hi = 5116,
  CODE_FOR_vpshldv_v4si = 5117,
  CODE_FOR_vpshldv_v2di = 5118,
  CODE_FOR_vpshldv_v32hi_mask = 5119,
  CODE_FOR_vpshldv_v16si_mask = 5120,
  CODE_FOR_vpshldv_v8di_mask = 5121,
  CODE_FOR_vpshldv_v16hi_mask = 5122,
  CODE_FOR_vpshldv_v8si_mask = 5123,
  CODE_FOR_vpshldv_v4di_mask = 5124,
  CODE_FOR_vpshldv_v8hi_mask = 5125,
  CODE_FOR_vpshldv_v4si_mask = 5126,
  CODE_FOR_vpshldv_v2di_mask = 5127,
  CODE_FOR_vpshldv_v32hi_maskz_1 = 5128,
  CODE_FOR_vpshldv_v16si_maskz_1 = 5129,
  CODE_FOR_vpshldv_v8di_maskz_1 = 5130,
  CODE_FOR_vpshldv_v16hi_maskz_1 = 5131,
  CODE_FOR_vpshldv_v8si_maskz_1 = 5132,
  CODE_FOR_vpshldv_v4di_maskz_1 = 5133,
  CODE_FOR_vpshldv_v8hi_maskz_1 = 5134,
  CODE_FOR_vpshldv_v4si_maskz_1 = 5135,
  CODE_FOR_vpshldv_v2di_maskz_1 = 5136,
  CODE_FOR_vpdpbusd_v16si = 5137,
  CODE_FOR_vpdpbusd_v8si = 5138,
  CODE_FOR_vpdpbusd_v4si = 5139,
  CODE_FOR_vpdpbusd_v16si_mask = 5140,
  CODE_FOR_vpdpbusd_v8si_mask = 5141,
  CODE_FOR_vpdpbusd_v4si_mask = 5142,
  CODE_FOR_vpdpbusd_v16si_maskz_1 = 5143,
  CODE_FOR_vpdpbusd_v8si_maskz_1 = 5144,
  CODE_FOR_vpdpbusd_v4si_maskz_1 = 5145,
  CODE_FOR_vpdpbusds_v16si = 5146,
  CODE_FOR_vpdpbusds_v8si = 5147,
  CODE_FOR_vpdpbusds_v4si = 5148,
  CODE_FOR_vpdpbusds_v16si_mask = 5149,
  CODE_FOR_vpdpbusds_v8si_mask = 5150,
  CODE_FOR_vpdpbusds_v4si_mask = 5151,
  CODE_FOR_vpdpbusds_v16si_maskz_1 = 5152,
  CODE_FOR_vpdpbusds_v8si_maskz_1 = 5153,
  CODE_FOR_vpdpbusds_v4si_maskz_1 = 5154,
  CODE_FOR_vpdpwssd_v16si = 5155,
  CODE_FOR_vpdpwssd_v8si = 5156,
  CODE_FOR_vpdpwssd_v4si = 5157,
  CODE_FOR_vpdpwssd_v16si_mask = 5158,
  CODE_FOR_vpdpwssd_v8si_mask = 5159,
  CODE_FOR_vpdpwssd_v4si_mask = 5160,
  CODE_FOR_vpdpwssd_v16si_maskz_1 = 5161,
  CODE_FOR_vpdpwssd_v8si_maskz_1 = 5162,
  CODE_FOR_vpdpwssd_v4si_maskz_1 = 5163,
  CODE_FOR_vpdpwssds_v16si = 5164,
  CODE_FOR_vpdpwssds_v8si = 5165,
  CODE_FOR_vpdpwssds_v4si = 5166,
  CODE_FOR_vpdpwssds_v16si_mask = 5167,
  CODE_FOR_vpdpwssds_v8si_mask = 5168,
  CODE_FOR_vpdpwssds_v4si_mask = 5169,
  CODE_FOR_vpdpwssds_v16si_maskz_1 = 5170,
  CODE_FOR_vpdpwssds_v8si_maskz_1 = 5171,
  CODE_FOR_vpdpwssds_v4si_maskz_1 = 5172,
  CODE_FOR_vaesdec_v32qi = 5173,
  CODE_FOR_vaesdec_v16qi = 5174,
  CODE_FOR_vaesdec_v64qi = 5175,
  CODE_FOR_vaesdeclast_v32qi = 5176,
  CODE_FOR_vaesdeclast_v16qi = 5177,
  CODE_FOR_vaesdeclast_v64qi = 5178,
  CODE_FOR_vaesenc_v32qi = 5179,
  CODE_FOR_vaesenc_v16qi = 5180,
  CODE_FOR_vaesenc_v64qi = 5181,
  CODE_FOR_vaesenclast_v32qi = 5182,
  CODE_FOR_vaesenclast_v16qi = 5183,
  CODE_FOR_vaesenclast_v64qi = 5184,
  CODE_FOR_vpclmulqdq_v8di = 5185,
  CODE_FOR_vpclmulqdq_v4di = 5186,
  CODE_FOR_vpclmulqdq_v2di = 5187,
  CODE_FOR_avx512vl_vpshufbitqmbv64qi = 5188,
  CODE_FOR_avx512vl_vpshufbitqmbv64qi_mask = 5189,
  CODE_FOR_avx512vl_vpshufbitqmbv32qi = 5190,
  CODE_FOR_avx512vl_vpshufbitqmbv32qi_mask = 5191,
  CODE_FOR_avx512vl_vpshufbitqmbv16qi = 5192,
  CODE_FOR_avx512vl_vpshufbitqmbv16qi_mask = 5193,
  CODE_FOR_mfence_sse2 = 5196,
  CODE_FOR_mfence_nosse = 5197,
  CODE_FOR_atomic_loaddi_fpu = 5198,
  CODE_FOR_atomic_storeqi_1 = 5199,
  CODE_FOR_atomic_storehi_1 = 5200,
  CODE_FOR_atomic_storesi_1 = 5201,
  CODE_FOR_atomic_storedi_1 = 5202,
  CODE_FOR_atomic_storedi_fpu = 5203,
  CODE_FOR_loaddi_via_fpu = 5204,
  CODE_FOR_storedi_via_fpu = 5205,
  CODE_FOR_loaddi_via_sse = 5206,
  CODE_FOR_storedi_via_sse = 5207,
  CODE_FOR_atomic_compare_and_swapdi_doubleword = 5208,
  CODE_FOR_atomic_compare_and_swapti_doubleword = 5209,
  CODE_FOR_atomic_compare_and_swapqi_1 = 5210,
  CODE_FOR_atomic_compare_and_swaphi_1 = 5211,
  CODE_FOR_atomic_compare_and_swapsi_1 = 5212,
  CODE_FOR_atomic_compare_and_swapdi_1 = 5213,
  CODE_FOR_atomic_fetch_addqi = 5214,
  CODE_FOR_atomic_fetch_addhi = 5215,
  CODE_FOR_atomic_fetch_addsi = 5216,
  CODE_FOR_atomic_fetch_adddi = 5217,
  CODE_FOR_atomic_exchangeqi = 5222,
  CODE_FOR_atomic_exchangehi = 5223,
  CODE_FOR_atomic_exchangesi = 5224,
  CODE_FOR_atomic_exchangedi = 5225,
  CODE_FOR_atomic_addqi = 5226,
  CODE_FOR_atomic_addhi = 5227,
  CODE_FOR_atomic_addsi = 5228,
  CODE_FOR_atomic_adddi = 5229,
  CODE_FOR_atomic_subqi = 5230,
  CODE_FOR_atomic_subhi = 5231,
  CODE_FOR_atomic_subsi = 5232,
  CODE_FOR_atomic_subdi = 5233,
  CODE_FOR_atomic_andqi = 5234,
  CODE_FOR_atomic_orqi = 5235,
  CODE_FOR_atomic_xorqi = 5236,
  CODE_FOR_atomic_andhi = 5237,
  CODE_FOR_atomic_orhi = 5238,
  CODE_FOR_atomic_xorhi = 5239,
  CODE_FOR_atomic_andsi = 5240,
  CODE_FOR_atomic_orsi = 5241,
  CODE_FOR_atomic_xorsi = 5242,
  CODE_FOR_atomic_anddi = 5243,
  CODE_FOR_atomic_ordi = 5244,
  CODE_FOR_atomic_xordi = 5245,
  CODE_FOR_atomic_bit_test_and_sethi_1 = 5246,
  CODE_FOR_atomic_bit_test_and_setsi_1 = 5247,
  CODE_FOR_atomic_bit_test_and_setdi_1 = 5248,
  CODE_FOR_atomic_bit_test_and_complementhi_1 = 5249,
  CODE_FOR_atomic_bit_test_and_complementsi_1 = 5250,
  CODE_FOR_atomic_bit_test_and_complementdi_1 = 5251,
  CODE_FOR_atomic_bit_test_and_resethi_1 = 5252,
  CODE_FOR_atomic_bit_test_and_resetsi_1 = 5253,
  CODE_FOR_atomic_bit_test_and_resetdi_1 = 5254,
  CODE_FOR_cbranchqi4 = 5255,
  CODE_FOR_cbranchhi4 = 5256,
  CODE_FOR_cbranchsi4 = 5257,
  CODE_FOR_cbranchdi4 = 5258,
  CODE_FOR_cbranchti4 = 5259,
  CODE_FOR_cstoreqi4 = 5260,
  CODE_FOR_cstorehi4 = 5261,
  CODE_FOR_cstoresi4 = 5262,
  CODE_FOR_cstoredi4 = 5263,
  CODE_FOR_cmpsi_1 = 5264,
  CODE_FOR_cmpdi_1 = 5265,
  CODE_FOR_cmpqi_ext_3 = 5266,
  CODE_FOR_cbranchxf4 = 5267,
  CODE_FOR_cstorexf4 = 5268,
  CODE_FOR_cbranchsf4 = 5269,
  CODE_FOR_cbranchdf4 = 5270,
  CODE_FOR_cstoresf4 = 5271,
  CODE_FOR_cstoredf4 = 5272,
  CODE_FOR_cbranchcc4 = 5273,
  CODE_FOR_cstorecc4 = 5274,
  CODE_FOR_reload_noff_store = 5275,
  CODE_FOR_reload_noff_load = 5276,
  CODE_FOR_movxi = 5277,
  CODE_FOR_movoi = 5278,
  CODE_FOR_movti = 5279,
  CODE_FOR_movcdi = 5280,
  CODE_FOR_movqi = 5281,
  CODE_FOR_movhi = 5282,
  CODE_FOR_movsi = 5283,
  CODE_FOR_movdi = 5284,
  CODE_FOR_movstrictqi = 5285,
  CODE_FOR_movstricthi = 5286,
  CODE_FOR_extvhi = 5287,
  CODE_FOR_extvsi = 5288,
  CODE_FOR_extzvhi = 5289,
  CODE_FOR_extzvsi = 5290,
  CODE_FOR_extzvdi = 5291,
  CODE_FOR_insvhi = 5292,
  CODE_FOR_insvsi = 5293,
  CODE_FOR_insvdi = 5294,
  CODE_FOR_movtf = 5295,
  CODE_FOR_movsf = 5296,
  CODE_FOR_movdf = 5297,
  CODE_FOR_movxf = 5298,
  CODE_FOR_zero_extendsidi2 = 5299,
  CODE_FOR_zero_extendqisi2 = 5300,
  CODE_FOR_zero_extendhisi2 = 5301,
  CODE_FOR_zero_extendqihi2 = 5302,
  CODE_FOR_extendsidi2 = 5303,
  CODE_FOR_extendsfdf2 = 5304,
  CODE_FOR_extendsfxf2 = 5305,
  CODE_FOR_extenddfxf2 = 5306,
  CODE_FOR_truncdfsf2 = 5307,
  CODE_FOR_truncdfsf2_with_temp = 5308,
  CODE_FOR_truncxfsf2 = 5309,
  CODE_FOR_truncxfdf2 = 5310,
  CODE_FOR_fix_truncxfdi2 = 5311,
  CODE_FOR_fix_truncsfdi2 = 5312,
  CODE_FOR_fix_truncdfdi2 = 5313,
  CODE_FOR_fix_truncxfsi2 = 5314,
  CODE_FOR_fix_truncsfsi2 = 5315,
  CODE_FOR_fix_truncdfsi2 = 5316,
  CODE_FOR_fix_truncsfhi2 = 5317,
  CODE_FOR_fix_truncdfhi2 = 5318,
  CODE_FOR_fix_truncxfhi2 = 5319,
  CODE_FOR_fixuns_truncsfsi2 = 5320,
  CODE_FOR_fixuns_truncdfsi2 = 5321,
  CODE_FOR_fixuns_truncsfhi2 = 5322,
  CODE_FOR_fixuns_truncdfhi2 = 5323,
  CODE_FOR_floatsisf2 = 5324,
  CODE_FOR_floatdisf2 = 5325,
  CODE_FOR_floatsidf2 = 5326,
  CODE_FOR_floatdidf2 = 5327,
  CODE_FOR_floatunsqisf2 = 5328,
  CODE_FOR_floatunshisf2 = 5329,
  CODE_FOR_floatunsqidf2 = 5330,
  CODE_FOR_floatunshidf2 = 5331,
  CODE_FOR_floatunssisf2 = 5332,
  CODE_FOR_floatunssidf2 = 5333,
  CODE_FOR_floatunssixf2 = 5334,
  CODE_FOR_floatunsdisf2 = 5335,
  CODE_FOR_floatunsdidf2 = 5336,
  CODE_FOR_addqi3 = 5337,
  CODE_FOR_addhi3 = 5338,
  CODE_FOR_addsi3 = 5339,
  CODE_FOR_adddi3 = 5340,
  CODE_FOR_addti3 = 5341,
  CODE_FOR_addvqi4 = 5342,
  CODE_FOR_addvhi4 = 5343,
  CODE_FOR_addvsi4 = 5344,
  CODE_FOR_addvdi4 = 5345,
  CODE_FOR_uaddvqi4 = 5346,
  CODE_FOR_uaddvhi4 = 5347,
  CODE_FOR_uaddvsi4 = 5348,
  CODE_FOR_uaddvdi4 = 5349,
  CODE_FOR_subqi3 = 5350,
  CODE_FOR_subhi3 = 5351,
  CODE_FOR_subsi3 = 5352,
  CODE_FOR_subdi3 = 5353,
  CODE_FOR_subti3 = 5354,
  CODE_FOR_subvqi4 = 5355,
  CODE_FOR_subvhi4 = 5356,
  CODE_FOR_subvsi4 = 5357,
  CODE_FOR_subvdi4 = 5358,
  CODE_FOR_usubvqi4 = 5359,
  CODE_FOR_usubvhi4 = 5360,
  CODE_FOR_usubvsi4 = 5361,
  CODE_FOR_usubvdi4 = 5362,
  CODE_FOR_addcarrysi_0 = 5363,
  CODE_FOR_addcarrydi_0 = 5364,
  CODE_FOR_subborrowsi_0 = 5365,
  CODE_FOR_subborrowdi_0 = 5366,
  CODE_FOR_addqi3_cconly_overflow = 5367,
  CODE_FOR_addxf3 = 5368,
  CODE_FOR_subxf3 = 5369,
  CODE_FOR_addsf3 = 5370,
  CODE_FOR_subsf3 = 5371,
  CODE_FOR_adddf3 = 5372,
  CODE_FOR_subdf3 = 5373,
  CODE_FOR_mulhi3 = 5374,
  CODE_FOR_mulsi3 = 5375,
  CODE_FOR_muldi3 = 5376,
  CODE_FOR_mulqi3 = 5377,
  CODE_FOR_mulvhi4 = 5378,
  CODE_FOR_mulvsi4 = 5379,
  CODE_FOR_mulvdi4 = 5380,
  CODE_FOR_umulvhi4 = 5381,
  CODE_FOR_umulvsi4 = 5382,
  CODE_FOR_umulvdi4 = 5383,
  CODE_FOR_mulvqi4 = 5384,
  CODE_FOR_umulvqi4 = 5385,
  CODE_FOR_mulsidi3 = 5386,
  CODE_FOR_umulsidi3 = 5387,
  CODE_FOR_mulditi3 = 5388,
  CODE_FOR_umulditi3 = 5389,
  CODE_FOR_mulqihi3 = 5390,
  CODE_FOR_umulqihi3 = 5391,
  CODE_FOR_smulsi3_highpart = 5392,
  CODE_FOR_umulsi3_highpart = 5393,
  CODE_FOR_smuldi3_highpart = 5394,
  CODE_FOR_umuldi3_highpart = 5395,
  CODE_FOR_mulxf3 = 5396,
  CODE_FOR_mulsf3 = 5397,
  CODE_FOR_muldf3 = 5398,
  CODE_FOR_divxf3 = 5399,
  CODE_FOR_divsf3 = 5400,
  CODE_FOR_divdf3 = 5401,
  CODE_FOR_divmodhi4 = 5402,
  CODE_FOR_divmodsi4 = 5403,
  CODE_FOR_divmoddi4 = 5404,
  CODE_FOR_divmodqi4 = 5405,
  CODE_FOR_udivmodhi4 = 5406,
  CODE_FOR_udivmodsi4 = 5407,
  CODE_FOR_udivmoddi4 = 5408,
  CODE_FOR_udivmodqi4 = 5409,
  CODE_FOR_testsi_ccno_1 = 5410,
  CODE_FOR_testqi_ccz_1 = 5411,
  CODE_FOR_testdi_ccno_1 = 5412,
  CODE_FOR_testqi_ext_1_ccno = 5413,
  CODE_FOR_andqi3 = 5414,
  CODE_FOR_andhi3 = 5415,
  CODE_FOR_andsi3 = 5416,
  CODE_FOR_anddi3 = 5417,
  CODE_FOR_iorqi3 = 5418,
  CODE_FOR_xorqi3 = 5419,
  CODE_FOR_iorhi3 = 5420,
  CODE_FOR_xorhi3 = 5421,
  CODE_FOR_iorsi3 = 5422,
  CODE_FOR_xorsi3 = 5423,
  CODE_FOR_iordi3 = 5424,
  CODE_FOR_xordi3 = 5425,
  CODE_FOR_xorqi_ext_1_cc = 5426,
  CODE_FOR_negqi2 = 5427,
  CODE_FOR_neghi2 = 5428,
  CODE_FOR_negsi2 = 5429,
  CODE_FOR_negdi2 = 5430,
  CODE_FOR_negti2 = 5431,
  CODE_FOR_negvqi3 = 5432,
  CODE_FOR_negvhi3 = 5433,
  CODE_FOR_negvsi3 = 5434,
  CODE_FOR_negvdi3 = 5435,
  CODE_FOR_abssf2 = 5436,
  CODE_FOR_negsf2 = 5437,
  CODE_FOR_absdf2 = 5438,
  CODE_FOR_negdf2 = 5439,
  CODE_FOR_absxf2 = 5440,
  CODE_FOR_negxf2 = 5441,
  CODE_FOR_abstf2 = 5442,
  CODE_FOR_negtf2 = 5443,
  CODE_FOR_copysignsf3 = 5444,
  CODE_FOR_copysigndf3 = 5445,
  CODE_FOR_copysigntf3 = 5446,
  CODE_FOR_one_cmplqi2 = 5447,
  CODE_FOR_one_cmplhi2 = 5448,
  CODE_FOR_one_cmplsi2 = 5449,
  CODE_FOR_one_cmpldi2 = 5450,
  CODE_FOR_ashlqi3 = 5451,
  CODE_FOR_ashlhi3 = 5452,
  CODE_FOR_ashlsi3 = 5453,
  CODE_FOR_ashldi3 = 5454,
  CODE_FOR_ashlti3 = 5455,
  CODE_FOR_x86_shiftsi_adj_1 = 5456,
  CODE_FOR_x86_shiftdi_adj_1 = 5457,
  CODE_FOR_x86_shiftsi_adj_2 = 5458,
  CODE_FOR_x86_shiftdi_adj_2 = 5459,
  CODE_FOR_lshrqi3 = 5460,
  CODE_FOR_ashrqi3 = 5461,
  CODE_FOR_lshrhi3 = 5462,
  CODE_FOR_ashrhi3 = 5463,
  CODE_FOR_lshrsi3 = 5464,
  CODE_FOR_ashrsi3 = 5465,
  CODE_FOR_lshrdi3 = 5466,
  CODE_FOR_ashrdi3 = 5467,
  CODE_FOR_lshrti3 = 5468,
  CODE_FOR_ashrti3 = 5469,
  CODE_FOR_x86_shiftsi_adj_3 = 5470,
  CODE_FOR_x86_shiftdi_adj_3 = 5471,
  CODE_FOR_rotlti3 = 5472,
  CODE_FOR_rotrti3 = 5473,
  CODE_FOR_rotldi3 = 5474,
  CODE_FOR_rotrdi3 = 5475,
  CODE_FOR_rotlqi3 = 5476,
  CODE_FOR_rotrqi3 = 5477,
  CODE_FOR_rotlhi3 = 5478,
  CODE_FOR_rotrhi3 = 5479,
  CODE_FOR_rotlsi3 = 5480,
  CODE_FOR_rotrsi3 = 5481,
  CODE_FOR_indirect_jump = 5482,
  CODE_FOR_tablejump = 5483,
  CODE_FOR_call = 5484,
  CODE_FOR_sibcall = 5485,
  CODE_FOR_call_pop = 5486,
  CODE_FOR_call_value = 5487,
  CODE_FOR_sibcall_value = 5488,
  CODE_FOR_call_value_pop = 5489,
  CODE_FOR_untyped_call = 5490,
  CODE_FOR_memory_blockage = 5491,
  CODE_FOR_return = 5492,
  CODE_FOR_simple_return = 5493,
  CODE_FOR_prologue = 5494,
  CODE_FOR_set_got = 5495,
  CODE_FOR_set_got_labelled = 5496,
  CODE_FOR_epilogue = 5497,
  CODE_FOR_sibcall_epilogue = 5498,
  CODE_FOR_eh_return = 5499,
  CODE_FOR_split_stack_prologue = 5500,
  CODE_FOR_split_stack_space_check = 5501,
  CODE_FOR_ffssi2 = 5502,
  CODE_FOR_ffsdi2 = 5503,
  CODE_FOR_clzsi2 = 5504,
  CODE_FOR_clzdi2 = 5505,
  CODE_FOR_bmi2_bzhi_si3 = 5506,
  CODE_FOR_bmi2_bzhi_di3 = 5507,
  CODE_FOR_bswapdi2 = 5508,
  CODE_FOR_bswapsi2 = 5509,
  CODE_FOR_bswaphi2 = 5510,
  CODE_FOR_paritydi2 = 5511,
  CODE_FOR_paritysi2 = 5512,
  CODE_FOR_tls_global_dynamic_32 = 5513,
  CODE_FOR_tls_global_dynamic_64_si = 5514,
  CODE_FOR_tls_global_dynamic_64_di = 5515,
  CODE_FOR_tls_local_dynamic_base_32 = 5516,
  CODE_FOR_tls_local_dynamic_base_64_si = 5517,
  CODE_FOR_tls_local_dynamic_base_64_di = 5518,
  CODE_FOR_tls_dynamic_gnu2_32 = 5519,
  CODE_FOR_tls_dynamic_gnu2_64 = 5520,
  CODE_FOR_rsqrtsf2 = 5521,
  CODE_FOR_sqrtsf2 = 5522,
  CODE_FOR_sqrtdf2 = 5523,
  CODE_FOR_fmodxf3 = 5524,
  CODE_FOR_fmodsf3 = 5525,
  CODE_FOR_fmoddf3 = 5526,
  CODE_FOR_remainderxf3 = 5527,
  CODE_FOR_remaindersf3 = 5528,
  CODE_FOR_remainderdf3 = 5529,
  CODE_FOR_sincossf3 = 5530,
  CODE_FOR_sincosdf3 = 5531,
  CODE_FOR_tanxf2 = 5532,
  CODE_FOR_tansf2 = 5533,
  CODE_FOR_tandf2 = 5534,
  CODE_FOR_atan2xf3 = 5535,
  CODE_FOR_atan2sf3 = 5536,
  CODE_FOR_atan2df3 = 5537,
  CODE_FOR_atanxf2 = 5538,
  CODE_FOR_atansf2 = 5539,
  CODE_FOR_atandf2 = 5540,
  CODE_FOR_asinxf2 = 5541,
  CODE_FOR_asinsf2 = 5542,
  CODE_FOR_asindf2 = 5543,
  CODE_FOR_acosxf2 = 5544,
  CODE_FOR_acossf2 = 5545,
  CODE_FOR_acosdf2 = 5546,
  CODE_FOR_logxf2 = 5547,
  CODE_FOR_logsf2 = 5548,
  CODE_FOR_logdf2 = 5549,
  CODE_FOR_log10xf2 = 5550,
  CODE_FOR_log10sf2 = 5551,
  CODE_FOR_log10df2 = 5552,
  CODE_FOR_log2xf2 = 5553,
  CODE_FOR_log2sf2 = 5554,
  CODE_FOR_log2df2 = 5555,
  CODE_FOR_log1pxf2 = 5556,
  CODE_FOR_log1psf2 = 5557,
  CODE_FOR_log1pdf2 = 5558,
  CODE_FOR_logbxf2 = 5559,
  CODE_FOR_logbsf2 = 5560,
  CODE_FOR_logbdf2 = 5561,
  CODE_FOR_ilogbxf2 = 5562,
  CODE_FOR_ilogbsf2 = 5563,
  CODE_FOR_ilogbdf2 = 5564,
  CODE_FOR_expNcorexf3 = 5565,
  CODE_FOR_expxf2 = 5566,
  CODE_FOR_expsf2 = 5567,
  CODE_FOR_expdf2 = 5568,
  CODE_FOR_exp10xf2 = 5569,
  CODE_FOR_exp10sf2 = 5570,
  CODE_FOR_exp10df2 = 5571,
  CODE_FOR_exp2xf2 = 5572,
  CODE_FOR_exp2sf2 = 5573,
  CODE_FOR_exp2df2 = 5574,
  CODE_FOR_expm1xf2 = 5575,
  CODE_FOR_expm1sf2 = 5576,
  CODE_FOR_expm1df2 = 5577,
  CODE_FOR_ldexpxf3 = 5578,
  CODE_FOR_ldexpsf3 = 5579,
  CODE_FOR_ldexpdf3 = 5580,
  CODE_FOR_scalbxf3 = 5581,
  CODE_FOR_scalbsf3 = 5582,
  CODE_FOR_scalbdf3 = 5583,
  CODE_FOR_significandxf2 = 5584,
  CODE_FOR_significandsf2 = 5585,
  CODE_FOR_significanddf2 = 5586,
  CODE_FOR_rintsf2 = 5587,
  CODE_FOR_rintdf2 = 5588,
  CODE_FOR_roundsf2 = 5589,
  CODE_FOR_rounddf2 = 5590,
  CODE_FOR_roundxf2 = 5591,
  CODE_FOR_lrintxfhi2 = 5592,
  CODE_FOR_lrintxfsi2 = 5593,
  CODE_FOR_lrintxfdi2 = 5594,
  CODE_FOR_lrintsfsi2 = 5595,
  CODE_FOR_lrintsfdi2 = 5596,
  CODE_FOR_lrintdfsi2 = 5597,
  CODE_FOR_lrintdfdi2 = 5598,
  CODE_FOR_lroundsfhi2 = 5599,
  CODE_FOR_lrounddfhi2 = 5600,
  CODE_FOR_lroundxfhi2 = 5601,
  CODE_FOR_lroundsfsi2 = 5602,
  CODE_FOR_lrounddfsi2 = 5603,
  CODE_FOR_lroundxfsi2 = 5604,
  CODE_FOR_lroundsfdi2 = 5605,
  CODE_FOR_lrounddfdi2 = 5606,
  CODE_FOR_lroundxfdi2 = 5607,
  CODE_FOR_floorxf2 = 5608,
  CODE_FOR_ceilxf2 = 5609,
  CODE_FOR_btruncxf2 = 5610,
  CODE_FOR_floorsf2 = 5611,
  CODE_FOR_ceilsf2 = 5612,
  CODE_FOR_btruncsf2 = 5613,
  CODE_FOR_floordf2 = 5614,
  CODE_FOR_ceildf2 = 5615,
  CODE_FOR_btruncdf2 = 5616,
  CODE_FOR_nearbyintxf2 = 5617,
  CODE_FOR_nearbyintsf2 = 5618,
  CODE_FOR_nearbyintdf2 = 5619,
  CODE_FOR_lfloorxfhi2 = 5620,
  CODE_FOR_lceilxfhi2 = 5621,
  CODE_FOR_lfloorxfsi2 = 5622,
  CODE_FOR_lceilxfsi2 = 5623,
  CODE_FOR_lfloorxfdi2 = 5624,
  CODE_FOR_lceilxfdi2 = 5625,
  CODE_FOR_lfloorsfsi2 = 5626,
  CODE_FOR_lceilsfsi2 = 5627,
  CODE_FOR_lfloorsfdi2 = 5628,
  CODE_FOR_lceilsfdi2 = 5629,
  CODE_FOR_lfloordfsi2 = 5630,
  CODE_FOR_lceildfsi2 = 5631,
  CODE_FOR_lfloordfdi2 = 5632,
  CODE_FOR_lceildfdi2 = 5633,
  CODE_FOR_isinfxf2 = 5634,
  CODE_FOR_isinfsf2 = 5635,
  CODE_FOR_isinfdf2 = 5636,
  CODE_FOR_signbittf2 = 5637,
  CODE_FOR_signbitxf2 = 5638,
  CODE_FOR_signbitdf2 = 5639,
  CODE_FOR_signbitsf2 = 5640,
  CODE_FOR_movmemsi = 5641,
  CODE_FOR_movmemdi = 5642,
  CODE_FOR_strmov = 5643,
  CODE_FOR_strmov_singleop = 5644,
  CODE_FOR_rep_mov = 5645,
  CODE_FOR_setmemsi = 5646,
  CODE_FOR_setmemdi = 5647,
  CODE_FOR_strset = 5648,
  CODE_FOR_strset_singleop = 5649,
  CODE_FOR_rep_stos = 5650,
  CODE_FOR_cmpstrnsi = 5651,
  CODE_FOR_cmpintqi = 5652,
  CODE_FOR_cmpstrnqi_nz_1 = 5653,
  CODE_FOR_cmpstrnqi_1 = 5654,
  CODE_FOR_strlensi = 5655,
  CODE_FOR_strlendi = 5656,
  CODE_FOR_strlenqi_1 = 5657,
  CODE_FOR_movqicc = 5658,
  CODE_FOR_movhicc = 5659,
  CODE_FOR_movsicc = 5660,
  CODE_FOR_movdicc = 5661,
  CODE_FOR_x86_movsicc_0_m1 = 5662,
  CODE_FOR_x86_movdicc_0_m1 = 5663,
  CODE_FOR_movsfcc = 5664,
  CODE_FOR_movdfcc = 5665,
  CODE_FOR_movxfcc = 5666,
  CODE_FOR_addqicc = 5667,
  CODE_FOR_addhicc = 5668,
  CODE_FOR_addsicc = 5669,
  CODE_FOR_adddicc = 5670,
  CODE_FOR_allocate_stack = 5671,
  CODE_FOR_probe_stack = 5672,
  CODE_FOR_builtin_setjmp_receiver = 5673,
  CODE_FOR_save_stack_nonlocal = 5674,
  CODE_FOR_restore_stack_nonlocal = 5675,
  CODE_FOR_prefetch = 5676,
  CODE_FOR_stack_protect_set = 5677,
  CODE_FOR_stack_protect_test = 5678,
  CODE_FOR_lwp_llwpcb = 5679,
  CODE_FOR_lwp_slwpcb = 5680,
  CODE_FOR_lwp_lwpvalsi3 = 5681,
  CODE_FOR_lwp_lwpvaldi3 = 5682,
  CODE_FOR_lwp_lwpinssi3 = 5683,
  CODE_FOR_lwp_lwpinsdi3 = 5684,
  CODE_FOR_pause = 5685,
  CODE_FOR_xbegin = 5686,
  CODE_FOR_xtest = 5687,
  CODE_FOR_bnd32_mk = 5688,
  CODE_FOR_bnd64_mk = 5689,
  CODE_FOR_movbnd32 = 5690,
  CODE_FOR_movbnd64 = 5691,
  CODE_FOR_bnd32_cl = 5692,
  CODE_FOR_bnd32_cu = 5693,
  CODE_FOR_bnd32_cn = 5694,
  CODE_FOR_bnd64_cl = 5695,
  CODE_FOR_bnd64_cu = 5696,
  CODE_FOR_bnd64_cn = 5697,
  CODE_FOR_bnd32_ldx = 5698,
  CODE_FOR_bnd64_ldx = 5699,
  CODE_FOR_bnd32_stx = 5700,
  CODE_FOR_bnd64_stx = 5701,
  CODE_FOR_rdpkru = 5702,
  CODE_FOR_wrpkru = 5703,
  CODE_FOR_movv8qi = 5704,
  CODE_FOR_movv4hi = 5705,
  CODE_FOR_movv2si = 5706,
  CODE_FOR_movv1di = 5707,
  CODE_FOR_movv2sf = 5708,
  CODE_FOR_movmisalignv8qi = 5709,
  CODE_FOR_movmisalignv4hi = 5710,
  CODE_FOR_movmisalignv2si = 5711,
  CODE_FOR_movmisalignv1di = 5712,
  CODE_FOR_movmisalignv2sf = 5713,
  CODE_FOR_mmx_addv2sf3 = 5714,
  CODE_FOR_mmx_subv2sf3 = 5715,
  CODE_FOR_mmx_subrv2sf3 = 5716,
  CODE_FOR_mmx_mulv2sf3 = 5717,
  CODE_FOR_mmx_smaxv2sf3 = 5718,
  CODE_FOR_mmx_sminv2sf3 = 5719,
  CODE_FOR_mmx_eqv2sf3 = 5720,
  CODE_FOR_vec_setv2sf = 5721,
  CODE_FOR_vec_extractv2sfsf = 5722,
  CODE_FOR_vec_initv2sfsf = 5723,
  CODE_FOR_mmx_addv8qi3 = 5724,
  CODE_FOR_mmx_subv8qi3 = 5725,
  CODE_FOR_mmx_addv4hi3 = 5726,
  CODE_FOR_mmx_subv4hi3 = 5727,
  CODE_FOR_mmx_addv2si3 = 5728,
  CODE_FOR_mmx_subv2si3 = 5729,
  CODE_FOR_mmx_addv1di3 = 5730,
  CODE_FOR_mmx_subv1di3 = 5731,
  CODE_FOR_mmx_ssaddv8qi3 = 5732,
  CODE_FOR_mmx_usaddv8qi3 = 5733,
  CODE_FOR_mmx_sssubv8qi3 = 5734,
  CODE_FOR_mmx_ussubv8qi3 = 5735,
  CODE_FOR_mmx_ssaddv4hi3 = 5736,
  CODE_FOR_mmx_usaddv4hi3 = 5737,
  CODE_FOR_mmx_sssubv4hi3 = 5738,
  CODE_FOR_mmx_ussubv4hi3 = 5739,
  CODE_FOR_mmx_mulv4hi3 = 5740,
  CODE_FOR_mmx_smulv4hi3_highpart = 5741,
  CODE_FOR_mmx_umulv4hi3_highpart = 5742,
  CODE_FOR_mmx_pmaddwd = 5743,
  CODE_FOR_mmx_pmulhrwv4hi3 = 5744,
  CODE_FOR_sse2_umulv1siv1di3 = 5745,
  CODE_FOR_mmx_smaxv4hi3 = 5746,
  CODE_FOR_mmx_sminv4hi3 = 5747,
  CODE_FOR_mmx_umaxv8qi3 = 5748,
  CODE_FOR_mmx_uminv8qi3 = 5749,
  CODE_FOR_mmx_eqv8qi3 = 5750,
  CODE_FOR_mmx_eqv4hi3 = 5751,
  CODE_FOR_mmx_eqv2si3 = 5752,
  CODE_FOR_mmx_andv8qi3 = 5753,
  CODE_FOR_mmx_iorv8qi3 = 5754,
  CODE_FOR_mmx_xorv8qi3 = 5755,
  CODE_FOR_mmx_andv4hi3 = 5756,
  CODE_FOR_mmx_iorv4hi3 = 5757,
  CODE_FOR_mmx_xorv4hi3 = 5758,
  CODE_FOR_mmx_andv2si3 = 5759,
  CODE_FOR_mmx_iorv2si3 = 5760,
  CODE_FOR_mmx_xorv2si3 = 5761,
  CODE_FOR_mmx_pinsrw = 5762,
  CODE_FOR_mmx_pshufw = 5763,
  CODE_FOR_vec_setv2si = 5764,
  CODE_FOR_vec_extractv2sisi = 5765,
  CODE_FOR_vec_initv2sisi = 5766,
  CODE_FOR_vec_setv4hi = 5767,
  CODE_FOR_vec_extractv4hihi = 5768,
  CODE_FOR_vec_initv4hihi = 5769,
  CODE_FOR_vec_setv8qi = 5770,
  CODE_FOR_vec_extractv8qiqi = 5771,
  CODE_FOR_vec_initv8qiqi = 5772,
  CODE_FOR_mmx_uavgv8qi3 = 5773,
  CODE_FOR_mmx_uavgv4hi3 = 5774,
  CODE_FOR_mmx_maskmovq = 5775,
  CODE_FOR_mmx_emms = 5776,
  CODE_FOR_mmx_femms = 5777,
  CODE_FOR_movv64qi = 5778,
  CODE_FOR_movv32qi = 5779,
  CODE_FOR_movv16qi = 5780,
  CODE_FOR_movv32hi = 5781,
  CODE_FOR_movv16hi = 5782,
  CODE_FOR_movv8hi = 5783,
  CODE_FOR_movv16si = 5784,
  CODE_FOR_movv8si = 5785,
  CODE_FOR_movv4si = 5786,
  CODE_FOR_movv8di = 5787,
  CODE_FOR_movv4di = 5788,
  CODE_FOR_movv2di = 5789,
  CODE_FOR_movv4ti = 5790,
  CODE_FOR_movv2ti = 5791,
  CODE_FOR_movv1ti = 5792,
  CODE_FOR_movv16sf = 5793,
  CODE_FOR_movv8sf = 5794,
  CODE_FOR_movv4sf = 5795,
  CODE_FOR_movv8df = 5796,
  CODE_FOR_movv4df = 5797,
  CODE_FOR_movv2df = 5798,
  CODE_FOR_movmisalignv64qi = 5799,
  CODE_FOR_movmisalignv32qi = 5800,
  CODE_FOR_movmisalignv16qi = 5801,
  CODE_FOR_movmisalignv32hi = 5802,
  CODE_FOR_movmisalignv16hi = 5803,
  CODE_FOR_movmisalignv8hi = 5804,
  CODE_FOR_movmisalignv16si = 5805,
  CODE_FOR_movmisalignv8si = 5806,
  CODE_FOR_movmisalignv4si = 5807,
  CODE_FOR_movmisalignv8di = 5808,
  CODE_FOR_movmisalignv4di = 5809,
  CODE_FOR_movmisalignv2di = 5810,
  CODE_FOR_movmisalignv4ti = 5811,
  CODE_FOR_movmisalignv2ti = 5812,
  CODE_FOR_movmisalignv1ti = 5813,
  CODE_FOR_movmisalignv16sf = 5814,
  CODE_FOR_movmisalignv8sf = 5815,
  CODE_FOR_movmisalignv4sf = 5816,
  CODE_FOR_movmisalignv8df = 5817,
  CODE_FOR_movmisalignv4df = 5818,
  CODE_FOR_movmisalignv2df = 5819,
  CODE_FOR_storentdi = 5820,
  CODE_FOR_storentsi = 5821,
  CODE_FOR_storentsf = 5822,
  CODE_FOR_storentdf = 5823,
  CODE_FOR_storentv8di = 5824,
  CODE_FOR_storentv4di = 5825,
  CODE_FOR_storentv2di = 5826,
  CODE_FOR_storentv16sf = 5827,
  CODE_FOR_storentv8sf = 5828,
  CODE_FOR_storentv4sf = 5829,
  CODE_FOR_storentv8df = 5830,
  CODE_FOR_storentv4df = 5831,
  CODE_FOR_storentv2df = 5832,
  CODE_FOR_kmovb = 5833,
  CODE_FOR_kmovw = 5834,
  CODE_FOR_kmovd = 5835,
  CODE_FOR_kmovq = 5836,
  CODE_FOR_absv16sf2 = 5837,
  CODE_FOR_negv16sf2 = 5838,
  CODE_FOR_absv8sf2 = 5839,
  CODE_FOR_negv8sf2 = 5840,
  CODE_FOR_absv4sf2 = 5841,
  CODE_FOR_negv4sf2 = 5842,
  CODE_FOR_absv8df2 = 5843,
  CODE_FOR_negv8df2 = 5844,
  CODE_FOR_absv4df2 = 5845,
  CODE_FOR_negv4df2 = 5846,
  CODE_FOR_absv2df2 = 5847,
  CODE_FOR_negv2df2 = 5848,
  CODE_FOR_addv16sf3 = 5849,
  CODE_FOR_addv16sf3_round = 5850,
  CODE_FOR_addv16sf3_mask = 5851,
  CODE_FOR_addv16sf3_mask_round = 5852,
  CODE_FOR_subv16sf3 = 5853,
  CODE_FOR_subv16sf3_round = 5854,
  CODE_FOR_subv16sf3_mask = 5855,
  CODE_FOR_subv16sf3_mask_round = 5856,
  CODE_FOR_addv8sf3 = 5857,
   CODE_FOR_addv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_addv8sf3_mask = 5858,
   CODE_FOR_addv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv8sf3 = 5859,
   CODE_FOR_subv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_subv8sf3_mask = 5860,
   CODE_FOR_subv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv4sf3 = 5861,
   CODE_FOR_addv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_addv4sf3_mask = 5862,
   CODE_FOR_addv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv4sf3 = 5863,
   CODE_FOR_subv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_subv4sf3_mask = 5864,
   CODE_FOR_subv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv8df3 = 5865,
  CODE_FOR_addv8df3_round = 5866,
  CODE_FOR_addv8df3_mask = 5867,
  CODE_FOR_addv8df3_mask_round = 5868,
  CODE_FOR_subv8df3 = 5869,
  CODE_FOR_subv8df3_round = 5870,
  CODE_FOR_subv8df3_mask = 5871,
  CODE_FOR_subv8df3_mask_round = 5872,
  CODE_FOR_addv4df3 = 5873,
   CODE_FOR_addv4df3_round = CODE_FOR_nothing,
  CODE_FOR_addv4df3_mask = 5874,
   CODE_FOR_addv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv4df3 = 5875,
   CODE_FOR_subv4df3_round = CODE_FOR_nothing,
  CODE_FOR_subv4df3_mask = 5876,
   CODE_FOR_subv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv2df3 = 5877,
   CODE_FOR_addv2df3_round = CODE_FOR_nothing,
  CODE_FOR_addv2df3_mask = 5878,
   CODE_FOR_addv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv2df3 = 5879,
   CODE_FOR_subv2df3_round = CODE_FOR_nothing,
  CODE_FOR_subv2df3_mask = 5880,
   CODE_FOR_subv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv16sf3 = 5881,
  CODE_FOR_mulv16sf3_round = 5882,
  CODE_FOR_mulv16sf3_mask = 5883,
  CODE_FOR_mulv16sf3_mask_round = 5884,
  CODE_FOR_mulv8sf3 = 5885,
   CODE_FOR_mulv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv8sf3_mask = 5886,
   CODE_FOR_mulv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv4sf3 = 5887,
   CODE_FOR_mulv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv4sf3_mask = 5888,
   CODE_FOR_mulv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv8df3 = 5889,
  CODE_FOR_mulv8df3_round = 5890,
  CODE_FOR_mulv8df3_mask = 5891,
  CODE_FOR_mulv8df3_mask_round = 5892,
  CODE_FOR_mulv4df3 = 5893,
   CODE_FOR_mulv4df3_round = CODE_FOR_nothing,
  CODE_FOR_mulv4df3_mask = 5894,
   CODE_FOR_mulv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv2df3 = 5895,
   CODE_FOR_mulv2df3_round = CODE_FOR_nothing,
  CODE_FOR_mulv2df3_mask = 5896,
   CODE_FOR_mulv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_divv8df3 = 5897,
  CODE_FOR_divv4df3 = 5898,
  CODE_FOR_divv2df3 = 5899,
  CODE_FOR_divv16sf3 = 5900,
  CODE_FOR_divv8sf3 = 5901,
  CODE_FOR_divv4sf3 = 5902,
  CODE_FOR_sqrtv8df2 = 5903,
  CODE_FOR_sqrtv4df2 = 5904,
  CODE_FOR_sqrtv2df2 = 5905,
  CODE_FOR_sqrtv16sf2 = 5906,
  CODE_FOR_sqrtv8sf2 = 5907,
  CODE_FOR_sqrtv4sf2 = 5908,
  CODE_FOR_rsqrtv8sf2 = 5909,
  CODE_FOR_rsqrtv4sf2 = 5910,
  CODE_FOR_rsqrtv16sf2 = 5911,
  CODE_FOR_smaxv16sf3 = 5912,
  CODE_FOR_smaxv16sf3_round = 5913,
  CODE_FOR_smaxv16sf3_mask = 5914,
  CODE_FOR_smaxv16sf3_mask_round = 5915,
  CODE_FOR_sminv16sf3 = 5916,
  CODE_FOR_sminv16sf3_round = 5917,
  CODE_FOR_sminv16sf3_mask = 5918,
  CODE_FOR_sminv16sf3_mask_round = 5919,
  CODE_FOR_smaxv8sf3 = 5920,
   CODE_FOR_smaxv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8sf3_mask = 5921,
   CODE_FOR_smaxv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv8sf3 = 5922,
   CODE_FOR_sminv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv8sf3_mask = 5923,
   CODE_FOR_sminv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4sf3 = 5924,
   CODE_FOR_smaxv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4sf3_mask = 5925,
   CODE_FOR_smaxv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv4sf3 = 5926,
   CODE_FOR_sminv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv4sf3_mask = 5927,
   CODE_FOR_sminv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8df3 = 5928,
  CODE_FOR_smaxv8df3_round = 5929,
  CODE_FOR_smaxv8df3_mask = 5930,
  CODE_FOR_smaxv8df3_mask_round = 5931,
  CODE_FOR_sminv8df3 = 5932,
  CODE_FOR_sminv8df3_round = 5933,
  CODE_FOR_sminv8df3_mask = 5934,
  CODE_FOR_sminv8df3_mask_round = 5935,
  CODE_FOR_smaxv4df3 = 5936,
   CODE_FOR_smaxv4df3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4df3_mask = 5937,
   CODE_FOR_smaxv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv4df3 = 5938,
   CODE_FOR_sminv4df3_round = CODE_FOR_nothing,
  CODE_FOR_sminv4df3_mask = 5939,
   CODE_FOR_sminv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv2df3 = 5940,
   CODE_FOR_smaxv2df3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv2df3_mask = 5941,
   CODE_FOR_smaxv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv2df3 = 5942,
   CODE_FOR_sminv2df3_round = CODE_FOR_nothing,
  CODE_FOR_sminv2df3_mask = 5943,
   CODE_FOR_sminv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse3_haddv2df3 = 5944,
  CODE_FOR_reduc_plus_scal_v8df = 5945,
  CODE_FOR_reduc_plus_scal_v4df = 5946,
  CODE_FOR_reduc_plus_scal_v2df = 5947,
  CODE_FOR_reduc_plus_scal_v16sf = 5948,
  CODE_FOR_reduc_plus_scal_v8sf = 5949,
  CODE_FOR_reduc_plus_scal_v4sf = 5950,
  CODE_FOR_reduc_smax_scal_v32qi = 5951,
  CODE_FOR_reduc_smin_scal_v32qi = 5952,
  CODE_FOR_reduc_smax_scal_v16hi = 5953,
  CODE_FOR_reduc_smin_scal_v16hi = 5954,
  CODE_FOR_reduc_smax_scal_v8si = 5955,
  CODE_FOR_reduc_smin_scal_v8si = 5956,
  CODE_FOR_reduc_smax_scal_v4di = 5957,
  CODE_FOR_reduc_smin_scal_v4di = 5958,
  CODE_FOR_reduc_smax_scal_v8sf = 5959,
  CODE_FOR_reduc_smin_scal_v8sf = 5960,
  CODE_FOR_reduc_smax_scal_v4df = 5961,
  CODE_FOR_reduc_smin_scal_v4df = 5962,
  CODE_FOR_reduc_smax_scal_v4sf = 5963,
  CODE_FOR_reduc_smin_scal_v4sf = 5964,
  CODE_FOR_reduc_smax_scal_v64qi = 5965,
  CODE_FOR_reduc_smin_scal_v64qi = 5966,
  CODE_FOR_reduc_smax_scal_v32hi = 5967,
  CODE_FOR_reduc_smin_scal_v32hi = 5968,
  CODE_FOR_reduc_smax_scal_v16si = 5969,
  CODE_FOR_reduc_smin_scal_v16si = 5970,
  CODE_FOR_reduc_smax_scal_v8di = 5971,
  CODE_FOR_reduc_smin_scal_v8di = 5972,
  CODE_FOR_reduc_smax_scal_v16sf = 5973,
  CODE_FOR_reduc_smin_scal_v16sf = 5974,
  CODE_FOR_reduc_smax_scal_v8df = 5975,
  CODE_FOR_reduc_smin_scal_v8df = 5976,
  CODE_FOR_reduc_umax_scal_v16si = 5977,
  CODE_FOR_reduc_umin_scal_v16si = 5978,
  CODE_FOR_reduc_umax_scal_v8di = 5979,
  CODE_FOR_reduc_umin_scal_v8di = 5980,
  CODE_FOR_reduc_umax_scal_v32hi = 5981,
  CODE_FOR_reduc_umin_scal_v32hi = 5982,
  CODE_FOR_reduc_umax_scal_v64qi = 5983,
  CODE_FOR_reduc_umin_scal_v64qi = 5984,
  CODE_FOR_reduc_umax_scal_v32qi = 5985,
  CODE_FOR_reduc_umin_scal_v32qi = 5986,
  CODE_FOR_reduc_umax_scal_v16hi = 5987,
  CODE_FOR_reduc_umin_scal_v16hi = 5988,
  CODE_FOR_reduc_umax_scal_v8si = 5989,
  CODE_FOR_reduc_umin_scal_v8si = 5990,
  CODE_FOR_reduc_umax_scal_v4di = 5991,
  CODE_FOR_reduc_umin_scal_v4di = 5992,
  CODE_FOR_reduc_umin_scal_v8hi = 5993,
  CODE_FOR_vec_cmpv16sihi = 5994,
  CODE_FOR_vec_cmpv8siqi = 5995,
  CODE_FOR_vec_cmpv4siqi = 5996,
  CODE_FOR_vec_cmpv8diqi = 5997,
  CODE_FOR_vec_cmpv4diqi = 5998,
  CODE_FOR_vec_cmpv2diqi = 5999,
  CODE_FOR_vec_cmpv16sfhi = 6000,
  CODE_FOR_vec_cmpv8sfqi = 6001,
  CODE_FOR_vec_cmpv4sfqi = 6002,
  CODE_FOR_vec_cmpv8dfqi = 6003,
  CODE_FOR_vec_cmpv4dfqi = 6004,
  CODE_FOR_vec_cmpv2dfqi = 6005,
  CODE_FOR_vec_cmpv64qidi = 6006,
  CODE_FOR_vec_cmpv16qihi = 6007,
  CODE_FOR_vec_cmpv32qisi = 6008,
  CODE_FOR_vec_cmpv32hisi = 6009,
  CODE_FOR_vec_cmpv16hihi = 6010,
  CODE_FOR_vec_cmpv8hiqi = 6011,
  CODE_FOR_vec_cmpv32qiv32qi = 6012,
  CODE_FOR_vec_cmpv16hiv16hi = 6013,
  CODE_FOR_vec_cmpv8siv8si = 6014,
  CODE_FOR_vec_cmpv4div4di = 6015,
  CODE_FOR_vec_cmpv16qiv16qi = 6016,
  CODE_FOR_vec_cmpv8hiv8hi = 6017,
  CODE_FOR_vec_cmpv4siv4si = 6018,
  CODE_FOR_vec_cmpv2div2di = 6019,
  CODE_FOR_vec_cmpv8sfv8si = 6020,
  CODE_FOR_vec_cmpv4dfv4di = 6021,
  CODE_FOR_vec_cmpv4sfv4si = 6022,
  CODE_FOR_vec_cmpv2dfv2di = 6023,
  CODE_FOR_vec_cmpuv16sihi = 6024,
  CODE_FOR_vec_cmpuv8siqi = 6025,
  CODE_FOR_vec_cmpuv4siqi = 6026,
  CODE_FOR_vec_cmpuv8diqi = 6027,
  CODE_FOR_vec_cmpuv4diqi = 6028,
  CODE_FOR_vec_cmpuv2diqi = 6029,
  CODE_FOR_vec_cmpuv64qidi = 6030,
  CODE_FOR_vec_cmpuv16qihi = 6031,
  CODE_FOR_vec_cmpuv32qisi = 6032,
  CODE_FOR_vec_cmpuv32hisi = 6033,
  CODE_FOR_vec_cmpuv16hihi = 6034,
  CODE_FOR_vec_cmpuv8hiqi = 6035,
  CODE_FOR_vec_cmpuv32qiv32qi = 6036,
  CODE_FOR_vec_cmpuv16hiv16hi = 6037,
  CODE_FOR_vec_cmpuv8siv8si = 6038,
  CODE_FOR_vec_cmpuv4div4di = 6039,
  CODE_FOR_vec_cmpuv16qiv16qi = 6040,
  CODE_FOR_vec_cmpuv8hiv8hi = 6041,
  CODE_FOR_vec_cmpuv4siv4si = 6042,
  CODE_FOR_vec_cmpuv2div2di = 6043,
  CODE_FOR_vec_cmpeqv2div2di = 6044,
  CODE_FOR_vcondv64qiv16sf = 6045,
  CODE_FOR_vcondv32hiv16sf = 6046,
  CODE_FOR_vcondv16siv16sf = 6047,
  CODE_FOR_vcondv8div16sf = 6048,
  CODE_FOR_vcondv16sfv16sf = 6049,
  CODE_FOR_vcondv8dfv16sf = 6050,
  CODE_FOR_vcondv64qiv8df = 6051,
  CODE_FOR_vcondv32hiv8df = 6052,
  CODE_FOR_vcondv16siv8df = 6053,
  CODE_FOR_vcondv8div8df = 6054,
  CODE_FOR_vcondv16sfv8df = 6055,
  CODE_FOR_vcondv8dfv8df = 6056,
  CODE_FOR_vcondv32qiv8sf = 6057,
  CODE_FOR_vcondv32qiv4df = 6058,
  CODE_FOR_vcondv16hiv8sf = 6059,
  CODE_FOR_vcondv16hiv4df = 6060,
  CODE_FOR_vcondv8siv8sf = 6061,
  CODE_FOR_vcondv8siv4df = 6062,
  CODE_FOR_vcondv4div8sf = 6063,
  CODE_FOR_vcondv4div4df = 6064,
  CODE_FOR_vcondv8sfv8sf = 6065,
  CODE_FOR_vcondv8sfv4df = 6066,
  CODE_FOR_vcondv4dfv8sf = 6067,
  CODE_FOR_vcondv4dfv4df = 6068,
  CODE_FOR_vcondv16qiv4sf = 6069,
  CODE_FOR_vcondv16qiv2df = 6070,
  CODE_FOR_vcondv8hiv4sf = 6071,
  CODE_FOR_vcondv8hiv2df = 6072,
  CODE_FOR_vcondv4siv4sf = 6073,
  CODE_FOR_vcondv4siv2df = 6074,
  CODE_FOR_vcondv2div4sf = 6075,
  CODE_FOR_vcondv2div2df = 6076,
  CODE_FOR_vcondv4sfv4sf = 6077,
  CODE_FOR_vcondv4sfv2df = 6078,
  CODE_FOR_vcondv2dfv4sf = 6079,
  CODE_FOR_vcondv2dfv2df = 6080,
  CODE_FOR_vcond_mask_v16sihi = 6081,
  CODE_FOR_vcond_mask_v8siqi = 6082,
  CODE_FOR_vcond_mask_v4siqi = 6083,
  CODE_FOR_vcond_mask_v8diqi = 6084,
  CODE_FOR_vcond_mask_v4diqi = 6085,
  CODE_FOR_vcond_mask_v2diqi = 6086,
  CODE_FOR_vcond_mask_v16sfhi = 6087,
  CODE_FOR_vcond_mask_v8sfqi = 6088,
  CODE_FOR_vcond_mask_v4sfqi = 6089,
  CODE_FOR_vcond_mask_v8dfqi = 6090,
  CODE_FOR_vcond_mask_v4dfqi = 6091,
  CODE_FOR_vcond_mask_v2dfqi = 6092,
  CODE_FOR_vcond_mask_v64qidi = 6093,
  CODE_FOR_vcond_mask_v16qihi = 6094,
  CODE_FOR_vcond_mask_v32qisi = 6095,
  CODE_FOR_vcond_mask_v32hisi = 6096,
  CODE_FOR_vcond_mask_v16hihi = 6097,
  CODE_FOR_vcond_mask_v8hiqi = 6098,
  CODE_FOR_vcond_mask_v32qiv32qi = 6099,
  CODE_FOR_vcond_mask_v16hiv16hi = 6100,
  CODE_FOR_vcond_mask_v8siv8si = 6101,
  CODE_FOR_vcond_mask_v4div4di = 6102,
  CODE_FOR_vcond_mask_v16qiv16qi = 6103,
  CODE_FOR_vcond_mask_v8hiv8hi = 6104,
  CODE_FOR_vcond_mask_v4siv4si = 6105,
  CODE_FOR_vcond_mask_v2div2di = 6106,
  CODE_FOR_vcond_mask_v8sfv8si = 6107,
  CODE_FOR_vcond_mask_v4dfv4di = 6108,
  CODE_FOR_vcond_mask_v4sfv4si = 6109,
  CODE_FOR_vcond_mask_v2dfv2di = 6110,
  CODE_FOR_andv8sf3 = 6111,
  CODE_FOR_andv8sf3_mask = 6112,
  CODE_FOR_iorv8sf3 = 6113,
  CODE_FOR_iorv8sf3_mask = 6114,
  CODE_FOR_xorv8sf3 = 6115,
  CODE_FOR_xorv8sf3_mask = 6116,
  CODE_FOR_andv4sf3 = 6117,
  CODE_FOR_andv4sf3_mask = 6118,
  CODE_FOR_iorv4sf3 = 6119,
  CODE_FOR_iorv4sf3_mask = 6120,
  CODE_FOR_xorv4sf3 = 6121,
  CODE_FOR_xorv4sf3_mask = 6122,
  CODE_FOR_andv4df3 = 6123,
  CODE_FOR_andv4df3_mask = 6124,
  CODE_FOR_iorv4df3 = 6125,
  CODE_FOR_iorv4df3_mask = 6126,
  CODE_FOR_xorv4df3 = 6127,
  CODE_FOR_xorv4df3_mask = 6128,
  CODE_FOR_andv2df3 = 6129,
  CODE_FOR_andv2df3_mask = 6130,
  CODE_FOR_iorv2df3 = 6131,
  CODE_FOR_iorv2df3_mask = 6132,
  CODE_FOR_xorv2df3 = 6133,
  CODE_FOR_xorv2df3_mask = 6134,
  CODE_FOR_andv16sf3 = 6135,
  CODE_FOR_andv16sf3_mask = 6136,
  CODE_FOR_iorv16sf3 = 6137,
  CODE_FOR_iorv16sf3_mask = 6138,
  CODE_FOR_xorv16sf3 = 6139,
  CODE_FOR_xorv16sf3_mask = 6140,
  CODE_FOR_andv8df3 = 6141,
  CODE_FOR_andv8df3_mask = 6142,
  CODE_FOR_iorv8df3 = 6143,
  CODE_FOR_iorv8df3_mask = 6144,
  CODE_FOR_xorv8df3 = 6145,
  CODE_FOR_xorv8df3_mask = 6146,
  CODE_FOR_copysignv16sf3 = 6147,
  CODE_FOR_copysignv8sf3 = 6148,
  CODE_FOR_copysignv4sf3 = 6149,
  CODE_FOR_copysignv8df3 = 6150,
  CODE_FOR_copysignv4df3 = 6151,
  CODE_FOR_copysignv2df3 = 6152,
  CODE_FOR_andtf3 = 6153,
  CODE_FOR_iortf3 = 6154,
  CODE_FOR_xortf3 = 6155,
  CODE_FOR_fmasf4 = 6156,
  CODE_FOR_fmadf4 = 6157,
  CODE_FOR_fmav4sf4 = 6158,
  CODE_FOR_fmav2df4 = 6159,
  CODE_FOR_fmav8sf4 = 6160,
  CODE_FOR_fmav4df4 = 6161,
  CODE_FOR_fmav16sf4 = 6162,
  CODE_FOR_fmav8df4 = 6163,
  CODE_FOR_fmssf4 = 6164,
  CODE_FOR_fmsdf4 = 6165,
  CODE_FOR_fmsv4sf4 = 6166,
  CODE_FOR_fmsv2df4 = 6167,
  CODE_FOR_fmsv8sf4 = 6168,
  CODE_FOR_fmsv4df4 = 6169,
  CODE_FOR_fmsv16sf4 = 6170,
  CODE_FOR_fmsv8df4 = 6171,
  CODE_FOR_fnmasf4 = 6172,
  CODE_FOR_fnmadf4 = 6173,
  CODE_FOR_fnmav4sf4 = 6174,
  CODE_FOR_fnmav2df4 = 6175,
  CODE_FOR_fnmav8sf4 = 6176,
  CODE_FOR_fnmav4df4 = 6177,
  CODE_FOR_fnmav16sf4 = 6178,
  CODE_FOR_fnmav8df4 = 6179,
  CODE_FOR_fnmssf4 = 6180,
  CODE_FOR_fnmsdf4 = 6181,
  CODE_FOR_fnmsv4sf4 = 6182,
  CODE_FOR_fnmsv2df4 = 6183,
  CODE_FOR_fnmsv8sf4 = 6184,
  CODE_FOR_fnmsv4df4 = 6185,
  CODE_FOR_fnmsv16sf4 = 6186,
  CODE_FOR_fnmsv8df4 = 6187,
  CODE_FOR_fma4i_fmadd_sf = 6188,
  CODE_FOR_fma4i_fmadd_df = 6189,
  CODE_FOR_fma4i_fmadd_v4sf = 6190,
  CODE_FOR_fma4i_fmadd_v2df = 6191,
  CODE_FOR_fma4i_fmadd_v8sf = 6192,
  CODE_FOR_fma4i_fmadd_v4df = 6193,
  CODE_FOR_fma4i_fmadd_v16sf = 6194,
  CODE_FOR_fma4i_fmadd_v8df = 6195,
  CODE_FOR_avx512f_fmadd_v16sf_maskz = 6196,
  CODE_FOR_avx512f_fmadd_v16sf_maskz_round = 6197,
  CODE_FOR_avx512vl_fmadd_v8sf_maskz = 6198,
  CODE_FOR_avx512vl_fmadd_v8sf_maskz_round = 6199,
  CODE_FOR_avx512vl_fmadd_v4sf_maskz = 6200,
  CODE_FOR_avx512vl_fmadd_v4sf_maskz_round = 6201,
  CODE_FOR_avx512f_fmadd_v8df_maskz = 6202,
  CODE_FOR_avx512f_fmadd_v8df_maskz_round = 6203,
  CODE_FOR_avx512vl_fmadd_v4df_maskz = 6204,
  CODE_FOR_avx512vl_fmadd_v4df_maskz_round = 6205,
  CODE_FOR_avx512vl_fmadd_v2df_maskz = 6206,
  CODE_FOR_avx512vl_fmadd_v2df_maskz_round = 6207,
  CODE_FOR_fmaddsub_v16sf = 6208,
  CODE_FOR_fmaddsub_v8sf = 6209,
  CODE_FOR_fmaddsub_v4sf = 6210,
  CODE_FOR_fmaddsub_v8df = 6211,
  CODE_FOR_fmaddsub_v4df = 6212,
  CODE_FOR_fmaddsub_v2df = 6213,
  CODE_FOR_avx512f_fmaddsub_v16sf_maskz = 6214,
  CODE_FOR_avx512f_fmaddsub_v16sf_maskz_round = 6215,
  CODE_FOR_avx512vl_fmaddsub_v8sf_maskz = 6216,
  CODE_FOR_avx512vl_fmaddsub_v8sf_maskz_round = 6217,
  CODE_FOR_avx512vl_fmaddsub_v4sf_maskz = 6218,
  CODE_FOR_avx512vl_fmaddsub_v4sf_maskz_round = 6219,
  CODE_FOR_avx512f_fmaddsub_v8df_maskz = 6220,
  CODE_FOR_avx512f_fmaddsub_v8df_maskz_round = 6221,
  CODE_FOR_avx512vl_fmaddsub_v4df_maskz = 6222,
  CODE_FOR_avx512vl_fmaddsub_v4df_maskz_round = 6223,
  CODE_FOR_avx512vl_fmaddsub_v2df_maskz = 6224,
  CODE_FOR_avx512vl_fmaddsub_v2df_maskz_round = 6225,
  CODE_FOR_fmai_vmfmadd_v4sf = 6226,
  CODE_FOR_fmai_vmfmadd_v4sf_round = 6227,
  CODE_FOR_fmai_vmfmadd_v2df = 6228,
  CODE_FOR_fmai_vmfmadd_v2df_round = 6229,
  CODE_FOR_fma4i_vmfmadd_v4sf = 6230,
  CODE_FOR_fma4i_vmfmadd_v2df = 6231,
  CODE_FOR_floatunsv16siv16sf2 = 6232,
  CODE_FOR_floatunsv8siv8sf2 = 6233,
  CODE_FOR_floatunsv4siv4sf2 = 6234,
  CODE_FOR_fixuns_truncv16sfv16si2 = 6235,
  CODE_FOR_fixuns_truncv8sfv8si2 = 6236,
  CODE_FOR_fixuns_truncv4sfv4si2 = 6237,
  CODE_FOR_avx_cvtpd2dq256_2 = 6238,
  CODE_FOR_avx_cvttpd2dq256_2 = 6239,
  CODE_FOR_sse2_cvtpd2ps = 6240,
  CODE_FOR_sse2_cvtpd2ps_mask = 6241,
  CODE_FOR_avx512bw_cvtmask2bv64qi = 6242,
  CODE_FOR_avx512vl_cvtmask2bv16qi = 6243,
  CODE_FOR_avx512vl_cvtmask2bv32qi = 6244,
  CODE_FOR_avx512bw_cvtmask2wv32hi = 6245,
  CODE_FOR_avx512vl_cvtmask2wv16hi = 6246,
  CODE_FOR_avx512vl_cvtmask2wv8hi = 6247,
  CODE_FOR_avx512f_cvtmask2dv16si = 6248,
  CODE_FOR_avx512vl_cvtmask2dv8si = 6249,
  CODE_FOR_avx512vl_cvtmask2dv4si = 6250,
  CODE_FOR_avx512f_cvtmask2qv8di = 6251,
  CODE_FOR_avx512vl_cvtmask2qv4di = 6252,
  CODE_FOR_avx512vl_cvtmask2qv2di = 6253,
  CODE_FOR_vec_unpacks_hi_v4sf = 6254,
  CODE_FOR_vec_unpacks_hi_v8sf = 6255,
  CODE_FOR_vec_unpacks_hi_v16sf = 6256,
  CODE_FOR_vec_unpacks_lo_v4sf = 6257,
  CODE_FOR_vec_unpacks_lo_v8sf = 6258,
  CODE_FOR_vec_unpacks_float_hi_v32hi = 6259,
  CODE_FOR_vec_unpacks_float_hi_v16hi = 6260,
  CODE_FOR_vec_unpacks_float_hi_v8hi = 6261,
  CODE_FOR_vec_unpacks_float_lo_v32hi = 6262,
  CODE_FOR_vec_unpacks_float_lo_v16hi = 6263,
  CODE_FOR_vec_unpacks_float_lo_v8hi = 6264,
  CODE_FOR_vec_unpacku_float_hi_v32hi = 6265,
  CODE_FOR_vec_unpacku_float_hi_v16hi = 6266,
  CODE_FOR_vec_unpacku_float_hi_v8hi = 6267,
  CODE_FOR_vec_unpacku_float_lo_v32hi = 6268,
  CODE_FOR_vec_unpacku_float_lo_v16hi = 6269,
  CODE_FOR_vec_unpacku_float_lo_v8hi = 6270,
  CODE_FOR_vec_unpacks_float_hi_v4si = 6271,
  CODE_FOR_vec_unpacks_float_lo_v4si = 6272,
  CODE_FOR_vec_unpacks_float_hi_v8si = 6273,
  CODE_FOR_vec_unpacks_float_lo_v8si = 6274,
  CODE_FOR_vec_unpacks_float_hi_v16si = 6275,
  CODE_FOR_vec_unpacks_float_lo_v16si = 6276,
  CODE_FOR_vec_unpacku_float_hi_v4si = 6277,
  CODE_FOR_vec_unpacku_float_lo_v4si = 6278,
  CODE_FOR_vec_unpacku_float_hi_v8si = 6279,
  CODE_FOR_vec_unpacku_float_hi_v16si = 6280,
  CODE_FOR_vec_unpacku_float_lo_v8si = 6281,
  CODE_FOR_vec_unpacku_float_lo_v16si = 6282,
  CODE_FOR_vec_pack_trunc_v8df = 6283,
  CODE_FOR_vec_pack_trunc_v4df = 6284,
  CODE_FOR_vec_pack_trunc_v2df = 6285,
  CODE_FOR_vec_pack_sfix_trunc_v8df = 6286,
  CODE_FOR_vec_pack_sfix_trunc_v4df = 6287,
  CODE_FOR_vec_pack_sfix_trunc_v2df = 6288,
  CODE_FOR_vec_pack_ufix_trunc_v8df = 6289,
  CODE_FOR_vec_pack_ufix_trunc_v4df = 6290,
  CODE_FOR_vec_pack_ufix_trunc_v2df = 6291,
  CODE_FOR_avx512f_vec_pack_sfix_v8df = 6292,
  CODE_FOR_vec_pack_sfix_v4df = 6293,
  CODE_FOR_vec_pack_sfix_v2df = 6294,
  CODE_FOR_sse_movhlps_exp = 6295,
  CODE_FOR_sse_movlhps_exp = 6296,
  CODE_FOR_vec_interleave_highv8sf = 6297,
  CODE_FOR_vec_interleave_lowv8sf = 6298,
  CODE_FOR_avx_shufps256 = 6299,
  CODE_FOR_avx_shufps256_mask = 6300,
  CODE_FOR_sse_shufps = 6301,
  CODE_FOR_sse_shufps_mask = 6302,
  CODE_FOR_sse_loadhps_exp = 6303,
  CODE_FOR_sse_loadlps_exp = 6304,
  CODE_FOR_vec_setv64qi = 6305,
  CODE_FOR_vec_setv32qi = 6306,
  CODE_FOR_vec_setv16qi = 6307,
  CODE_FOR_vec_setv32hi = 6308,
  CODE_FOR_vec_setv16hi = 6309,
  CODE_FOR_vec_setv8hi = 6310,
  CODE_FOR_vec_setv16si = 6311,
  CODE_FOR_vec_setv8si = 6312,
  CODE_FOR_vec_setv4si = 6313,
  CODE_FOR_vec_setv8di = 6314,
  CODE_FOR_vec_setv4di = 6315,
  CODE_FOR_vec_setv2di = 6316,
  CODE_FOR_vec_setv16sf = 6317,
  CODE_FOR_vec_setv8sf = 6318,
  CODE_FOR_vec_setv4sf = 6319,
  CODE_FOR_vec_setv8df = 6320,
  CODE_FOR_vec_setv4df = 6321,
  CODE_FOR_vec_setv2df = 6322,
  CODE_FOR_avx512dq_vextractf64x2_mask = 6323,
  CODE_FOR_avx512dq_vextracti64x2_mask = 6324,
  CODE_FOR_avx512f_vextractf32x4_mask = 6325,
  CODE_FOR_avx512f_vextracti32x4_mask = 6326,
  CODE_FOR_avx512dq_vextractf32x8_mask = 6327,
  CODE_FOR_avx512dq_vextracti32x8_mask = 6328,
  CODE_FOR_avx512f_vextractf64x4_mask = 6329,
  CODE_FOR_avx512f_vextracti64x4_mask = 6330,
  CODE_FOR_avx512vl_vextractf128v8si = 6331,
  CODE_FOR_avx512vl_vextractf128v8sf = 6332,
  CODE_FOR_avx512vl_vextractf128v4di = 6333,
  CODE_FOR_avx512vl_vextractf128v4df = 6334,
  CODE_FOR_avx_vextractf128v32qi = 6335,
  CODE_FOR_avx_vextractf128v16hi = 6336,
  CODE_FOR_avx_vextractf128v8si = 6337,
  CODE_FOR_avx_vextractf128v4di = 6338,
  CODE_FOR_avx_vextractf128v8sf = 6339,
  CODE_FOR_avx_vextractf128v4df = 6340,
  CODE_FOR_vec_extractv64qiqi = 6341,
  CODE_FOR_vec_extractv32qiqi = 6342,
  CODE_FOR_vec_extractv16qiqi = 6343,
  CODE_FOR_vec_extractv32hihi = 6344,
  CODE_FOR_vec_extractv16hihi = 6345,
  CODE_FOR_vec_extractv8hihi = 6346,
  CODE_FOR_vec_extractv16sisi = 6347,
  CODE_FOR_vec_extractv8sisi = 6348,
  CODE_FOR_vec_extractv4sisi = 6349,
  CODE_FOR_vec_extractv8didi = 6350,
  CODE_FOR_vec_extractv4didi = 6351,
  CODE_FOR_vec_extractv2didi = 6352,
  CODE_FOR_vec_extractv16sfsf = 6353,
  CODE_FOR_vec_extractv8sfsf = 6354,
  CODE_FOR_vec_extractv4sfsf = 6355,
  CODE_FOR_vec_extractv8dfdf = 6356,
  CODE_FOR_vec_extractv4dfdf = 6357,
  CODE_FOR_vec_extractv2dfdf = 6358,
  CODE_FOR_vec_extractv4titi = 6359,
  CODE_FOR_vec_extractv2titi = 6360,
  CODE_FOR_vec_extractv64qiv32qi = 6361,
  CODE_FOR_vec_extractv32hiv16hi = 6362,
  CODE_FOR_vec_extractv16siv8si = 6363,
  CODE_FOR_vec_extractv8div4di = 6364,
  CODE_FOR_vec_extractv16sfv8sf = 6365,
  CODE_FOR_vec_extractv8dfv4df = 6366,
  CODE_FOR_vec_interleave_highv4df = 6367,
  CODE_FOR_vec_interleave_highv2df = 6368,
  CODE_FOR_avx512f_movddup512 = 6369,
  CODE_FOR_avx512f_movddup512_mask = 6370,
  CODE_FOR_avx512f_unpcklpd512 = 6371,
  CODE_FOR_avx512f_unpcklpd512_mask = 6372,
  CODE_FOR_avx_movddup256 = 6373,
  CODE_FOR_avx_movddup256_mask = 6374,
  CODE_FOR_avx_unpcklpd256 = 6375,
  CODE_FOR_avx_unpcklpd256_mask = 6376,
  CODE_FOR_vec_interleave_lowv4df = 6377,
  CODE_FOR_vec_interleave_lowv2df = 6378,
  CODE_FOR_avx512f_vternlogv16si_maskz = 6379,
  CODE_FOR_avx512vl_vternlogv8si_maskz = 6380,
  CODE_FOR_avx512vl_vternlogv4si_maskz = 6381,
  CODE_FOR_avx512f_vternlogv8di_maskz = 6382,
  CODE_FOR_avx512vl_vternlogv4di_maskz = 6383,
  CODE_FOR_avx512vl_vternlogv2di_maskz = 6384,
  CODE_FOR_avx512f_shufps512_mask = 6385,
  CODE_FOR_avx512f_fixupimmv16sf_maskz = 6386,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_round = 6387,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz = 6388,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_round = 6389,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz = 6390,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_round = 6391,
  CODE_FOR_avx512f_fixupimmv8df_maskz = 6392,
  CODE_FOR_avx512f_fixupimmv8df_maskz_round = 6393,
  CODE_FOR_avx512vl_fixupimmv4df_maskz = 6394,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_round = 6395,
  CODE_FOR_avx512vl_fixupimmv2df_maskz = 6396,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_round = 6397,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz = 6398,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_round = 6399,
  CODE_FOR_avx512f_sfixupimmv2df_maskz = 6400,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_round = 6401,
  CODE_FOR_avx512f_shufpd512_mask = 6402,
  CODE_FOR_avx_shufpd256 = 6403,
  CODE_FOR_avx_shufpd256_mask = 6404,
  CODE_FOR_sse2_shufpd = 6405,
  CODE_FOR_sse2_shufpd_mask = 6406,
  CODE_FOR_sse2_loadhpd_exp = 6407,
  CODE_FOR_sse2_loadlpd_exp = 6408,
  CODE_FOR_avx512f_ss_truncatev16siv16qi2_mask_store = 6409,
  CODE_FOR_avx512f_truncatev16siv16qi2_mask_store = 6410,
  CODE_FOR_avx512f_us_truncatev16siv16qi2_mask_store = 6411,
  CODE_FOR_avx512f_ss_truncatev16siv16hi2_mask_store = 6412,
  CODE_FOR_avx512f_truncatev16siv16hi2_mask_store = 6413,
  CODE_FOR_avx512f_us_truncatev16siv16hi2_mask_store = 6414,
  CODE_FOR_avx512f_ss_truncatev8div8si2_mask_store = 6415,
  CODE_FOR_avx512f_truncatev8div8si2_mask_store = 6416,
  CODE_FOR_avx512f_us_truncatev8div8si2_mask_store = 6417,
  CODE_FOR_avx512f_ss_truncatev8div8hi2_mask_store = 6418,
  CODE_FOR_avx512f_truncatev8div8hi2_mask_store = 6419,
  CODE_FOR_avx512f_us_truncatev8div8hi2_mask_store = 6420,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2_mask_store = 6421,
  CODE_FOR_avx512bw_truncatev32hiv32qi2_mask_store = 6422,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2_mask_store = 6423,
  CODE_FOR_avx512vl_ss_truncatev4div4si2_mask_store = 6424,
  CODE_FOR_avx512vl_truncatev4div4si2_mask_store = 6425,
  CODE_FOR_avx512vl_us_truncatev4div4si2_mask_store = 6426,
  CODE_FOR_avx512vl_ss_truncatev8siv8hi2_mask_store = 6427,
  CODE_FOR_avx512vl_truncatev8siv8hi2_mask_store = 6428,
  CODE_FOR_avx512vl_us_truncatev8siv8hi2_mask_store = 6429,
  CODE_FOR_avx512vl_ss_truncatev16hiv16qi2_mask_store = 6430,
  CODE_FOR_avx512vl_truncatev16hiv16qi2_mask_store = 6431,
  CODE_FOR_avx512vl_us_truncatev16hiv16qi2_mask_store = 6432,
  CODE_FOR_negv64qi2 = 6433,
  CODE_FOR_negv32qi2 = 6434,
  CODE_FOR_negv16qi2 = 6435,
  CODE_FOR_negv32hi2 = 6436,
  CODE_FOR_negv16hi2 = 6437,
  CODE_FOR_negv8hi2 = 6438,
  CODE_FOR_negv16si2 = 6439,
  CODE_FOR_negv8si2 = 6440,
  CODE_FOR_negv4si2 = 6441,
  CODE_FOR_negv8di2 = 6442,
  CODE_FOR_negv4di2 = 6443,
  CODE_FOR_negv2di2 = 6444,
  CODE_FOR_addv64qi3 = 6445,
  CODE_FOR_subv64qi3 = 6446,
  CODE_FOR_addv32qi3 = 6447,
  CODE_FOR_subv32qi3 = 6448,
  CODE_FOR_addv16qi3 = 6449,
  CODE_FOR_subv16qi3 = 6450,
  CODE_FOR_addv32hi3 = 6451,
  CODE_FOR_subv32hi3 = 6452,
  CODE_FOR_addv16hi3 = 6453,
  CODE_FOR_subv16hi3 = 6454,
  CODE_FOR_addv8hi3 = 6455,
  CODE_FOR_subv8hi3 = 6456,
  CODE_FOR_addv16si3 = 6457,
  CODE_FOR_subv16si3 = 6458,
  CODE_FOR_addv8si3 = 6459,
  CODE_FOR_subv8si3 = 6460,
  CODE_FOR_addv4si3 = 6461,
  CODE_FOR_subv4si3 = 6462,
  CODE_FOR_addv8di3 = 6463,
  CODE_FOR_subv8di3 = 6464,
  CODE_FOR_addv4di3 = 6465,
  CODE_FOR_subv4di3 = 6466,
  CODE_FOR_addv2di3 = 6467,
  CODE_FOR_subv2di3 = 6468,
  CODE_FOR_addv16si3_mask = 6469,
  CODE_FOR_subv16si3_mask = 6470,
  CODE_FOR_addv8si3_mask = 6471,
  CODE_FOR_subv8si3_mask = 6472,
  CODE_FOR_addv4si3_mask = 6473,
  CODE_FOR_subv4si3_mask = 6474,
  CODE_FOR_addv8di3_mask = 6475,
  CODE_FOR_subv8di3_mask = 6476,
  CODE_FOR_addv4di3_mask = 6477,
  CODE_FOR_subv4di3_mask = 6478,
  CODE_FOR_addv2di3_mask = 6479,
  CODE_FOR_subv2di3_mask = 6480,
  CODE_FOR_addv64qi3_mask = 6481,
  CODE_FOR_subv64qi3_mask = 6482,
  CODE_FOR_addv16qi3_mask = 6483,
  CODE_FOR_subv16qi3_mask = 6484,
  CODE_FOR_addv32qi3_mask = 6485,
  CODE_FOR_subv32qi3_mask = 6486,
  CODE_FOR_addv32hi3_mask = 6487,
  CODE_FOR_subv32hi3_mask = 6488,
  CODE_FOR_addv16hi3_mask = 6489,
  CODE_FOR_subv16hi3_mask = 6490,
  CODE_FOR_addv8hi3_mask = 6491,
  CODE_FOR_subv8hi3_mask = 6492,
  CODE_FOR_avx512bw_ssaddv64qi3 = 6493,
  CODE_FOR_avx512bw_ssaddv64qi3_mask = 6494,
  CODE_FOR_avx512bw_usaddv64qi3 = 6495,
  CODE_FOR_avx512bw_usaddv64qi3_mask = 6496,
  CODE_FOR_avx512bw_sssubv64qi3 = 6497,
  CODE_FOR_avx512bw_sssubv64qi3_mask = 6498,
  CODE_FOR_avx512bw_ussubv64qi3 = 6499,
  CODE_FOR_avx512bw_ussubv64qi3_mask = 6500,
  CODE_FOR_avx2_ssaddv32qi3 = 6501,
  CODE_FOR_avx2_ssaddv32qi3_mask = 6502,
  CODE_FOR_avx2_usaddv32qi3 = 6503,
  CODE_FOR_avx2_usaddv32qi3_mask = 6504,
  CODE_FOR_avx2_sssubv32qi3 = 6505,
  CODE_FOR_avx2_sssubv32qi3_mask = 6506,
  CODE_FOR_avx2_ussubv32qi3 = 6507,
  CODE_FOR_avx2_ussubv32qi3_mask = 6508,
  CODE_FOR_sse2_ssaddv16qi3 = 6509,
  CODE_FOR_sse2_ssaddv16qi3_mask = 6510,
  CODE_FOR_sse2_usaddv16qi3 = 6511,
  CODE_FOR_sse2_usaddv16qi3_mask = 6512,
  CODE_FOR_sse2_sssubv16qi3 = 6513,
  CODE_FOR_sse2_sssubv16qi3_mask = 6514,
  CODE_FOR_sse2_ussubv16qi3 = 6515,
  CODE_FOR_sse2_ussubv16qi3_mask = 6516,
  CODE_FOR_avx512bw_ssaddv32hi3 = 6517,
  CODE_FOR_avx512bw_ssaddv32hi3_mask = 6518,
  CODE_FOR_avx512bw_usaddv32hi3 = 6519,
  CODE_FOR_avx512bw_usaddv32hi3_mask = 6520,
  CODE_FOR_avx512bw_sssubv32hi3 = 6521,
  CODE_FOR_avx512bw_sssubv32hi3_mask = 6522,
  CODE_FOR_avx512bw_ussubv32hi3 = 6523,
  CODE_FOR_avx512bw_ussubv32hi3_mask = 6524,
  CODE_FOR_avx2_ssaddv16hi3 = 6525,
  CODE_FOR_avx2_ssaddv16hi3_mask = 6526,
  CODE_FOR_avx2_usaddv16hi3 = 6527,
  CODE_FOR_avx2_usaddv16hi3_mask = 6528,
  CODE_FOR_avx2_sssubv16hi3 = 6529,
  CODE_FOR_avx2_sssubv16hi3_mask = 6530,
  CODE_FOR_avx2_ussubv16hi3 = 6531,
  CODE_FOR_avx2_ussubv16hi3_mask = 6532,
  CODE_FOR_sse2_ssaddv8hi3 = 6533,
  CODE_FOR_sse2_ssaddv8hi3_mask = 6534,
  CODE_FOR_sse2_usaddv8hi3 = 6535,
  CODE_FOR_sse2_usaddv8hi3_mask = 6536,
  CODE_FOR_sse2_sssubv8hi3 = 6537,
  CODE_FOR_sse2_sssubv8hi3_mask = 6538,
  CODE_FOR_sse2_ussubv8hi3 = 6539,
  CODE_FOR_sse2_ussubv8hi3_mask = 6540,
  CODE_FOR_mulv64qi3 = 6541,
  CODE_FOR_mulv64qi3_mask = 6542,
  CODE_FOR_mulv32qi3 = 6543,
  CODE_FOR_mulv32qi3_mask = 6544,
  CODE_FOR_mulv16qi3 = 6545,
  CODE_FOR_mulv16qi3_mask = 6546,
  CODE_FOR_mulv32hi3 = 6547,
  CODE_FOR_mulv32hi3_mask = 6548,
  CODE_FOR_mulv16hi3 = 6549,
  CODE_FOR_mulv16hi3_mask = 6550,
  CODE_FOR_mulv8hi3 = 6551,
  CODE_FOR_mulv8hi3_mask = 6552,
  CODE_FOR_smulv32hi3_highpart = 6553,
  CODE_FOR_smulv32hi3_highpart_mask = 6554,
  CODE_FOR_umulv32hi3_highpart = 6555,
  CODE_FOR_umulv32hi3_highpart_mask = 6556,
  CODE_FOR_smulv16hi3_highpart = 6557,
  CODE_FOR_smulv16hi3_highpart_mask = 6558,
  CODE_FOR_umulv16hi3_highpart = 6559,
  CODE_FOR_umulv16hi3_highpart_mask = 6560,
  CODE_FOR_smulv8hi3_highpart = 6561,
  CODE_FOR_smulv8hi3_highpart_mask = 6562,
  CODE_FOR_umulv8hi3_highpart = 6563,
  CODE_FOR_umulv8hi3_highpart_mask = 6564,
  CODE_FOR_vec_widen_umult_even_v16si = 6565,
  CODE_FOR_vec_widen_umult_even_v16si_mask = 6566,
  CODE_FOR_vec_widen_umult_even_v8si = 6567,
  CODE_FOR_vec_widen_umult_even_v8si_mask = 6568,
  CODE_FOR_vec_widen_umult_even_v4si = 6569,
  CODE_FOR_vec_widen_umult_even_v4si_mask = 6570,
  CODE_FOR_vec_widen_smult_even_v16si = 6571,
  CODE_FOR_vec_widen_smult_even_v16si_mask = 6572,
  CODE_FOR_vec_widen_smult_even_v8si = 6573,
  CODE_FOR_vec_widen_smult_even_v8si_mask = 6574,
  CODE_FOR_sse4_1_mulv2siv2di3 = 6575,
  CODE_FOR_sse4_1_mulv2siv2di3_mask = 6576,
  CODE_FOR_avx2_pmaddwd = 6577,
  CODE_FOR_sse2_pmaddwd = 6578,
  CODE_FOR_mulv16si3 = 6579,
  CODE_FOR_mulv16si3_mask = 6580,
  CODE_FOR_mulv8si3 = 6581,
  CODE_FOR_mulv8si3_mask = 6582,
  CODE_FOR_mulv4si3 = 6583,
  CODE_FOR_mulv4si3_mask = 6584,
  CODE_FOR_mulv8di3 = 6585,
  CODE_FOR_mulv4di3 = 6586,
  CODE_FOR_mulv2di3 = 6587,
  CODE_FOR_vec_widen_smult_hi_v32qi = 6588,
  CODE_FOR_vec_widen_umult_hi_v32qi = 6589,
  CODE_FOR_vec_widen_smult_hi_v16qi = 6590,
  CODE_FOR_vec_widen_umult_hi_v16qi = 6591,
  CODE_FOR_vec_widen_smult_hi_v16hi = 6592,
  CODE_FOR_vec_widen_umult_hi_v16hi = 6593,
  CODE_FOR_vec_widen_smult_hi_v8hi = 6594,
  CODE_FOR_vec_widen_umult_hi_v8hi = 6595,
  CODE_FOR_vec_widen_smult_hi_v8si = 6596,
  CODE_FOR_vec_widen_umult_hi_v8si = 6597,
  CODE_FOR_vec_widen_smult_hi_v4si = 6598,
  CODE_FOR_vec_widen_umult_hi_v4si = 6599,
  CODE_FOR_vec_widen_smult_lo_v32qi = 6600,
  CODE_FOR_vec_widen_umult_lo_v32qi = 6601,
  CODE_FOR_vec_widen_smult_lo_v16qi = 6602,
  CODE_FOR_vec_widen_umult_lo_v16qi = 6603,
  CODE_FOR_vec_widen_smult_lo_v16hi = 6604,
  CODE_FOR_vec_widen_umult_lo_v16hi = 6605,
  CODE_FOR_vec_widen_smult_lo_v8hi = 6606,
  CODE_FOR_vec_widen_umult_lo_v8hi = 6607,
  CODE_FOR_vec_widen_smult_lo_v8si = 6608,
  CODE_FOR_vec_widen_umult_lo_v8si = 6609,
  CODE_FOR_vec_widen_smult_lo_v4si = 6610,
  CODE_FOR_vec_widen_umult_lo_v4si = 6611,
  CODE_FOR_vec_widen_smult_even_v4si = 6612,
  CODE_FOR_vec_widen_smult_odd_v16si = 6613,
  CODE_FOR_vec_widen_umult_odd_v16si = 6614,
  CODE_FOR_vec_widen_smult_odd_v8si = 6615,
  CODE_FOR_vec_widen_umult_odd_v8si = 6616,
  CODE_FOR_vec_widen_smult_odd_v4si = 6617,
  CODE_FOR_vec_widen_umult_odd_v4si = 6618,
  CODE_FOR_sdot_prodv32hi = 6619,
  CODE_FOR_sdot_prodv16hi = 6620,
  CODE_FOR_sdot_prodv8hi = 6621,
  CODE_FOR_sdot_prodv4si = 6622,
  CODE_FOR_usadv16qi = 6623,
  CODE_FOR_usadv32qi = 6624,
  CODE_FOR_vec_shr_v16qi = 6625,
  CODE_FOR_vec_shr_v8hi = 6626,
  CODE_FOR_vec_shr_v4si = 6627,
  CODE_FOR_vec_shr_v2di = 6628,
  CODE_FOR_smaxv32qi3 = 6629,
  CODE_FOR_sminv32qi3 = 6630,
  CODE_FOR_umaxv32qi3 = 6631,
  CODE_FOR_uminv32qi3 = 6632,
  CODE_FOR_smaxv16hi3 = 6633,
  CODE_FOR_sminv16hi3 = 6634,
  CODE_FOR_umaxv16hi3 = 6635,
  CODE_FOR_uminv16hi3 = 6636,
  CODE_FOR_smaxv8si3 = 6637,
  CODE_FOR_sminv8si3 = 6638,
  CODE_FOR_umaxv8si3 = 6639,
  CODE_FOR_uminv8si3 = 6640,
  CODE_FOR_smaxv64qi3 = 6641,
  CODE_FOR_sminv64qi3 = 6642,
  CODE_FOR_umaxv64qi3 = 6643,
  CODE_FOR_uminv64qi3 = 6644,
  CODE_FOR_smaxv32hi3 = 6645,
  CODE_FOR_sminv32hi3 = 6646,
  CODE_FOR_umaxv32hi3 = 6647,
  CODE_FOR_uminv32hi3 = 6648,
  CODE_FOR_smaxv16si3 = 6649,
  CODE_FOR_sminv16si3 = 6650,
  CODE_FOR_umaxv16si3 = 6651,
  CODE_FOR_uminv16si3 = 6652,
  CODE_FOR_smaxv16si3_mask = 6653,
  CODE_FOR_sminv16si3_mask = 6654,
  CODE_FOR_umaxv16si3_mask = 6655,
  CODE_FOR_uminv16si3_mask = 6656,
  CODE_FOR_smaxv8si3_mask = 6657,
  CODE_FOR_sminv8si3_mask = 6658,
  CODE_FOR_umaxv8si3_mask = 6659,
  CODE_FOR_uminv8si3_mask = 6660,
  CODE_FOR_smaxv4si3_mask = 6661,
  CODE_FOR_sminv4si3_mask = 6662,
  CODE_FOR_umaxv4si3_mask = 6663,
  CODE_FOR_uminv4si3_mask = 6664,
  CODE_FOR_smaxv8di3_mask = 6665,
  CODE_FOR_sminv8di3_mask = 6666,
  CODE_FOR_umaxv8di3_mask = 6667,
  CODE_FOR_uminv8di3_mask = 6668,
  CODE_FOR_smaxv4di3_mask = 6669,
  CODE_FOR_sminv4di3_mask = 6670,
  CODE_FOR_umaxv4di3_mask = 6671,
  CODE_FOR_uminv4di3_mask = 6672,
  CODE_FOR_smaxv2di3_mask = 6673,
  CODE_FOR_sminv2di3_mask = 6674,
  CODE_FOR_umaxv2di3_mask = 6675,
  CODE_FOR_uminv2di3_mask = 6676,
  CODE_FOR_smaxv8di3 = 6677,
  CODE_FOR_sminv8di3 = 6678,
  CODE_FOR_umaxv8di3 = 6679,
  CODE_FOR_uminv8di3 = 6680,
  CODE_FOR_smaxv4di3 = 6681,
  CODE_FOR_sminv4di3 = 6682,
  CODE_FOR_umaxv4di3 = 6683,
  CODE_FOR_uminv4di3 = 6684,
  CODE_FOR_smaxv2di3 = 6685,
  CODE_FOR_sminv2di3 = 6686,
  CODE_FOR_umaxv2di3 = 6687,
  CODE_FOR_uminv2di3 = 6688,
  CODE_FOR_smaxv16qi3 = 6689,
  CODE_FOR_sminv16qi3 = 6690,
  CODE_FOR_smaxv8hi3 = 6691,
  CODE_FOR_sminv8hi3 = 6692,
  CODE_FOR_smaxv4si3 = 6693,
  CODE_FOR_sminv4si3 = 6694,
  CODE_FOR_umaxv16qi3 = 6695,
  CODE_FOR_uminv16qi3 = 6696,
  CODE_FOR_umaxv8hi3 = 6697,
  CODE_FOR_uminv8hi3 = 6698,
  CODE_FOR_umaxv4si3 = 6699,
  CODE_FOR_uminv4si3 = 6700,
  CODE_FOR_avx2_eqv32qi3 = 6701,
  CODE_FOR_avx2_eqv16hi3 = 6702,
  CODE_FOR_avx2_eqv8si3 = 6703,
  CODE_FOR_avx2_eqv4di3 = 6704,
  CODE_FOR_avx512bw_eqv64qi3 = 6705,
  CODE_FOR_avx512bw_eqv64qi3_mask = 6706,
  CODE_FOR_avx512vl_eqv16qi3 = 6707,
  CODE_FOR_avx512vl_eqv16qi3_mask = 6708,
  CODE_FOR_avx512vl_eqv32qi3 = 6709,
  CODE_FOR_avx512vl_eqv32qi3_mask = 6710,
  CODE_FOR_avx512bw_eqv32hi3 = 6711,
  CODE_FOR_avx512bw_eqv32hi3_mask = 6712,
  CODE_FOR_avx512vl_eqv16hi3 = 6713,
  CODE_FOR_avx512vl_eqv16hi3_mask = 6714,
  CODE_FOR_avx512vl_eqv8hi3 = 6715,
  CODE_FOR_avx512vl_eqv8hi3_mask = 6716,
  CODE_FOR_avx512f_eqv16si3 = 6717,
  CODE_FOR_avx512f_eqv16si3_mask = 6718,
  CODE_FOR_avx512vl_eqv8si3 = 6719,
  CODE_FOR_avx512vl_eqv8si3_mask = 6720,
  CODE_FOR_avx512vl_eqv4si3 = 6721,
  CODE_FOR_avx512vl_eqv4si3_mask = 6722,
  CODE_FOR_avx512f_eqv8di3 = 6723,
  CODE_FOR_avx512f_eqv8di3_mask = 6724,
  CODE_FOR_avx512vl_eqv4di3 = 6725,
  CODE_FOR_avx512vl_eqv4di3_mask = 6726,
  CODE_FOR_avx512vl_eqv2di3 = 6727,
  CODE_FOR_avx512vl_eqv2di3_mask = 6728,
  CODE_FOR_sse2_eqv16qi3 = 6729,
  CODE_FOR_sse2_eqv8hi3 = 6730,
  CODE_FOR_sse2_eqv4si3 = 6731,
  CODE_FOR_sse4_1_eqv2di3 = 6732,
  CODE_FOR_vcondv64qiv16si = 6733,
  CODE_FOR_vcondv32hiv16si = 6734,
  CODE_FOR_vcondv16siv16si = 6735,
  CODE_FOR_vcondv8div16si = 6736,
  CODE_FOR_vcondv16sfv16si = 6737,
  CODE_FOR_vcondv8dfv16si = 6738,
  CODE_FOR_vcondv64qiv8di = 6739,
  CODE_FOR_vcondv32hiv8di = 6740,
  CODE_FOR_vcondv16siv8di = 6741,
  CODE_FOR_vcondv8div8di = 6742,
  CODE_FOR_vcondv16sfv8di = 6743,
  CODE_FOR_vcondv8dfv8di = 6744,
  CODE_FOR_vcondv64qiv32hi = 6745,
  CODE_FOR_vcondv32hiv32hi = 6746,
  CODE_FOR_vcondv16siv32hi = 6747,
  CODE_FOR_vcondv8div32hi = 6748,
  CODE_FOR_vcondv16sfv32hi = 6749,
  CODE_FOR_vcondv8dfv32hi = 6750,
  CODE_FOR_vcondv64qiv64qi = 6751,
  CODE_FOR_vcondv32hiv64qi = 6752,
  CODE_FOR_vcondv16siv64qi = 6753,
  CODE_FOR_vcondv8div64qi = 6754,
  CODE_FOR_vcondv16sfv64qi = 6755,
  CODE_FOR_vcondv8dfv64qi = 6756,
  CODE_FOR_vcondv32qiv32qi = 6757,
  CODE_FOR_vcondv16hiv32qi = 6758,
  CODE_FOR_vcondv8siv32qi = 6759,
  CODE_FOR_vcondv4div32qi = 6760,
  CODE_FOR_vcondv8sfv32qi = 6761,
  CODE_FOR_vcondv4dfv32qi = 6762,
  CODE_FOR_vcondv32qiv16hi = 6763,
  CODE_FOR_vcondv16hiv16hi = 6764,
  CODE_FOR_vcondv8siv16hi = 6765,
  CODE_FOR_vcondv4div16hi = 6766,
  CODE_FOR_vcondv8sfv16hi = 6767,
  CODE_FOR_vcondv4dfv16hi = 6768,
  CODE_FOR_vcondv32qiv8si = 6769,
  CODE_FOR_vcondv16hiv8si = 6770,
  CODE_FOR_vcondv8siv8si = 6771,
  CODE_FOR_vcondv4div8si = 6772,
  CODE_FOR_vcondv8sfv8si = 6773,
  CODE_FOR_vcondv4dfv8si = 6774,
  CODE_FOR_vcondv32qiv4di = 6775,
  CODE_FOR_vcondv16hiv4di = 6776,
  CODE_FOR_vcondv8siv4di = 6777,
  CODE_FOR_vcondv4div4di = 6778,
  CODE_FOR_vcondv8sfv4di = 6779,
  CODE_FOR_vcondv4dfv4di = 6780,
  CODE_FOR_vcondv16qiv16qi = 6781,
  CODE_FOR_vcondv8hiv16qi = 6782,
  CODE_FOR_vcondv4siv16qi = 6783,
  CODE_FOR_vcondv2div16qi = 6784,
  CODE_FOR_vcondv4sfv16qi = 6785,
  CODE_FOR_vcondv2dfv16qi = 6786,
  CODE_FOR_vcondv16qiv8hi = 6787,
  CODE_FOR_vcondv8hiv8hi = 6788,
  CODE_FOR_vcondv4siv8hi = 6789,
  CODE_FOR_vcondv2div8hi = 6790,
  CODE_FOR_vcondv4sfv8hi = 6791,
  CODE_FOR_vcondv2dfv8hi = 6792,
  CODE_FOR_vcondv16qiv4si = 6793,
  CODE_FOR_vcondv8hiv4si = 6794,
  CODE_FOR_vcondv4siv4si = 6795,
  CODE_FOR_vcondv2div4si = 6796,
  CODE_FOR_vcondv4sfv4si = 6797,
  CODE_FOR_vcondv2dfv4si = 6798,
  CODE_FOR_vcondv2div2di = 6799,
  CODE_FOR_vcondv2dfv2di = 6800,
  CODE_FOR_vconduv64qiv16si = 6801,
  CODE_FOR_vconduv32hiv16si = 6802,
  CODE_FOR_vconduv16siv16si = 6803,
  CODE_FOR_vconduv8div16si = 6804,
  CODE_FOR_vconduv16sfv16si = 6805,
  CODE_FOR_vconduv8dfv16si = 6806,
  CODE_FOR_vconduv64qiv8di = 6807,
  CODE_FOR_vconduv32hiv8di = 6808,
  CODE_FOR_vconduv16siv8di = 6809,
  CODE_FOR_vconduv8div8di = 6810,
  CODE_FOR_vconduv16sfv8di = 6811,
  CODE_FOR_vconduv8dfv8di = 6812,
  CODE_FOR_vconduv64qiv32hi = 6813,
  CODE_FOR_vconduv32hiv32hi = 6814,
  CODE_FOR_vconduv16siv32hi = 6815,
  CODE_FOR_vconduv8div32hi = 6816,
  CODE_FOR_vconduv16sfv32hi = 6817,
  CODE_FOR_vconduv8dfv32hi = 6818,
  CODE_FOR_vconduv64qiv64qi = 6819,
  CODE_FOR_vconduv32hiv64qi = 6820,
  CODE_FOR_vconduv16siv64qi = 6821,
  CODE_FOR_vconduv8div64qi = 6822,
  CODE_FOR_vconduv16sfv64qi = 6823,
  CODE_FOR_vconduv8dfv64qi = 6824,
  CODE_FOR_vconduv32qiv32qi = 6825,
  CODE_FOR_vconduv16hiv32qi = 6826,
  CODE_FOR_vconduv8siv32qi = 6827,
  CODE_FOR_vconduv4div32qi = 6828,
  CODE_FOR_vconduv8sfv32qi = 6829,
  CODE_FOR_vconduv4dfv32qi = 6830,
  CODE_FOR_vconduv32qiv16hi = 6831,
  CODE_FOR_vconduv16hiv16hi = 6832,
  CODE_FOR_vconduv8siv16hi = 6833,
  CODE_FOR_vconduv4div16hi = 6834,
  CODE_FOR_vconduv8sfv16hi = 6835,
  CODE_FOR_vconduv4dfv16hi = 6836,
  CODE_FOR_vconduv32qiv8si = 6837,
  CODE_FOR_vconduv16hiv8si = 6838,
  CODE_FOR_vconduv8siv8si = 6839,
  CODE_FOR_vconduv4div8si = 6840,
  CODE_FOR_vconduv8sfv8si = 6841,
  CODE_FOR_vconduv4dfv8si = 6842,
  CODE_FOR_vconduv32qiv4di = 6843,
  CODE_FOR_vconduv16hiv4di = 6844,
  CODE_FOR_vconduv8siv4di = 6845,
  CODE_FOR_vconduv4div4di = 6846,
  CODE_FOR_vconduv8sfv4di = 6847,
  CODE_FOR_vconduv4dfv4di = 6848,
  CODE_FOR_vconduv16qiv16qi = 6849,
  CODE_FOR_vconduv8hiv16qi = 6850,
  CODE_FOR_vconduv4siv16qi = 6851,
  CODE_FOR_vconduv2div16qi = 6852,
  CODE_FOR_vconduv4sfv16qi = 6853,
  CODE_FOR_vconduv2dfv16qi = 6854,
  CODE_FOR_vconduv16qiv8hi = 6855,
  CODE_FOR_vconduv8hiv8hi = 6856,
  CODE_FOR_vconduv4siv8hi = 6857,
  CODE_FOR_vconduv2div8hi = 6858,
  CODE_FOR_vconduv4sfv8hi = 6859,
  CODE_FOR_vconduv2dfv8hi = 6860,
  CODE_FOR_vconduv16qiv4si = 6861,
  CODE_FOR_vconduv8hiv4si = 6862,
  CODE_FOR_vconduv4siv4si = 6863,
  CODE_FOR_vconduv2div4si = 6864,
  CODE_FOR_vconduv4sfv4si = 6865,
  CODE_FOR_vconduv2dfv4si = 6866,
  CODE_FOR_vconduv2div2di = 6867,
  CODE_FOR_vconduv2dfv2di = 6868,
  CODE_FOR_vcondeqv2div2di = 6869,
  CODE_FOR_vcondeqv2dfv2di = 6870,
  CODE_FOR_vec_permv16qi = 6871,
  CODE_FOR_vec_permv8hi = 6872,
  CODE_FOR_vec_permv4si = 6873,
  CODE_FOR_vec_permv2di = 6874,
  CODE_FOR_vec_permv4sf = 6875,
  CODE_FOR_vec_permv2df = 6876,
  CODE_FOR_vec_permv32qi = 6877,
  CODE_FOR_vec_permv16hi = 6878,
  CODE_FOR_vec_permv8si = 6879,
  CODE_FOR_vec_permv4di = 6880,
  CODE_FOR_vec_permv8sf = 6881,
  CODE_FOR_vec_permv4df = 6882,
  CODE_FOR_vec_permv16sf = 6883,
  CODE_FOR_vec_permv8df = 6884,
  CODE_FOR_vec_permv16si = 6885,
  CODE_FOR_vec_permv8di = 6886,
  CODE_FOR_vec_permv32hi = 6887,
  CODE_FOR_vec_permv64qi = 6888,
  CODE_FOR_one_cmplv16si2 = 6889,
  CODE_FOR_one_cmplv8di2 = 6890,
  CODE_FOR_one_cmplv64qi2 = 6891,
  CODE_FOR_one_cmplv32qi2 = 6892,
  CODE_FOR_one_cmplv16qi2 = 6893,
  CODE_FOR_one_cmplv32hi2 = 6894,
  CODE_FOR_one_cmplv16hi2 = 6895,
  CODE_FOR_one_cmplv8hi2 = 6896,
  CODE_FOR_one_cmplv8si2 = 6897,
  CODE_FOR_one_cmplv4si2 = 6898,
  CODE_FOR_one_cmplv4di2 = 6899,
  CODE_FOR_one_cmplv2di2 = 6900,
  CODE_FOR_avx512bw_andnotv64qi3 = 6901,
  CODE_FOR_avx2_andnotv32qi3 = 6902,
  CODE_FOR_sse2_andnotv16qi3 = 6903,
  CODE_FOR_avx512bw_andnotv32hi3 = 6904,
  CODE_FOR_avx2_andnotv16hi3 = 6905,
  CODE_FOR_sse2_andnotv8hi3 = 6906,
  CODE_FOR_avx512f_andnotv16si3 = 6907,
  CODE_FOR_avx2_andnotv8si3 = 6908,
  CODE_FOR_sse2_andnotv4si3 = 6909,
  CODE_FOR_avx512f_andnotv8di3 = 6910,
  CODE_FOR_avx2_andnotv4di3 = 6911,
  CODE_FOR_sse2_andnotv2di3 = 6912,
  CODE_FOR_avx512f_andnotv16si3_mask = 6913,
  CODE_FOR_avx2_andnotv8si3_mask = 6914,
  CODE_FOR_sse2_andnotv4si3_mask = 6915,
  CODE_FOR_avx512f_andnotv8di3_mask = 6916,
  CODE_FOR_avx2_andnotv4di3_mask = 6917,
  CODE_FOR_sse2_andnotv2di3_mask = 6918,
  CODE_FOR_avx512bw_andnotv64qi3_mask = 6919,
  CODE_FOR_sse2_andnotv16qi3_mask = 6920,
  CODE_FOR_avx2_andnotv32qi3_mask = 6921,
  CODE_FOR_avx512bw_andnotv32hi3_mask = 6922,
  CODE_FOR_avx2_andnotv16hi3_mask = 6923,
  CODE_FOR_sse2_andnotv8hi3_mask = 6924,
  CODE_FOR_andv16si3 = 6925,
  CODE_FOR_iorv16si3 = 6926,
  CODE_FOR_xorv16si3 = 6927,
  CODE_FOR_andv8di3 = 6928,
  CODE_FOR_iorv8di3 = 6929,
  CODE_FOR_xorv8di3 = 6930,
  CODE_FOR_andv64qi3 = 6931,
  CODE_FOR_iorv64qi3 = 6932,
  CODE_FOR_xorv64qi3 = 6933,
  CODE_FOR_andv32qi3 = 6934,
  CODE_FOR_iorv32qi3 = 6935,
  CODE_FOR_xorv32qi3 = 6936,
  CODE_FOR_andv16qi3 = 6937,
  CODE_FOR_iorv16qi3 = 6938,
  CODE_FOR_xorv16qi3 = 6939,
  CODE_FOR_andv32hi3 = 6940,
  CODE_FOR_iorv32hi3 = 6941,
  CODE_FOR_xorv32hi3 = 6942,
  CODE_FOR_andv16hi3 = 6943,
  CODE_FOR_iorv16hi3 = 6944,
  CODE_FOR_xorv16hi3 = 6945,
  CODE_FOR_andv8hi3 = 6946,
  CODE_FOR_iorv8hi3 = 6947,
  CODE_FOR_xorv8hi3 = 6948,
  CODE_FOR_andv8si3 = 6949,
  CODE_FOR_iorv8si3 = 6950,
  CODE_FOR_xorv8si3 = 6951,
  CODE_FOR_andv4si3 = 6952,
  CODE_FOR_iorv4si3 = 6953,
  CODE_FOR_xorv4si3 = 6954,
  CODE_FOR_andv4di3 = 6955,
  CODE_FOR_iorv4di3 = 6956,
  CODE_FOR_xorv4di3 = 6957,
  CODE_FOR_andv2di3 = 6958,
  CODE_FOR_iorv2di3 = 6959,
  CODE_FOR_xorv2di3 = 6960,
  CODE_FOR_vec_pack_trunc_v32hi = 6961,
  CODE_FOR_vec_pack_trunc_v16hi = 6962,
  CODE_FOR_vec_pack_trunc_v8hi = 6963,
  CODE_FOR_vec_pack_trunc_v16si = 6964,
  CODE_FOR_vec_pack_trunc_v8si = 6965,
  CODE_FOR_vec_pack_trunc_v4si = 6966,
  CODE_FOR_vec_pack_trunc_v8di = 6967,
  CODE_FOR_vec_pack_trunc_v4di = 6968,
  CODE_FOR_vec_pack_trunc_v2di = 6969,
  CODE_FOR_vec_pack_trunc_qi = 6970,
  CODE_FOR_vec_pack_trunc_hi = 6971,
  CODE_FOR_vec_pack_trunc_si = 6972,
  CODE_FOR_vec_interleave_highv32qi = 6973,
  CODE_FOR_vec_interleave_highv16hi = 6974,
  CODE_FOR_vec_interleave_highv8si = 6975,
  CODE_FOR_vec_interleave_highv4di = 6976,
  CODE_FOR_vec_interleave_lowv32qi = 6977,
  CODE_FOR_vec_interleave_lowv16hi = 6978,
  CODE_FOR_vec_interleave_lowv8si = 6979,
  CODE_FOR_vec_interleave_lowv4di = 6980,
  CODE_FOR_avx512dq_vinsertf64x2_mask = 6981,
  CODE_FOR_avx512dq_vinserti64x2_mask = 6982,
  CODE_FOR_avx512f_vinsertf32x4_mask = 6983,
  CODE_FOR_avx512f_vinserti32x4_mask = 6984,
  CODE_FOR_avx512dq_vinsertf32x8_mask = 6985,
  CODE_FOR_avx512dq_vinserti32x8_mask = 6986,
  CODE_FOR_avx512f_vinsertf64x4_mask = 6987,
  CODE_FOR_avx512f_vinserti64x4_mask = 6988,
  CODE_FOR_avx512dq_shuf_i64x2_mask = 6989,
  CODE_FOR_avx512dq_shuf_f64x2_mask = 6990,
  CODE_FOR_avx512f_shuf_f64x2_mask = 6991,
  CODE_FOR_avx512f_shuf_i64x2_mask = 6992,
  CODE_FOR_avx512vl_shuf_i32x4_mask = 6993,
  CODE_FOR_avx512vl_shuf_f32x4_mask = 6994,
  CODE_FOR_avx512f_shuf_f32x4_mask = 6995,
  CODE_FOR_avx512f_shuf_i32x4_mask = 6996,
  CODE_FOR_avx512f_pshufdv3_mask = 6997,
  CODE_FOR_avx512vl_pshufdv3_mask = 6998,
  CODE_FOR_avx2_pshufdv3 = 6999,
  CODE_FOR_avx512vl_pshufd_mask = 7000,
  CODE_FOR_sse2_pshufd = 7001,
  CODE_FOR_avx512vl_pshuflwv3_mask = 7002,
  CODE_FOR_avx2_pshuflwv3 = 7003,
  CODE_FOR_avx512vl_pshuflw_mask = 7004,
  CODE_FOR_sse2_pshuflw = 7005,
  CODE_FOR_avx2_pshufhwv3 = 7006,
  CODE_FOR_avx512vl_pshufhwv3_mask = 7007,
  CODE_FOR_avx512vl_pshufhw_mask = 7008,
  CODE_FOR_sse2_pshufhw = 7009,
  CODE_FOR_sse2_loadd = 7010,
  CODE_FOR_vec_unpacks_lo_v64qi = 7011,
  CODE_FOR_vec_unpacks_lo_v32qi = 7012,
  CODE_FOR_vec_unpacks_lo_v16qi = 7013,
  CODE_FOR_vec_unpacks_lo_v32hi = 7014,
  CODE_FOR_vec_unpacks_lo_v16hi = 7015,
  CODE_FOR_vec_unpacks_lo_v8hi = 7016,
  CODE_FOR_vec_unpacks_lo_v16si = 7017,
  CODE_FOR_vec_unpacks_lo_v8si = 7018,
  CODE_FOR_vec_unpacks_lo_v4si = 7019,
  CODE_FOR_vec_unpacks_hi_v64qi = 7020,
  CODE_FOR_vec_unpacks_hi_v32qi = 7021,
  CODE_FOR_vec_unpacks_hi_v16qi = 7022,
  CODE_FOR_vec_unpacks_hi_v32hi = 7023,
  CODE_FOR_vec_unpacks_hi_v16hi = 7024,
  CODE_FOR_vec_unpacks_hi_v8hi = 7025,
  CODE_FOR_vec_unpacks_hi_v16si = 7026,
  CODE_FOR_vec_unpacks_hi_v8si = 7027,
  CODE_FOR_vec_unpacks_hi_v4si = 7028,
  CODE_FOR_vec_unpacku_lo_v64qi = 7029,
  CODE_FOR_vec_unpacku_lo_v32qi = 7030,
  CODE_FOR_vec_unpacku_lo_v16qi = 7031,
  CODE_FOR_vec_unpacku_lo_v32hi = 7032,
  CODE_FOR_vec_unpacku_lo_v16hi = 7033,
  CODE_FOR_vec_unpacku_lo_v8hi = 7034,
  CODE_FOR_vec_unpacku_lo_v16si = 7035,
  CODE_FOR_vec_unpacku_lo_v8si = 7036,
  CODE_FOR_vec_unpacku_lo_v4si = 7037,
  CODE_FOR_vec_unpacks_lo_hi = 7038,
  CODE_FOR_vec_unpacks_lo_si = 7039,
  CODE_FOR_vec_unpacks_lo_di = 7040,
  CODE_FOR_vec_unpacku_hi_v64qi = 7041,
  CODE_FOR_vec_unpacku_hi_v32qi = 7042,
  CODE_FOR_vec_unpacku_hi_v16qi = 7043,
  CODE_FOR_vec_unpacku_hi_v32hi = 7044,
  CODE_FOR_vec_unpacku_hi_v16hi = 7045,
  CODE_FOR_vec_unpacku_hi_v8hi = 7046,
  CODE_FOR_vec_unpacku_hi_v16si = 7047,
  CODE_FOR_vec_unpacku_hi_v8si = 7048,
  CODE_FOR_vec_unpacku_hi_v4si = 7049,
  CODE_FOR_vec_unpacks_hi_hi = 7050,
  CODE_FOR_vec_unpacks_hi_si = 7051,
  CODE_FOR_vec_unpacks_hi_di = 7052,
  CODE_FOR_avx512bw_uavgv64qi3 = 7053,
  CODE_FOR_avx512bw_uavgv64qi3_mask = 7054,
  CODE_FOR_avx2_uavgv32qi3 = 7055,
  CODE_FOR_avx2_uavgv32qi3_mask = 7056,
  CODE_FOR_sse2_uavgv16qi3 = 7057,
  CODE_FOR_sse2_uavgv16qi3_mask = 7058,
  CODE_FOR_avx512bw_uavgv32hi3 = 7059,
  CODE_FOR_avx512bw_uavgv32hi3_mask = 7060,
  CODE_FOR_avx2_uavgv16hi3 = 7061,
  CODE_FOR_avx2_uavgv16hi3_mask = 7062,
  CODE_FOR_sse2_uavgv8hi3 = 7063,
  CODE_FOR_sse2_uavgv8hi3_mask = 7064,
  CODE_FOR_sse2_maskmovdqu = 7065,
  CODE_FOR_ssse3_pmulhrswv4hi3_mask = 7066,
  CODE_FOR_ssse3_pmulhrswv8hi3_mask = 7067,
  CODE_FOR_avx2_pmulhrswv16hi3_mask = 7068,
  CODE_FOR_ssse3_pmulhrswv4hi3 = 7069,
  CODE_FOR_ssse3_pmulhrswv8hi3 = 7070,
  CODE_FOR_avx2_pmulhrswv16hi3 = 7071,
  CODE_FOR_absv64qi2 = 7072,
  CODE_FOR_absv32qi2 = 7073,
  CODE_FOR_absv16qi2 = 7074,
  CODE_FOR_absv32hi2 = 7075,
  CODE_FOR_absv16hi2 = 7076,
  CODE_FOR_absv8hi2 = 7077,
  CODE_FOR_absv16si2 = 7078,
  CODE_FOR_absv8si2 = 7079,
  CODE_FOR_absv4si2 = 7080,
  CODE_FOR_absv8di2 = 7081,
  CODE_FOR_absv4di2 = 7082,
  CODE_FOR_absv2di2 = 7083,
  CODE_FOR_avx2_pblendw = 7084,
  CODE_FOR_avx_roundps_sfix256 = 7085,
  CODE_FOR_sse4_1_roundps_sfix = 7086,
  CODE_FOR_avx512f_roundps512 = 7087,
  CODE_FOR_avx512f_roundpd512 = 7088,
  CODE_FOR_avx512f_roundps512_sfix = 7089,
  CODE_FOR_avx512f_roundpd_vec_pack_sfix512 = 7090,
  CODE_FOR_avx_roundpd_vec_pack_sfix256 = 7091,
  CODE_FOR_sse4_1_roundpd_vec_pack_sfix = 7092,
  CODE_FOR_roundv16sf2 = 7093,
  CODE_FOR_roundv8sf2 = 7094,
  CODE_FOR_roundv4sf2 = 7095,
  CODE_FOR_roundv8df2 = 7096,
  CODE_FOR_roundv4df2 = 7097,
  CODE_FOR_roundv2df2 = 7098,
  CODE_FOR_roundv16sf2_sfix = 7099,
  CODE_FOR_roundv8sf2_sfix = 7100,
  CODE_FOR_roundv4sf2_sfix = 7101,
  CODE_FOR_roundv8df2_vec_pack_sfix = 7102,
  CODE_FOR_roundv4df2_vec_pack_sfix = 7103,
  CODE_FOR_roundv2df2_vec_pack_sfix = 7104,
  CODE_FOR_avx512pf_gatherpfv16sisf = 7105,
  CODE_FOR_avx512pf_gatherpfv8disf = 7106,
  CODE_FOR_avx512pf_gatherpfv8sidf = 7107,
  CODE_FOR_avx512pf_gatherpfv8didf = 7108,
  CODE_FOR_avx512pf_scatterpfv16sisf = 7109,
  CODE_FOR_avx512pf_scatterpfv8disf = 7110,
  CODE_FOR_avx512pf_scatterpfv8sidf = 7111,
  CODE_FOR_avx512pf_scatterpfv8didf = 7112,
  CODE_FOR_rotlv16qi3 = 7113,
  CODE_FOR_rotlv8hi3 = 7114,
  CODE_FOR_rotlv4si3 = 7115,
  CODE_FOR_rotlv2di3 = 7116,
  CODE_FOR_rotrv16qi3 = 7117,
  CODE_FOR_rotrv8hi3 = 7118,
  CODE_FOR_rotrv4si3 = 7119,
  CODE_FOR_rotrv2di3 = 7120,
  CODE_FOR_vrotrv16qi3 = 7121,
  CODE_FOR_vrotrv8hi3 = 7122,
  CODE_FOR_vrotrv4si3 = 7123,
  CODE_FOR_vrotrv2di3 = 7124,
  CODE_FOR_vrotlv16qi3 = 7125,
  CODE_FOR_vrotlv8hi3 = 7126,
  CODE_FOR_vrotlv4si3 = 7127,
  CODE_FOR_vrotlv2di3 = 7128,
  CODE_FOR_vlshrv16qi3 = 7129,
  CODE_FOR_vlshrv8hi3 = 7130,
  CODE_FOR_vlshrv4si3 = 7131,
  CODE_FOR_vlshrv2di3 = 7132,
  CODE_FOR_vlshrv16si3 = 7133,
  CODE_FOR_vlshrv8di3 = 7134,
  CODE_FOR_vlshrv8si3 = 7135,
  CODE_FOR_vlshrv4di3 = 7136,
  CODE_FOR_vashrv8hi3 = 7137,
  CODE_FOR_vashrv8hi3_mask = 7138,
  CODE_FOR_vashrv16qi3 = 7139,
  CODE_FOR_vashrv2di3 = 7140,
  CODE_FOR_vashrv2di3_mask = 7141,
  CODE_FOR_vashrv4si3 = 7142,
  CODE_FOR_vashrv16si3 = 7143,
  CODE_FOR_vashrv8si3 = 7144,
  CODE_FOR_vashlv16qi3 = 7145,
  CODE_FOR_vashlv8hi3 = 7146,
  CODE_FOR_vashlv4si3 = 7147,
  CODE_FOR_vashlv2di3 = 7148,
  CODE_FOR_vashlv16si3 = 7149,
  CODE_FOR_vashlv8di3 = 7150,
  CODE_FOR_vashlv8si3 = 7151,
  CODE_FOR_vashlv4di3 = 7152,
  CODE_FOR_ashlv64qi3 = 7153,
  CODE_FOR_lshrv64qi3 = 7154,
  CODE_FOR_ashrv64qi3 = 7155,
  CODE_FOR_ashlv32qi3 = 7156,
  CODE_FOR_lshrv32qi3 = 7157,
  CODE_FOR_ashrv32qi3 = 7158,
  CODE_FOR_ashlv16qi3 = 7159,
  CODE_FOR_lshrv16qi3 = 7160,
  CODE_FOR_ashrv16qi3 = 7161,
  CODE_FOR_ashrv2di3 = 7162,
  CODE_FOR_xop_vmfrczv4sf2 = 7163,
  CODE_FOR_xop_vmfrczv2df2 = 7164,
  CODE_FOR_avx_vzeroall = 7165,
  CODE_FOR_avx2_permv4di = 7166,
  CODE_FOR_avx2_permv4df = 7167,
  CODE_FOR_avx512vl_permv4di_mask = 7168,
  CODE_FOR_avx512vl_permv4df_mask = 7169,
  CODE_FOR_avx512f_permv8df = 7170,
  CODE_FOR_avx512f_permv8di = 7171,
  CODE_FOR_avx512f_permv8df_mask = 7172,
  CODE_FOR_avx512f_permv8di_mask = 7173,
  CODE_FOR_avx512f_vpermilv8df = 7174,
  CODE_FOR_avx512f_vpermilv8df_mask = 7175,
  CODE_FOR_avx_vpermilv4df = 7176,
  CODE_FOR_avx_vpermilv4df_mask = 7177,
  CODE_FOR_avx_vpermilv2df = 7178,
  CODE_FOR_avx_vpermilv2df_mask = 7179,
  CODE_FOR_avx512f_vpermilv16sf = 7180,
  CODE_FOR_avx512f_vpermilv16sf_mask = 7181,
  CODE_FOR_avx_vpermilv8sf = 7182,
  CODE_FOR_avx_vpermilv8sf_mask = 7183,
  CODE_FOR_avx_vpermilv4sf = 7184,
  CODE_FOR_avx_vpermilv4sf_mask = 7185,
  CODE_FOR_avx512f_vpermi2varv16si3_mask = 7186,
  CODE_FOR_avx512f_vpermi2varv16sf3_mask = 7187,
  CODE_FOR_avx512f_vpermi2varv8di3_mask = 7188,
  CODE_FOR_avx512f_vpermi2varv8df3_mask = 7189,
  CODE_FOR_avx512vl_vpermi2varv8si3_mask = 7190,
  CODE_FOR_avx512vl_vpermi2varv8sf3_mask = 7191,
  CODE_FOR_avx512vl_vpermi2varv4di3_mask = 7192,
  CODE_FOR_avx512vl_vpermi2varv4df3_mask = 7193,
  CODE_FOR_avx512vl_vpermi2varv4si3_mask = 7194,
  CODE_FOR_avx512vl_vpermi2varv4sf3_mask = 7195,
  CODE_FOR_avx512vl_vpermi2varv2di3_mask = 7196,
  CODE_FOR_avx512vl_vpermi2varv2df3_mask = 7197,
  CODE_FOR_avx512bw_vpermi2varv32hi3_mask = 7198,
  CODE_FOR_avx512vl_vpermi2varv16hi3_mask = 7199,
  CODE_FOR_avx512vl_vpermi2varv8hi3_mask = 7200,
  CODE_FOR_avx512bw_vpermi2varv64qi3_mask = 7201,
  CODE_FOR_avx512vl_vpermi2varv32qi3_mask = 7202,
  CODE_FOR_avx512vl_vpermi2varv16qi3_mask = 7203,
  CODE_FOR_avx512f_vpermt2varv16si3_maskz = 7204,
  CODE_FOR_avx512f_vpermt2varv16sf3_maskz = 7205,
  CODE_FOR_avx512f_vpermt2varv8di3_maskz = 7206,
  CODE_FOR_avx512f_vpermt2varv8df3_maskz = 7207,
  CODE_FOR_avx512vl_vpermt2varv8si3_maskz = 7208,
  CODE_FOR_avx512vl_vpermt2varv8sf3_maskz = 7209,
  CODE_FOR_avx512vl_vpermt2varv4di3_maskz = 7210,
  CODE_FOR_avx512vl_vpermt2varv4df3_maskz = 7211,
  CODE_FOR_avx512vl_vpermt2varv4si3_maskz = 7212,
  CODE_FOR_avx512vl_vpermt2varv4sf3_maskz = 7213,
  CODE_FOR_avx512vl_vpermt2varv2di3_maskz = 7214,
  CODE_FOR_avx512vl_vpermt2varv2df3_maskz = 7215,
  CODE_FOR_avx512bw_vpermt2varv32hi3_maskz = 7216,
  CODE_FOR_avx512vl_vpermt2varv16hi3_maskz = 7217,
  CODE_FOR_avx512vl_vpermt2varv8hi3_maskz = 7218,
  CODE_FOR_avx512bw_vpermt2varv64qi3_maskz = 7219,
  CODE_FOR_avx512vl_vpermt2varv32qi3_maskz = 7220,
  CODE_FOR_avx512vl_vpermt2varv16qi3_maskz = 7221,
  CODE_FOR_avx_vperm2f128v8si3 = 7222,
  CODE_FOR_avx_vperm2f128v8sf3 = 7223,
  CODE_FOR_avx_vperm2f128v4df3 = 7224,
  CODE_FOR_avx512vl_vinsertv8si = 7225,
  CODE_FOR_avx512vl_vinsertv8sf = 7226,
  CODE_FOR_avx512vl_vinsertv4di = 7227,
  CODE_FOR_avx512vl_vinsertv4df = 7228,
  CODE_FOR_avx_vinsertf128v32qi = 7229,
  CODE_FOR_avx_vinsertf128v16hi = 7230,
  CODE_FOR_avx_vinsertf128v8si = 7231,
  CODE_FOR_avx_vinsertf128v4di = 7232,
  CODE_FOR_avx_vinsertf128v8sf = 7233,
  CODE_FOR_avx_vinsertf128v4df = 7234,
  CODE_FOR_maskloadv4sfv4si = 7235,
  CODE_FOR_maskloadv2dfv2di = 7236,
  CODE_FOR_maskloadv8sfv8si = 7237,
  CODE_FOR_maskloadv4dfv4di = 7238,
  CODE_FOR_maskloadv4siv4si = 7239,
  CODE_FOR_maskloadv2div2di = 7240,
  CODE_FOR_maskloadv8siv8si = 7241,
  CODE_FOR_maskloadv4div4di = 7242,
  CODE_FOR_maskloadv16sihi = 7243,
  CODE_FOR_maskloadv8siqi = 7244,
  CODE_FOR_maskloadv4siqi = 7245,
  CODE_FOR_maskloadv8diqi = 7246,
  CODE_FOR_maskloadv4diqi = 7247,
  CODE_FOR_maskloadv2diqi = 7248,
  CODE_FOR_maskloadv16sfhi = 7249,
  CODE_FOR_maskloadv8sfqi = 7250,
  CODE_FOR_maskloadv4sfqi = 7251,
  CODE_FOR_maskloadv8dfqi = 7252,
  CODE_FOR_maskloadv4dfqi = 7253,
  CODE_FOR_maskloadv2dfqi = 7254,
  CODE_FOR_maskloadv64qidi = 7255,
  CODE_FOR_maskloadv16qihi = 7256,
  CODE_FOR_maskloadv32qisi = 7257,
  CODE_FOR_maskloadv32hisi = 7258,
  CODE_FOR_maskloadv16hihi = 7259,
  CODE_FOR_maskloadv8hiqi = 7260,
  CODE_FOR_maskstorev4sfv4si = 7261,
  CODE_FOR_maskstorev2dfv2di = 7262,
  CODE_FOR_maskstorev8sfv8si = 7263,
  CODE_FOR_maskstorev4dfv4di = 7264,
  CODE_FOR_maskstorev4siv4si = 7265,
  CODE_FOR_maskstorev2div2di = 7266,
  CODE_FOR_maskstorev8siv8si = 7267,
  CODE_FOR_maskstorev4div4di = 7268,
  CODE_FOR_maskstorev16sihi = 7269,
  CODE_FOR_maskstorev8siqi = 7270,
  CODE_FOR_maskstorev4siqi = 7271,
  CODE_FOR_maskstorev8diqi = 7272,
  CODE_FOR_maskstorev4diqi = 7273,
  CODE_FOR_maskstorev2diqi = 7274,
  CODE_FOR_maskstorev16sfhi = 7275,
  CODE_FOR_maskstorev8sfqi = 7276,
  CODE_FOR_maskstorev4sfqi = 7277,
  CODE_FOR_maskstorev8dfqi = 7278,
  CODE_FOR_maskstorev4dfqi = 7279,
  CODE_FOR_maskstorev2dfqi = 7280,
  CODE_FOR_maskstorev64qidi = 7281,
  CODE_FOR_maskstorev16qihi = 7282,
  CODE_FOR_maskstorev32qisi = 7283,
  CODE_FOR_maskstorev32hisi = 7284,
  CODE_FOR_maskstorev16hihi = 7285,
  CODE_FOR_maskstorev8hiqi = 7286,
  CODE_FOR_cbranchv4si4 = 7287,
  CODE_FOR_cbranchv2di4 = 7288,
  CODE_FOR_cbranchv8si4 = 7289,
  CODE_FOR_cbranchv4di4 = 7290,
  CODE_FOR_vec_initv64qiqi = 7291,
  CODE_FOR_vec_initv32qiqi = 7292,
  CODE_FOR_vec_initv16qiqi = 7293,
  CODE_FOR_vec_initv32hihi = 7294,
  CODE_FOR_vec_initv16hihi = 7295,
  CODE_FOR_vec_initv8hihi = 7296,
  CODE_FOR_vec_initv16sisi = 7297,
  CODE_FOR_vec_initv8sisi = 7298,
  CODE_FOR_vec_initv4sisi = 7299,
  CODE_FOR_vec_initv8didi = 7300,
  CODE_FOR_vec_initv4didi = 7301,
  CODE_FOR_vec_initv2didi = 7302,
  CODE_FOR_vec_initv16sfsf = 7303,
  CODE_FOR_vec_initv8sfsf = 7304,
  CODE_FOR_vec_initv4sfsf = 7305,
  CODE_FOR_vec_initv8dfdf = 7306,
  CODE_FOR_vec_initv4dfdf = 7307,
  CODE_FOR_vec_initv2dfdf = 7308,
  CODE_FOR_vec_initv4titi = 7309,
  CODE_FOR_vec_initv2titi = 7310,
  CODE_FOR_vec_initv64qiv32qi = 7311,
  CODE_FOR_vec_initv32qiv16qi = 7312,
  CODE_FOR_vec_initv16qiv8qi = 7313,
  CODE_FOR_vec_initv32hiv16hi = 7314,
  CODE_FOR_vec_initv16hiv8hi = 7315,
  CODE_FOR_vec_initv8hiv4hi = 7316,
  CODE_FOR_vec_initv16siv8si = 7317,
  CODE_FOR_vec_initv8siv4si = 7318,
  CODE_FOR_vec_initv4siv2si = 7319,
  CODE_FOR_vec_initv8div4di = 7320,
  CODE_FOR_vec_initv4div2di = 7321,
  CODE_FOR_vec_initv16sfv8sf = 7322,
  CODE_FOR_vec_initv8sfv4sf = 7323,
  CODE_FOR_vec_initv4sfv2sf = 7324,
  CODE_FOR_vec_initv8dfv4df = 7325,
  CODE_FOR_vec_initv4dfv2df = 7326,
  CODE_FOR_vec_initv4tiv2ti = 7327,
  CODE_FOR_vcvtps2ph_mask = 7328,
  CODE_FOR_vcvtps2ph = 7329,
  CODE_FOR_avx2_gathersiv2di = 7330,
  CODE_FOR_avx2_gathersiv2df = 7331,
  CODE_FOR_avx2_gathersiv4di = 7332,
  CODE_FOR_avx2_gathersiv4df = 7333,
  CODE_FOR_avx2_gathersiv4si = 7334,
  CODE_FOR_avx2_gathersiv4sf = 7335,
  CODE_FOR_avx2_gathersiv8si = 7336,
  CODE_FOR_avx2_gathersiv8sf = 7337,
  CODE_FOR_avx2_gatherdiv2di = 7338,
  CODE_FOR_avx2_gatherdiv2df = 7339,
  CODE_FOR_avx2_gatherdiv4di = 7340,
  CODE_FOR_avx2_gatherdiv4df = 7341,
  CODE_FOR_avx2_gatherdiv4si = 7342,
  CODE_FOR_avx2_gatherdiv4sf = 7343,
  CODE_FOR_avx2_gatherdiv8si = 7344,
  CODE_FOR_avx2_gatherdiv8sf = 7345,
  CODE_FOR_avx512f_gathersiv16si = 7346,
  CODE_FOR_avx512f_gathersiv16sf = 7347,
  CODE_FOR_avx512f_gathersiv8di = 7348,
  CODE_FOR_avx512f_gathersiv8df = 7349,
  CODE_FOR_avx512vl_gathersiv8si = 7350,
  CODE_FOR_avx512vl_gathersiv8sf = 7351,
  CODE_FOR_avx512vl_gathersiv4di = 7352,
  CODE_FOR_avx512vl_gathersiv4df = 7353,
  CODE_FOR_avx512vl_gathersiv4si = 7354,
  CODE_FOR_avx512vl_gathersiv4sf = 7355,
  CODE_FOR_avx512vl_gathersiv2di = 7356,
  CODE_FOR_avx512vl_gathersiv2df = 7357,
  CODE_FOR_avx512f_gatherdiv16si = 7358,
  CODE_FOR_avx512f_gatherdiv16sf = 7359,
  CODE_FOR_avx512f_gatherdiv8di = 7360,
  CODE_FOR_avx512f_gatherdiv8df = 7361,
  CODE_FOR_avx512vl_gatherdiv8si = 7362,
  CODE_FOR_avx512vl_gatherdiv8sf = 7363,
  CODE_FOR_avx512vl_gatherdiv4di = 7364,
  CODE_FOR_avx512vl_gatherdiv4df = 7365,
  CODE_FOR_avx512vl_gatherdiv4si = 7366,
  CODE_FOR_avx512vl_gatherdiv4sf = 7367,
  CODE_FOR_avx512vl_gatherdiv2di = 7368,
  CODE_FOR_avx512vl_gatherdiv2df = 7369,
  CODE_FOR_avx512f_scattersiv16si = 7370,
  CODE_FOR_avx512f_scattersiv16sf = 7371,
  CODE_FOR_avx512f_scattersiv8di = 7372,
  CODE_FOR_avx512f_scattersiv8df = 7373,
  CODE_FOR_avx512vl_scattersiv8si = 7374,
  CODE_FOR_avx512vl_scattersiv8sf = 7375,
  CODE_FOR_avx512vl_scattersiv4di = 7376,
  CODE_FOR_avx512vl_scattersiv4df = 7377,
  CODE_FOR_avx512vl_scattersiv4si = 7378,
  CODE_FOR_avx512vl_scattersiv4sf = 7379,
  CODE_FOR_avx512vl_scattersiv2di = 7380,
  CODE_FOR_avx512vl_scattersiv2df = 7381,
  CODE_FOR_avx512f_scatterdiv16si = 7382,
  CODE_FOR_avx512f_scatterdiv16sf = 7383,
  CODE_FOR_avx512f_scatterdiv8di = 7384,
  CODE_FOR_avx512f_scatterdiv8df = 7385,
  CODE_FOR_avx512vl_scatterdiv8si = 7386,
  CODE_FOR_avx512vl_scatterdiv8sf = 7387,
  CODE_FOR_avx512vl_scatterdiv4di = 7388,
  CODE_FOR_avx512vl_scatterdiv4df = 7389,
  CODE_FOR_avx512vl_scatterdiv4si = 7390,
  CODE_FOR_avx512vl_scatterdiv4sf = 7391,
  CODE_FOR_avx512vl_scatterdiv2di = 7392,
  CODE_FOR_avx512vl_scatterdiv2df = 7393,
  CODE_FOR_avx512f_expandv16si_maskz = 7394,
  CODE_FOR_avx512f_expandv16sf_maskz = 7395,
  CODE_FOR_avx512f_expandv8di_maskz = 7396,
  CODE_FOR_avx512f_expandv8df_maskz = 7397,
  CODE_FOR_avx512vl_expandv8si_maskz = 7398,
  CODE_FOR_avx512vl_expandv8sf_maskz = 7399,
  CODE_FOR_avx512vl_expandv4di_maskz = 7400,
  CODE_FOR_avx512vl_expandv4df_maskz = 7401,
  CODE_FOR_avx512vl_expandv4si_maskz = 7402,
  CODE_FOR_avx512vl_expandv4sf_maskz = 7403,
  CODE_FOR_avx512vl_expandv2di_maskz = 7404,
  CODE_FOR_avx512vl_expandv2df_maskz = 7405,
  CODE_FOR_expandv64qi_maskz = 7406,
  CODE_FOR_expandv16qi_maskz = 7407,
  CODE_FOR_expandv32qi_maskz = 7408,
  CODE_FOR_expandv32hi_maskz = 7409,
  CODE_FOR_expandv16hi_maskz = 7410,
  CODE_FOR_expandv8hi_maskz = 7411,
  CODE_FOR_vpamdd52huqv8di_maskz = 7412,
  CODE_FOR_vpamdd52huqv4di_maskz = 7413,
  CODE_FOR_vpamdd52huqv2di_maskz = 7414,
  CODE_FOR_vpamdd52luqv8di_maskz = 7415,
  CODE_FOR_vpamdd52luqv4di_maskz = 7416,
  CODE_FOR_vpamdd52luqv2di_maskz = 7417,
  CODE_FOR_movv64sf = 7418,
  CODE_FOR_movv64si = 7419,
  CODE_FOR_vpshrdv_v32hi_maskz = 7420,
  CODE_FOR_vpshrdv_v16si_maskz = 7421,
  CODE_FOR_vpshrdv_v8di_maskz = 7422,
  CODE_FOR_vpshrdv_v16hi_maskz = 7423,
  CODE_FOR_vpshrdv_v8si_maskz = 7424,
  CODE_FOR_vpshrdv_v4di_maskz = 7425,
  CODE_FOR_vpshrdv_v8hi_maskz = 7426,
  CODE_FOR_vpshrdv_v4si_maskz = 7427,
  CODE_FOR_vpshrdv_v2di_maskz = 7428,
  CODE_FOR_vpshldv_v32hi_maskz = 7429,
  CODE_FOR_vpshldv_v16si_maskz = 7430,
  CODE_FOR_vpshldv_v8di_maskz = 7431,
  CODE_FOR_vpshldv_v16hi_maskz = 7432,
  CODE_FOR_vpshldv_v8si_maskz = 7433,
  CODE_FOR_vpshldv_v4di_maskz = 7434,
  CODE_FOR_vpshldv_v8hi_maskz = 7435,
  CODE_FOR_vpshldv_v4si_maskz = 7436,
  CODE_FOR_vpshldv_v2di_maskz = 7437,
  CODE_FOR_vpdpbusd_v16si_maskz = 7438,
  CODE_FOR_vpdpbusd_v8si_maskz = 7439,
  CODE_FOR_vpdpbusd_v4si_maskz = 7440,
  CODE_FOR_vpdpbusds_v16si_maskz = 7441,
  CODE_FOR_vpdpbusds_v8si_maskz = 7442,
  CODE_FOR_vpdpbusds_v4si_maskz = 7443,
  CODE_FOR_vpdpwssd_v16si_maskz = 7444,
  CODE_FOR_vpdpwssd_v8si_maskz = 7445,
  CODE_FOR_vpdpwssd_v4si_maskz = 7446,
  CODE_FOR_vpdpwssds_v16si_maskz = 7447,
  CODE_FOR_vpdpwssds_v8si_maskz = 7448,
  CODE_FOR_vpdpwssds_v4si_maskz = 7449,
  CODE_FOR_sse2_lfence = 7450,
  CODE_FOR_sse_sfence = 7451,
  CODE_FOR_sse2_mfence = 7452,
  CODE_FOR_mem_thread_fence = 7453,
  CODE_FOR_atomic_loadqi = 7454,
  CODE_FOR_atomic_loadhi = 7455,
  CODE_FOR_atomic_loadsi = 7456,
  CODE_FOR_atomic_loaddi = 7457,
  CODE_FOR_atomic_storeqi = 7458,
  CODE_FOR_atomic_storehi = 7459,
  CODE_FOR_atomic_storesi = 7460,
  CODE_FOR_atomic_storedi = 7461,
  CODE_FOR_atomic_compare_and_swapqi = 7462,
  CODE_FOR_atomic_compare_and_swaphi = 7463,
  CODE_FOR_atomic_compare_and_swapsi = 7464,
  CODE_FOR_atomic_compare_and_swapdi = 7465,
  CODE_FOR_atomic_compare_and_swapti = 7466,
  CODE_FOR_atomic_bit_test_and_sethi = 7467,
  CODE_FOR_atomic_bit_test_and_setsi = 7468,
  CODE_FOR_atomic_bit_test_and_setdi = 7469,
  CODE_FOR_atomic_bit_test_and_complementhi = 7470,
  CODE_FOR_atomic_bit_test_and_complementsi = 7471,
  CODE_FOR_atomic_bit_test_and_complementdi = 7472,
  CODE_FOR_atomic_bit_test_and_resethi = 7473,
  CODE_FOR_atomic_bit_test_and_resetsi = 7474,
  CODE_FOR_atomic_bit_test_and_resetdi = 7475
};

const unsigned int NUM_INSN_CODES = 7476;
#endif /* GCC_INSN_CODES_H */
